@@ -10,11 +10,26 @@
 //			
 // project :      TANGO Device Server
 //
-// $Author: pascal_verdier $
+// $Author: taurel $
 //
-// $Revision: 2.29 $
+// $Revision: 2.31 $
 //
 // $Log: DataBaseClass.h,v $
+// Revision 2.31  2008/03/26 17:31:00  taurel
+// - Committed after merge with the Release 4 branch
+//
+// Revision 2.30.2.2  2008/02/28 17:07:37  taurel
+// - Added the StoredProcedureRelease attribute
+//
+// Revision 2.30.2.1  2008/01/25 12:29:50  taurel
+// - Add a new command DbDeleteAllDeviceAttributeProperty
+// - The DbImportDevice now also returns the device class
+// - Fix bug in the DbGetHostServerList command
+//
+// Revision 2.30  2007/11/06 07:24:07  taurel
+// - Added the DbGetDataForServerCache command (with timing stats)
+// - Add timing stats for the DbPutClassProperty command
+//
 // Revision 2.29  2006/09/28 11:18:20  pascal_verdier
 // DbGetClassForDevice and DbGetClassInheritanceForDevice commands added.
 //
@@ -194,9 +209,69 @@ public:
 	{return (static_cast<DataBase *>(dev))->is_Timing_average_allowed(ty);}
 };
 
+class StoredProcedureReleaseAttrib: public Tango::Attr
+{
+public:
+	StoredProcedureReleaseAttrib():Attr("StoredProcedureRelease", Tango::DEV_STRING, Tango::READ) {};
+	~StoredProcedureReleaseAttrib() {};
+	
+	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
+	{(static_cast<DataBase *>(dev))->read_StoredProcedureRelease(att);}
+	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
+	{return (static_cast<DataBase *>(dev))->is_StoredProcedureRelease_allowed(ty);}
+};
+
 //=========================================
 //	Define classes for commands
 //=========================================
+class DbDeleteAllDeviceAttributePropertyClass : public Tango::Command
+{
+public:
+	DbDeleteAllDeviceAttributePropertyClass(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out,
+				   const char        *in_desc,
+				   const char        *out_desc,
+				   Tango::DispLevel  level)
+	:Command(name,in,out,in_desc,out_desc, level)	{};
+
+	DbDeleteAllDeviceAttributePropertyClass(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out)
+	:Command(name,in,out)	{};
+	~DbDeleteAllDeviceAttributePropertyClass() {};
+	
+	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
+	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
+	{return (static_cast<DataBase *>(dev))->is_DbDeleteAllDeviceAttributeProperty_allowed(any);}
+};
+
+
+
+class DbGetDataForServerCacheCmd : public Tango::Command
+{
+public:
+	DbGetDataForServerCacheCmd(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out,
+				   const char        *in_desc,
+				   const char        *out_desc,
+				   Tango::DispLevel  level)
+	:Command(name,in,out,in_desc,out_desc, level)	{};
+
+	DbGetDataForServerCacheCmd(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out)
+	:Command(name,in,out)	{};
+	~DbGetDataForServerCacheCmd() {};
+	
+	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
+	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
+	{return (static_cast<DataBase *>(dev))->is_DbGetDataForServerCache_allowed(any);}
+};
+
+
+
 class ResetTimingValuesClass : public Tango::Command
 {
 public:
@@ -2001,7 +2076,11 @@ public:
 // The DataBaseClass singleton definition
 //
 
-class DataBaseClass : public Tango::DeviceClass
+class
+#ifdef WIN32
+	__declspec(dllexport)
+#endif
+	DataBaseClass : public Tango::DeviceClass
 {
 public:
 //	properties member data
