@@ -1,4 +1,4 @@
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/DataBaseUtils.cpp,v 2.5 2006/11/06 15:31:03 jlpons Exp $";
+static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/DataBaseUtils.cpp,v 2.7 2008/03/26 17:31:01 taurel Exp $";
 //+=============================================================================
 //
 // file :         DataBaseUtils.cpp
@@ -309,7 +309,8 @@ void DataBase::init_timing_stats()
 {
 	TimingStatsStruct new_timing_stats = {0.0, 0.0, 0.0, 0.0, 0.0};
 
-
+	timing_stats_mutex.lock();
+	
 	timing_stats_map["DbImportDevice"] = new TimingStatsStruct;
 	timing_stats_map["DbExportDevice"] = new TimingStatsStruct;
 	timing_stats_map["DbGetHostServerList"] = new TimingStatsStruct;
@@ -333,6 +334,8 @@ void DataBase::init_timing_stats()
 	timing_stats_map["DbGetDeviceExportedList"] = new TimingStatsStruct;
 	timing_stats_map["DbExportEvent"] = new TimingStatsStruct;
 	timing_stats_map["DbImportEvent"] = new TimingStatsStruct;
+	timing_stats_map["DbGetDataForServerCache"] = new TimingStatsStruct;
+	timing_stats_map["DbPutClassProperty"] = new TimingStatsStruct;
 
 	timing_stats_size = timing_stats_map.size();
 	timing_stats_average = new double[timing_stats_size];
@@ -350,9 +353,10 @@ void DataBase::init_timing_stats()
 		*(iter->second) = new_timing_stats;
 		timing_stats_index[i] = (char*)malloc(strlen(iter->first.c_str())+1);
 		strcpy(timing_stats_index[i],iter->first.c_str());
-//			cout << "init_timing_stats(): index[" << i << "] = " << timing_stats_index[i] << endl;
 		i++;
 	}
+	
+	timing_stats_mutex.unlock();
 }
 
 //+----------------------------------------------------------------------------
@@ -369,11 +373,11 @@ void DataBase::check_history_tables()
 
 	INFO_STREAM << "DataBase::check_history_tables(): entering" << endl;
 
-        sql_query_stream.str("");
-  	    sql_query_stream << "SELECT count(*) FROM property_device_hist";
-        DEBUG_STREAM << "DataBase::check_history_tables(): sql_query " << sql_query_stream.str() << endl;
-        result = query(sql_query_stream.str(),"check_history_tables()");
-        mysql_free_result(result);
+	sql_query_stream.str("");
+	sql_query_stream << "SELECT count(*) FROM property_device_hist";
+	DEBUG_STREAM << "DataBase::check_history_tables(): sql_query " << sql_query_stream.str() << endl;
+	result = query(sql_query_stream.str(),"check_history_tables()");
+	mysql_free_result(result);
 }
 
 //+----------------------------------------------------------------------------
@@ -381,39 +385,66 @@ void DataBase::check_history_tables()
 // method : 		DataBase::get_id()
 // 
 // description : 	Return history id
+//					In this method, we don't use the classical query()
+//					method in order to be sure that the UPDATE and the following
+//					mysql_insert_id() are done using the same MySQL connection
 //
 //-----------------------------------------------------------------------------
-unsigned int DataBase::get_id(const char *name) {
 
+unsigned int DataBase::get_id(const char *name,int con_nb) 
+{
 	TangoSys_MemStream sql_query;
-	TangoSys_OMemStream o;
-  	MYSQL_RES *result;
-	MYSQL_ROW row;
 
-    // Get id
+//
+// If no MySQL connection passed to this method,
+// get one
+//
+
+	bool need_release = false;
+	
+	if (con_nb == -1)
+	{
+		con_nb = get_connection();
+		need_release = true;
+	}
+
     sql_query.str("");
-    sql_query << "SELECT id FROM history_ids WHERE name='" << name << "'";
-	result = query(sql_query.str(),"get_id()");
-	if(mysql_num_rows(result)==0) {
-       mysql_free_result(result);
-	   o << "Failed to get history id : " << name;
-	   Tango::Except::throw_exception((const char *)DB_SQLError,o.str(),"DataBase::get_id()");	
-	}
-	row = mysql_fetch_row(result);
-    unsigned int ret = atoi(row[0]);
-    mysql_free_result(result);
+    sql_query << "UPDATE " << name << "_history_id SET id=LAST_INSERT_ID(id+1)";
+	string tmp_str = sql_query.str();
 
-    // Increment id
-	sql_query.str("");
-	if(ret==4294967295) {
-      sql_query << "UPDATE history_ids SET id = 0 WHERE name='" << name << "'";
-	  simple_query(sql_query.str(),"get_id()");	
-	} else {
-      sql_query << "UPDATE history_ids SET id = id + 1 WHERE name='" << name << "'";
-	  simple_query(sql_query.str(),"get_id()");
+	if (mysql_real_query(conn_pool[con_nb].db, tmp_str.c_str(),tmp_str.length()) != 0)
+	{
+		TangoSys_OMemStream o;
+		  
+		WARN_STREAM << "DataBase::get_id() failed to query TANGO database:" << endl;
+		WARN_STREAM << "  query = " << tmp_str << endl;
+		WARN_STREAM << " (SQL error=" << mysql_error(conn_pool[con_nb].db) << ")" << endl;
+		
+		o << "Failed to query TANGO database (error=" << mysql_error(conn_pool[con_nb].db) << ")" << ends;
+
+		if (need_release == true)
+			release_connection(con_nb);
+			
+		Tango::Except::throw_exception((const char *)DB_SQLError,o.str(),(const char *)"DataBase::get_id()");
 	}
 
-	return ret;			
+	my_ulonglong val = mysql_insert_id(conn_pool[con_nb].db);
+	if (val == 0)
+	{
+		TangoSys_OMemStream o;
+
+		o << "Failed to get history id : " << name;
+		
+		if (need_release == true)
+			release_connection(con_nb);
+			
+		Tango::Except::throw_exception((const char *)DB_SQLError,o.str(),(const char *)"DataBase::get_id()");	
+	}
+
+	if (need_release == true)
+		release_connection(con_nb);
+
+	return val;
 }
 
 //+----------------------------------------------------------------------------
@@ -423,21 +454,50 @@ unsigned int DataBase::get_id(const char *name) {
 // description : 	Execute a SQL query , ignore the result.
 //
 //-----------------------------------------------------------------------------
-void DataBase::simple_query(string sql_query,const char *method) {
+void DataBase::simple_query(string sql_query,const char *method,int con_nb)
+{
 	
-	TangoSys_OMemStream o;
-	TangoSys_OMemStream o2;
+//
+// If no MySQL connection passed to this method,
+// get one
+//
+
+	bool need_release = false;
 	
-	if (mysql_real_query(&mysql, sql_query.c_str(),sql_query.length()) != 0)
+	if (con_nb == -1)
 	{
-	   WARN_STREAM << "DataBase::" << method << " failed to query TANGO database:" << endl;
-	   WARN_STREAM << "  query = " << sql_query << endl;
-	   WARN_STREAM << " (SQL error=" << mysql_error(&mysql) << ")" << endl;
-	   o << "Failed to query TANGO database (error=" << mysql_error(&mysql) << ")";
-	   o2 << "DataBase::" << method;
-	   Tango::Except::throw_exception((const char *)DB_SQLError,o.str(),o2.str());
+		con_nb = get_connection();
+		need_release = true;
 	}
 
+	DEBUG_STREAM << "Using MySQL connection with semaphore " << con_nb << endl;	
+
+//
+// Call MySQL
+//
+
+	if (mysql_real_query(conn_pool[con_nb].db, sql_query.c_str(),sql_query.length()) != 0)
+	{
+		TangoSys_OMemStream o;
+		TangoSys_OMemStream o2;	 
+		  
+		WARN_STREAM << "DataBase::" << method << " failed to query TANGO database:" << endl;
+		WARN_STREAM << "  query = " << sql_query << endl;
+		WARN_STREAM << " (SQL error=" << mysql_error(conn_pool[con_nb].db) << ")" << endl;
+		
+		o << "Failed to query TANGO database (error=" << mysql_error(conn_pool[con_nb].db) << ")";
+		o << "\n.The query was: " << sql_query << ends;
+		o2 << "DataBase::" << method << ends;
+
+		if (need_release == true)
+			release_connection(con_nb);
+				
+		Tango::Except::throw_exception((const char *)DB_SQLError,o.str(),o2.str());
+	}
+
+	if (need_release)
+		release_connection(con_nb);
+
 }
 
 //+----------------------------------------------------------------------------
@@ -447,34 +507,110 @@ void DataBase::simple_query(string sql_query,const char *method) {
 // description : 	Execute a SQL query and return the result.
 //
 //-----------------------------------------------------------------------------
-MYSQL_RES *DataBase::query(string sql_query,const char *method) {
-
-	TangoSys_OMemStream o;
-	TangoSys_OMemStream o2;
+MYSQL_RES *DataBase::query(string sql_query,const char *method,int con_nb)
+{
 	MYSQL_RES *result;
+
+//
+// If no MySQL connection passed to this method,
+// get one
+//
+
+	bool need_release = false;
 	
-	if (mysql_real_query(&mysql, sql_query.c_str(),sql_query.length()) != 0)
+	if (con_nb == -1)
 	{
-	   WARN_STREAM << "DataBase::" << method << " failed to query TANGO database:" << endl;
-	   WARN_STREAM << "  query = " << sql_query << endl;
-	   WARN_STREAM << " (SQL error=" << mysql_error(&mysql) << ")" << endl;
-	   o << "Failed to query TANGO database (error=" << mysql_error(&mysql) << ")";
-	   o2 << "DataBase::" << method;
-	   Tango::Except::throw_exception((const char *)DB_SQLError,o.str(),o2.str());
+		con_nb = get_connection();
+		need_release = true;
 	}
 
-	if ((result = mysql_store_result(&mysql)) == NULL)
+	DEBUG_STREAM << "Using MySQL connection with semaphore " << con_nb << endl;
+		
+//
+// Call MySQL
+//
+		
+	if (mysql_real_query(conn_pool[con_nb].db, sql_query.c_str(),sql_query.length()) != 0)
 	{
-	   WARN_STREAM << "DataBase:: " << method << " : mysql_store_result() failed  (error=" << mysql_error(&mysql) << ")" << endl;
-	   o << "mysql_store_result() failed (error=" << mysql_error(&mysql) << ")";
-	   o2 << "DataBase::" << method;	   
-	   Tango::Except::throw_exception((const char *)DB_SQLError,o.str(),o2.str());
-	}
+		TangoSys_OMemStream o;
+		TangoSys_OMemStream o2;
 	
+		WARN_STREAM << "DataBase::" << method << " failed to query TANGO database:" << endl;
+		WARN_STREAM << "  query = " << sql_query << endl;
+		WARN_STREAM << " (SQL error=" << mysql_error(conn_pool[con_nb].db) << ")" << endl;
+		
+		o << "Failed to query TANGO database (error=" << mysql_error(conn_pool[con_nb].db) << ")";
+		o << "\nThe query was: " << sql_query << ends;
+		o2 << "DataBase::" << method << ends;
+
+		if (need_release == true)
+			release_connection(con_nb);
+		
+		Tango::Except::throw_exception((const char *)DB_SQLError,o.str(),o2.str());
+	}
+
+	if ((result = mysql_store_result(conn_pool[con_nb].db)) == NULL)
+	{
+		TangoSys_OMemStream o;
+		TangoSys_OMemStream o2;
+		
+		WARN_STREAM << "DataBase:: " << method << " : mysql_store_result() failed  (error=" << mysql_error(conn_pool[con_nb].db) << ")" << endl;
+	   
+		o << "mysql_store_result() failed (error=" << mysql_error(conn_pool[con_nb].db) << ")";
+		o2 << "DataBase::" << method;	   
+
+		if (need_release == true)
+			release_connection(con_nb);
+		
+		Tango::Except::throw_exception((const char *)DB_SQLError,o.str(),o2.str());
+	}
+
+	if (need_release)
+		release_connection(con_nb);
+		
 	return result;
 
 }
 
+//+----------------------------------------------------------------------------
+//
+// method : 		DataBase::get_connection()
+// 
+// description : 	Get a MySQl connection from the connection pool
+//
+//-----------------------------------------------------------------------------
+int DataBase::get_connection()
+{
+	
+//
+// Get a MySQL connection and lock it
+// If none available, wait for one
+//
+
+	int loop = 0;
+	while (conn_pool[loop].the_sema.trywait() == 0)
+	{
+		loop++;
+		if (loop == conn_pool_size)
+		{
+			int sem_to_wait;
+			{
+				omni_mutex_lock oml(sem_wait_mutex);
+				sem_to_wait = last_sem_wait++;
+				if (last_sem_wait == conn_pool_size)
+					last_sem_wait = 0;
+			}
+			loop = sem_to_wait;
+			WARN_STREAM << "Waiting for one free MySQL connection on semaphore " << loop << endl;
+			conn_pool[loop].the_sema.wait();
+			break;
+		}
+	}
+	
+	return loop;
+}
+
+
 //+------------------------------------------------------------------
 /**
  *	method:	purge_property()
@@ -483,7 +619,7 @@ MYSQL_RES *DataBase::query(string sql_query,const char *method) {
  *
  */
 //+------------------------------------------------------------------
-void DataBase::purge_property(const char *table,const char *field,const char *object,const char *name) {
+void DataBase::purge_property(const char *table,const char *field,const char *object,const char *name,int con_nb) {
 
   TangoSys_MemStream sql_query;
   MYSQL_RES *result;
@@ -498,7 +634,7 @@ void DataBase::purge_property(const char *table,const char *field,const char *ob
             << " WHERE " << field << "=\"" << object << "\" AND name=\"" << name 
             << "\" ORDER by date";
 	    
-  result = query(sql_query.str(),"purge_property()");
+  result = query(sql_query.str(),"purge_property()",con_nb);
   int nb_item = mysql_num_rows(result);
 
   GetTime(after);
@@ -514,7 +650,7 @@ void DataBase::purge_property(const char *table,const char *field,const char *ob
         row2 = mysql_fetch_row(result);
         sql_query.str("");
         sql_query << "DELETE FROM " << table << " WHERE id='" << row2[0] << "'";
-	simple_query(sql_query.str(),"purge_property()");
+		simple_query(sql_query.str(),"purge_property()",con_nb);
     }
   }
   
@@ -534,7 +670,7 @@ void DataBase::purge_property(const char *table,const char *field,const char *ob
  *
  */
 //+------------------------------------------------------------------
-void DataBase::purge_att_property(const char *table,const char *field,const char *object,const char *attribute,const char *name) {
+void DataBase::purge_att_property(const char *table,const char *field,const char *object,const char *attribute,const char *name,int con_nb) {
 
   TangoSys_MemStream sql_query;
   MYSQL_RES *result;
@@ -551,7 +687,7 @@ void DataBase::purge_att_property(const char *table,const char *field,const char
             << " WHERE " << field << "=\"" << object << "\" AND name=\"" << name 
             << "\" AND attribute=\"" << attribute << "\" ORDER by date";
 
-  result = query(sql_query.str(),"purge_att_property()");
+  result = query(sql_query.str(),"purge_att_property()",con_nb);
   int nb_item = mysql_num_rows(result);
 
   GetTime(after);
@@ -564,10 +700,10 @@ void DataBase::purge_att_property(const char *table,const char *field,const char
     // Purge 
     int toDelete = nb_item-historyDepth;
     for(j=0;j<toDelete;j++) {
-	row2 = mysql_fetch_row(result);
+		row2 = mysql_fetch_row(result);
         sql_query.str("");
         sql_query << "DELETE FROM " << table << " WHERE id='" << row2[0] << "'";
-	simple_query(sql_query.str(),"purge_att_property()");
+		simple_query(sql_query.str(),"purge_att_property()",con_nb);
     }
   }
   mysql_free_result(result);
@@ -578,4 +714,98 @@ void DataBase::purge_att_property(const char *table,const char *field,const char
       
 }
 
+//+------------------------------------------------------------------
+/**
+ *	method:	create_connection_pool()
+ *
+ *	description:	Create the MySQL connections pool
+ *
+ */
+//+------------------------------------------------------------------
+
+void DataBase::create_connection_pool(char *mysql_user,char *mysql_password)
+{
+	char *database = (char *)"tango";
+	
+	WARN_STREAM << "DataBase::create_connection_pool(): mysql database user =  " << mysql_user 
+	            << " , password = " << mysql_password << endl;
+					
+	for (int loop = 0;loop < conn_pool_size;loop++)
+	{
+	
+// Initialise mysql database structure and connect to TANGO database
+
+		conn_pool[loop].db = mysql_init(conn_pool[loop].db);
+		mysql_options(conn_pool[loop].db,MYSQL_READ_DEFAULT_GROUP,"client");
+
+
+#if   (MYSQL_VERSION_ID > 50000)
+
+	//C. Scafuri: auto reconnection has been off since 5.0.3. From 5.0.13 it is possible to set it as an option
+	// with reconnection enabled DataBase keeps working after timeouts and mysql shutdown/restart
+		if(mysql_get_client_version() >= 50013)
+		{
+			my_bool my_auto_reconnect=1;
+			if (mysql_options(conn_pool[loop].db,MYSQL_OPT_RECONNECT,&my_auto_reconnect) !=0)
+				ERROR_STREAM << "DataBase: error setting mysql auto reconnection: " << mysql_error(conn_pool[loop].db) << endl;
+			else
+				WARN_STREAM << "DataBase: set mysql auto reconnect to true" << endl;
+		}
+#endif
+
+
+		if (!mysql_real_connect(conn_pool[loop].db, NULL, mysql_user, mysql_password, database, 0, NULL, CLIENT_MULTI_STATEMENTS))
+		{
+			TangoSys_MemStream out_stream;
+			out_stream << "Failed to connect to TANGO database (error = " << mysql_error(conn_pool[loop].db) << ")" << ends;
+			
+			Tango::Except::throw_exception((const char *)"CANNOT_CONNECT_MYSQL",
+											out_stream.str(),
+											(const char *)"DataBase::init_device()");
+		}
+	}
+
+	mysql_svr_version = mysql_get_server_version(conn_pool[0].db);
+	last_sem_wait = 0;
+	
+}
+
+//+------------------------------------------------------------------
+/**
+ *	method:	AutoLock class ctor and dtor
+ *
+ *	description:	AutoLock is a small helper class which get a
+ *					MySQL connection from the pool and which lock
+ *					table(s). The exact lock statemen is passed to the
+ *					ctor as a parameter
+ *					The dtor release the table(s) lock
+ *
+ */
+//+------------------------------------------------------------------
+
+AutoLock::AutoLock(const char *lock_cmd,DataBase *db):the_db(db)
+{
+	con_nb = the_db->get_connection();
+	TangoSys_MemStream sql_query_stream;
+	
+	sql_query_stream << lock_cmd;
+	try
+	{
+		the_db->simple_query(sql_query_stream.str(),"AutoLock",con_nb);
+	}
+	catch (...)
+	{
+		the_db->release_connection(con_nb);
+		throw;
+	}
+}
+
+AutoLock::~AutoLock()
+{
+	TangoSys_MemStream sql_query_stream;
+	sql_query_stream << "UNLOCK TABLES";
+	the_db->simple_query(sql_query_stream.str(),"~AutoLock",con_nb);
+	the_db->release_connection(con_nb);
+}
+
 }
