@@ -1,4 +1,4 @@
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/StartProcessThread.cpp,v 3.9 2007/03/23 09:23:44 pascal_verdier Exp $";
+static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/StartProcessThread.cpp,v 3.15 2008/04/04 14:30:51 jensmeyer Exp $";
 //+=============================================================================
 //
 // file :         Starter.cpp
@@ -7,11 +7,30 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 //
 // project :      TANGO Device Server
 //
-// $Author: pascal_verdier $
+// $Author: jensmeyer $
 //
-// $Revision: 3.9 $
+// $Revision: 3.15 $
 //
 // $Log: StartProcessThread.cpp,v $
+// Revision 3.15  2008/04/04 14:30:51  jensmeyer
+// Added compile option for FreeBSD and MacOSX.
+// They have a different signal handling.
+//
+// Revision 3.14  2008/02/29 15:15:05  pascal_verdier
+// Checking running processes by system call added.
+//
+// Revision 3.13  2007/09/28 06:12:30  pascal_verdier
+// Remove traces
+//
+// Revision 3.12  2007/09/27 15:11:15  pascal_verdier
+// GetLastError() management added on Win 32
+//
+// Revision 3.11  2007/09/25 13:06:17  pascal_verdier
+// In win32 char* replaced by LPWSTR :-)
+//
+// Revision 3.10  2007/06/20 06:48:21  pascal_verdier
+// delete replaced by  delete[].
+//
 // Revision 3.9  2007/03/23 09:23:44  pascal_verdier
 // Bug in windows batch file startup fixed.
 //
@@ -74,6 +93,7 @@ StartProcessThread::StartProcessThread(vector<NewProcess *> v_np, int level, Sta
 		processes.push_back(v_np[i]);
 	thread_level = level;
 	starter      = st;
+
 }
 //+------------------------------------------------------------------
 /**
@@ -83,6 +103,7 @@ StartProcessThread::StartProcessThread(vector<NewProcess *> v_np, int level, Sta
 void *StartProcessThread::run_undetached(void *ptr)
 {
 	unsigned int	i;
+	CheckProcessUtil	*process_util = starter->util->proc_util;
 	//	Check if a previous level thread is terminated
 	if (thread_level>1)
 	{
@@ -102,36 +123,62 @@ void *StartProcessThread::run_undetached(void *ptr)
 
 		start_process(processes[i]);
 
+		//	Build server name from admin name
+		char	*p = processes[i]->adminname + strlen("dserver/");
+		string	servname(p);
+		bool	failed = false;
+
 		//	Sleep a bit between startups			 		
 		//	to do not overload database
 		bool	started = false;
 		time_t	t0 = time(NULL);
 		time_t	t1 = t0;
 		Tango::DeviceProxy	*serv;
-		while (started==false && (t1-t0)<starter->serverStartupTimeout)
+		while (failed==false && started==false && (t1-t0)<starter->serverStartupTimeout)
 		{
 #		ifdef WIN32
 			_sleep(1000);
 #		else
 			sleep(1);
 #		endif
-			//	Check if server has been started
-			serv = NULL;
-			try
-			{
-				serv = new Tango::DeviceProxy(processes[i]->adminname);
-				serv->ping();
-				started = true;
-			}
-			catch(Tango::DevFailed &)
-			{
-				//cout << e.errors[0].desc << endl;
-			}
-			if (serv!=NULL)
-				delete serv;
+
+
+			//	do not test before 4 seconds to be sure
+			//	process list has been updated.
 			t1 = time(NULL);
+			if ((t1-t0)>=4)
+			{
+				//	Check if server running or failed
+				if (process_util->is_server_running(servname)==false)
+					failed = true;
+				else
+				{
+					//	if  not failed check if startup is terminated
+					serv = NULL;
+					try
+					{
+						serv = new Tango::DeviceProxy(processes[i]->adminname);
+						serv->ping();
+						started = true;
+					}
+					catch(Tango::DevFailed &)
+					{
+						//cout << e.errors[0].desc << endl;
+					}
+					if (serv!=NULL)
+						delete serv;
+					t1 = time(NULL);
+				}
+			}
 		}
-		
+		if (started)
+			cout << "----- " << servname << " started in " << (t1-t0) << " sec." << endl;
+		else
+		if (failed)
+			cout << "----- " << servname << " failed in " << (t1-t0) << " sec." << endl;
+		else
+			cout << "----- " << servname << " Timeout = " << (t1-t0) << endl;
+
 		ms_sleep(TIME_BETWEEN_STARTUPS);
 	}
 
@@ -149,8 +196,8 @@ void *StartProcessThread::run_undetached(void *ptr)
 	{
 		free(processes[i]->servname);
 		free(processes[i]->instancename);
-		delete processes[i]->adminname;
-		delete processes[i]->logfile;
+		delete [] processes[i]->adminname;
+		delete [] processes[i]->logfile;
 	}
 
 //	starter->starting_processes--;
@@ -174,7 +221,8 @@ void StartProcessThread::start_process(NewProcess *process)
 	argv[i++] = NULL;
 
 	//	Fork a child process to start the device server
-	switch(fork())
+	int	fork_id = fork();
+	switch(fork_id)
 	{
 	case -1:
 		cerr << "Fork Failed !" << endl;
@@ -188,11 +236,15 @@ void StartProcessThread::start_process(NewProcess *process)
 		case 0:
 			{
 				// Change process group and close control tty
+#if (defined __darwin__ || defined __freebsd__)
+				setpgrp(0,setsid());
+#else										
 				setpgrp();
 
 				//	Call setsid() to do NOT stop children if Starter is killed.
 				//--------------------------------------------------------------
 				setsid();
+#endif
 
 				//	Close the stderr and re-open it on a log file.
 				//-------------------------------------------------
@@ -213,6 +265,7 @@ void StartProcessThread::start_process(NewProcess *process)
 			}
 			break;
 		default:
+			cout << fork_id << " exit()" << endl;
 			_exit(0);
 			break;
 		}
@@ -246,6 +299,7 @@ void *StartWinThread::run_undetached(void *ptr)
 {
 	//	Check if batch file
 	string	str_server(process->servname);
+	
 	if (str_server.find(".bat") != string::npos)
 	{
 		char	*argv[3];
@@ -258,9 +312,9 @@ void *StartWinThread::run_undetached(void *ptr)
 		cout << "log file: " << process->logfile << endl;
 
 		//	Write a starting message in log file
-		ofstream	ofs1(process->logfile);
-		ofs1 << "Starting " << argv[0] << "....." << endl << endl;
-		ofs1.close();
+		ofstream	ofs(process->logfile);
+		ofs << "Starting " << argv[0] << "....." << endl << endl;
+		ofs.close();
 
 		//	Close the stderr and re-open it on a log file.
 		//-------------------------------------------------
@@ -269,11 +323,28 @@ void *StartWinThread::run_undetached(void *ptr)
 		{
 			//	Write error in log file
 			//-------------------------------
-			ofstream	ofs(process->logfile);
-			ofs << "Exec(" << argv[0] << ") failed " << endl;
-			ofs << strerror(errno) << endl;
-			ofs << "_spawnv has returned " << ret << endl;
-			ofs.close();
+			ofstream	ofs1(process->logfile);
+			ofs1 << "Exec(" << argv[0] << ") failed " << endl;
+			ofs1 << "_spawnv has returned " << ret << endl;
+			DWORD	errcode = GetLastError();
+		    LPTSTR lp_err;
+			if (::FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_ARGUMENT_ARRAY,
+							NULL,
+							errcode,
+							LANG_NEUTRAL,
+							(LPTSTR)&lp_err,
+							0,
+							NULL))
+			{
+				char	str[0x80];
+				char	*p = str;
+				for (int i=0 ; lp_err[i]!=0 ; i++)
+					*p++ = lp_err[i];
+				*p = '\0';
+				ofs1 << str << " errcode = " << errcode << endl;
+				::LocalFree(lp_err);
+			}
+			ofs1.close();
 		}
 		else
 		{
@@ -298,7 +369,11 @@ void *StartWinThread::run_undetached(void *ptr)
 		sa.nLength = sizeof(sa);
 		sa.lpSecurityDescriptor = NULL;
 
-		HANDLE hStdError  = CreateFile( process->logfile, 
+		int buffSize = (int)strlen(process->logfile) + 1;
+		LPWSTR lpw_logfile = new wchar_t[buffSize];
+		MultiByteToWideChar(CP_ACP, 0, process->logfile, buffSize, lpw_logfile, buffSize);
+
+		HANDLE hStdError  = CreateFile( lpw_logfile, 
 			GENERIC_WRITE,
 			FILE_SHARE_WRITE | FILE_SHARE_READ, 
 			&sa, //lpSecurityAttributes 
@@ -314,9 +389,15 @@ void *StartWinThread::run_undetached(void *ptr)
 		si.dwFlags     = STARTF_USESHOWWINDOW;
 		si.wShowWindow = SW_HIDE;
 
-		if( !CreateProcess(
+		buffSize = (int)strlen(cmd_line.c_str()) + 1;
+		LPWSTR lpw_cmd = new wchar_t[buffSize];
+		MultiByteToWideChar(CP_ACP, 0, cmd_line.c_str(), buffSize, lpw_cmd, buffSize);
+
+		ofstream	ofs(process->logfile);
+		ofs << "Creating process " << cmd_line << endl;
+		if(CreateProcess(
 				NULL ,            // pointer to name of executable module
-				(char*)cmd_line.c_str(), // pointer to command line string
+				lpw_cmd,          // pointer to command line string
 				&sa,              // process security attributes
 				&sa,              // thread security attributes
 				TRUE,             // handle inheritance flag
@@ -325,20 +406,40 @@ void *StartWinThread::run_undetached(void *ptr)
 				NULL,             // pointer to current directory name
 				&si,              // pointer to STARTUPINFO
 				&pi               // pointer to PROCESS_INFORMATION
-				) )
+				)==0)
 		{
 			_sleep(2000);
-			cerr << "Error when trying to create process " << cmd_line <<endl << strerror(errno) << endl;
+			cerr << "Error when trying to create process " << cmd_line << endl;
 
 			//	Write a starting message in log file
 			ofstream	ofs1(process->logfile);
-			ofs1 << "Error when trying to create process " << cmd_line <<endl << strerror(errno) << endl;
-			ofs1.close();
+			ofs << "Error when trying to create process " << cmd_line <<endl;
+			DWORD	errcode = GetLastError();
+		    LPTSTR lp_err;
+			if (::FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_ARGUMENT_ARRAY,
+							NULL,
+							errcode,
+							LANG_NEUTRAL,
+							(LPTSTR)&lp_err,
+							0,
+							NULL))
+			{
+				char	str[0x80];
+				char	*p = str;
+				for (int i=0 ; lp_err[i]!=0 ; i++)
+					*p++ = lp_err[i];
+				*p = '\0';
+				ofs << str << " errcode = " << errcode << endl;
+				::LocalFree(lp_err);
+			}
 		}
 		else
 		{
-			cout << "...OK (" << cmd_line << ")" << endl;
+			ofs << ".......OK (" << cmd_line << ")" << endl;
 		}
+		delete lpw_cmd;
+		delete lpw_logfile;
+		ofs.close();
 	}
 	return NULL;
 }
