@@ -12,9 +12,12 @@
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 3.10 $
+// $Revision: 3.11 $
 //
 // $Log: StarterClass.h,v $
+// Revision 3.11  2008/02/29 15:15:05  pascal_verdier
+// Checking running processes by system call added.
+//
 // Revision 3.10  2006/06/05 07:20:36  pascal_verdier
 // Server startup is now delayed (with timeout) in a startup level.
 // New state MOVING added.
@@ -105,6 +108,18 @@ namespace Starter_ns
 //=====================================
 //	Define classes for attributes
 //=====================================
+class ServersAttrib: public Tango::SpectrumAttr
+{
+public:
+	ServersAttrib():SpectrumAttr("Servers", Tango::DEV_STRING, Tango::READ, 1024) {};
+	~ServersAttrib() {};
+	
+	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
+	{(static_cast<Starter *>(dev))->read_Servers(att);}
+	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
+	{return (static_cast<Starter *>(dev))->is_Servers_allowed(ty);}
+};
+
 class StoppedServersAttrib: public Tango::SpectrumAttr
 {
 public:
@@ -141,9 +156,45 @@ public:
 	{return (static_cast<Starter *>(dev))->is_HostState_allowed(ty);}
 };
 
+class NotifdStateAttrib: public Tango::Attr
+{
+public:
+	NotifdStateAttrib():Attr("NotifdState", Tango::DEV_STATE, Tango::READ) {};
+	~NotifdStateAttrib() {};
+	
+	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
+	{(static_cast<Starter *>(dev))->read_NotifdState(att);}
+	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
+	{return (static_cast<Starter *>(dev))->is_NotifdState_allowed(ty);}
+};
+
 //=========================================
 //	Define classes for commands
 //=========================================
+class UpdateServersInfoCmd : public Tango::Command
+{
+public:
+	UpdateServersInfoCmd(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out,
+				   const char        *in_desc,
+				   const char        *out_desc,
+				   Tango::DispLevel  level)
+	:Command(name,in,out,in_desc,out_desc, level)	{};
+
+	UpdateServersInfoCmd(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out)
+	:Command(name,in,out)	{};
+	~UpdateServersInfoCmd() {};
+	
+	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
+	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
+	{return (static_cast<Starter *>(dev))->is_UpdateServersInfo_allowed(any);}
+};
+
+
+
 class NotifyDaemonStateCmd : public Tango::Command
 {
 public:
@@ -168,10 +219,10 @@ public:
 
 
 
-class UpdateServersInfoCmd : public Tango::Command
+class HardKillServerClass : public Tango::Command
 {
 public:
-	UpdateServersInfoCmd(const char   *name,
+	HardKillServerClass(const char   *name,
 	               Tango::CmdArgType in,
 				   Tango::CmdArgType out,
 				   const char        *in_desc,
@@ -179,15 +230,15 @@ public:
 				   Tango::DispLevel  level)
 	:Command(name,in,out,in_desc,out_desc, level)	{};
 
-	UpdateServersInfoCmd(const char   *name,
+	HardKillServerClass(const char   *name,
 	               Tango::CmdArgType in,
 				   Tango::CmdArgType out)
 	:Command(name,in,out)	{};
-	~UpdateServersInfoCmd() {};
+	~HardKillServerClass() {};
 	
 	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
 	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
-	{return (static_cast<Starter *>(dev))->is_UpdateServersInfo_allowed(any);}
+	{return (static_cast<Starter *>(dev))->is_HardKillServer_allowed(any);}
 };
 
 
@@ -364,7 +415,11 @@ public:
 // The StarterClass singleton definition
 //
 
-class StarterClass : public Tango::DeviceClass
+class
+#ifdef WIN32
+	__declspec(dllexport)
+#endif
+	StarterClass : public Tango::DeviceClass
 {
 public:
 //	properties member data
