@@ -1,4 +1,4 @@
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/StarterUtil.cpp,v 3.15 2007/05/25 06:08:00 pascal_verdier Exp $";
+static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/StarterUtil.cpp,v 3.20 2008/03/27 07:55:17 pascal_verdier Exp $";
 //+=============================================================================
 //
 // file :         StarterUtil.cpp
@@ -9,9 +9,24 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 3.15 $
+// $Revision: 3.20 $
 //
 // $Log: StarterUtil.cpp,v $
+// Revision 3.20  2008/03/27 07:55:17  pascal_verdier
+// *** empty log message ***
+//
+// Revision 3.19  2008/03/03 13:25:35  pascal_verdier
+// Do not try to narrow notifd if not running.
+//
+// Revision 3.18  2008/02/29 15:15:05  pascal_verdier
+// Checking running processes by system call added.
+//
+// Revision 3.17  2007/11/12 08:09:28  pascal_verdier
+// const char* cast added on throw_exception call.
+//
+// Revision 3.16  2007/09/25 12:12:00  pascal_verdier
+// little memory leaks fixed.
+//
 // Revision 3.15  2007/05/25 06:08:00  pascal_verdier
 // Iterator management modified (strange in vc8 ?)
 //
@@ -147,6 +162,9 @@ StarterUtil::StarterUtil(Tango::DeviceProxy *database, vector<string> host_names
 	//	Remove the Fully Qualify Domain Name for tango less than 5.2 compatibility
 	for (unsigned int i=0 ; i<host_names.size() ; i++)
 		hostnames.push_back(removeFQDN(host_names[i]));
+
+	proc_util = new CheckProcessUtil();
+	proc_util->start();
 }
 //+------------------------------------------------------------------
 /**
@@ -275,7 +293,7 @@ char *StarterUtil::check_exe_file(char *servname, vector<string>v_path)
 	for (i=0 ; i<v_path.size() ; i++)
 		out_stream << " - " << v_path[i] << endl;
 	out_stream << ends;
-	Tango::Except::throw_exception("CANNOT_RUN_FILE",
+	Tango::Except::throw_exception((const char *)"CANNOT_RUN_FILE",
 				out_stream.str(),
 				(const char *)"StarterUtil::check_exe_file()");
 	return NULL;
@@ -551,15 +569,16 @@ void StarterUtil::build_server_ctrl_object(vector<ControledServer> *servers)
 			server.dev = NULL;
 			server.controled = (atoi((*pos++).c_str())==0)? false: true;
 			server.startup_level =  atoi((*pos++).c_str());
-			server.running = true;
+			server.state   = Tango::FAULT;
 
 			//	Add a thread to ping server
 			server.thread_data = new PingThreadData(server.name);
-			server.thread = new PingThread(server.thread_data, server.name);
+			server.thread = 
+				new PingThread(server.thread_data, server.name, proc_util);
 			server.thread->start();
 
 			servers->push_back(server);
-			ms_sleep(50);
+			ms_sleep(50);	//	wait for server updated.
 		}
 		else
 		{
@@ -668,26 +687,39 @@ void StarterUtil::import_notifyd()
 //-----------------------------------------------------------------------------
 Tango::DevState StarterUtil::is_notifyd_alive()
 {
-	try
+	string	notify_procname("notifd");
+	Tango::DevState	notifd_state;
+	if (proc_util->is_process_running(notify_procname))
 	{
-		if (ch_factory==NULL)
-			import_notifyd();
+		try
+		{
+			//cout << notify_procname << " is running" << endl;
+			if (ch_factory==NULL)
+				import_notifyd();
 
-		CosNotifyChannelAdmin::ChannelIDSeq	*ch_id = ch_factory->get_all_channels();
-		delete ch_id;
+			CosNotifyChannelAdmin::ChannelIDSeq	*ch_id =
+							ch_factory->get_all_channels();
+			delete ch_id;
 
-		cout2 << notifyd_name << "EventChannelFactory is ON" << endl;
- 		return Tango::DevState(Tango::ON);
+			cout2 << notifyd_name << "EventChannelFactory is ON" << endl;
+ 			
+			notifd_state = Tango::ON;
+		}
+		catch (...)
+		{
+			cerr << notify_procname << " is running  BUT not responding" << endl;			
+			//cerr << "Failed to narrow the EventChannelFactory on " << notifyd_name << endl;
+			ch_factory = NULL;
+ 			notifd_state = Tango::UNKNOWN;
+		}
 	}
-	catch (...)
+	else
 	{
-		cerr << "Failed to narrow the EventChannelFactory on " << notifyd_name << endl;
-		cerr << " make sure the notifd process is running on this host" << endl;
-		ch_factory = NULL;
- 		return Tango::DevState(Tango::FAULT);
+		//cout << notify_procname << " is NOT running" << endl;
+ 		notifd_state = Tango::FAULT;
 	}
-	//	Never used.
-	return Tango::DevState(Tango::ON);
+
+	return notifd_state;
 }
 
 
@@ -700,7 +732,6 @@ Tango::DevState StarterUtil::is_notifyd_alive()
 
 
 
-
 //+----------------------------------------------------------------------------
 /**
  *	Create a thread to start  polling.
@@ -765,7 +796,9 @@ void PollingState::add_obj_polling(Tango::DServer *adm_dev,string type, string o
 	CORBA::Any	any;
 	any <<= lsa;
 	try {
-		adm_dev->command_inout("AddObjPolling", any);
+		CORBA::Any	*any_out =
+			adm_dev->command_inout("AddObjPolling", any);
+		delete any_out;
 	}
 	catch (Tango::DevFailed &e) {
 
@@ -775,7 +808,9 @@ void PollingState::add_obj_polling(Tango::DServer *adm_dev,string type, string o
 			//	If already polled -> just update period.
 			//----------------------------------------------
 			try {
-				adm_dev->command_inout("UpdObjPollingPeriod", any);
+				CORBA::Any	*any_out =
+					adm_dev->command_inout("UpdObjPollingPeriod", any);
+				delete any_out;
 				//cout <<"UpdObjPollingPeriod(" << objname <<
 				//			", " << upd_period << " ms)" << endl;
 			}
