@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: api_util.cpp,v 3.12 2007/04/20 14:38:33 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: api_util.cpp,v 3.14 2008/03/11 14:36:44 taurel Exp $\n$Name:  $";
 
 //
 // cpp 	- C++ source code file for TANGO api class ApiUtil
@@ -8,6 +8,12 @@ static const char *RcsId = "$Id: api_util.cpp,v 3.12 2007/04/20 14:38:33 taurel
 // original 	- May 2002
 //
 // $Log: api_util.cpp,v $
+// Revision 3.14  2008/03/11 14:36:44  taurel
+// - Apply patches from Frederic Picca about compilation with gcc 4.2
+//
+// Revision 3.13  2008/02/29 13:24:30  taurel
+// - Add a try/catch in the get_asynch_replies() method
+//
 // Revision 3.12  2007/04/20 14:38:33  taurel
 // - Ported to Windows 64 bits x64 architecture
 //
@@ -573,42 +579,50 @@ void ApiUtil::get_asynch_replies(long call_timeout)
 				nanosleep(&to_wait,&inter);
 #endif
 				nb--;
-			
-				if (_orb->poll_next_response() == true)
-				{
-					_orb->get_next_response(req);
+
+				try
+				{			
+					if (_orb->poll_next_response() == true)
+					{
+						_orb->get_next_response(req);
 					
 //
 // Retrieve this request in the cb request map and mark it as "arrived" in both maps
 //
 
-					TgRequest &tg_req = asyn_p_table->get_request(req);
+						TgRequest &tg_req = asyn_p_table->get_request(req);
 
-					tg_req.arrived = true;
-					asyn_p_table->mark_as_arrived(req);
+						tg_req.arrived = true;
+						asyn_p_table->mark_as_arrived(req);
 					
 //		
 // Is it a request for our device, process it ?
 //
 
-					switch (tg_req.req_type)
-					{
-					case TgRequest::CMD_INOUT :
-						tg_req.dev->Cb_Cmd_Request(req,tg_req.cb_ptr);
-						break;
-				
-					case TgRequest::READ_ATTR :
-						tg_req.dev->Cb_ReadAttr_Request(req,tg_req.cb_ptr);
-						break;
-				
-					case TgRequest::WRITE_ATTR :
-					case TgRequest::WRITE_ATTR_SINGLE :
-						tg_req.dev->Cb_WriteAttr_Request(req,tg_req.cb_ptr);
-						break;
+						switch (tg_req.req_type)
+						{
+						case TgRequest::CMD_INOUT :
+							tg_req.dev->Cb_Cmd_Request(req,tg_req.cb_ptr);
+							break;
+
+						case TgRequest::READ_ATTR :
+							tg_req.dev->Cb_ReadAttr_Request(req,tg_req.cb_ptr);
+							break;
+
+						case TgRequest::WRITE_ATTR :
+						case TgRequest::WRITE_ATTR_SINGLE :
+							tg_req.dev->Cb_WriteAttr_Request(req,tg_req.cb_ptr);
+							break;
+						}
+
+						tg_req.dev->dec_asynch_counter(CALL_BACK);
+						asyn_p_table->remove_request(tg_req.dev,req);
 					}
-			
-					tg_req.dev->dec_asynch_counter(CALL_BACK);
-					asyn_p_table->remove_request(tg_req.dev,req);
+				} 
+				catch (CORBA::BAD_INV_ORDER &e)
+				{
+					if (e.minor() != omni::BAD_INV_ORDER_RequestNotSentYet)
+						throw;
 				}
 			}
 			
@@ -635,39 +649,47 @@ void ApiUtil::get_asynch_replies(long call_timeout)
 
 			while (asyn_p_table->get_cb_request_nb() != 0)
 			{					
-				_orb->get_next_response(req);
+				try
+				{
+					_orb->get_next_response(req);
 								
 //
 // Retrieve this request in the cb request map and mark it as "arrived" in both maps
 //
 
-				TgRequest &tg_req = asyn_p_table->get_request(req);
+					TgRequest &tg_req = asyn_p_table->get_request(req);
 
-				tg_req.arrived = true;
-				asyn_p_table->mark_as_arrived(req);
+					tg_req.arrived = true;
+					asyn_p_table->mark_as_arrived(req);
 				
 //		
 // Process the reply
 //
 
-				switch (tg_req.req_type)
-				{
-				case TgRequest::CMD_INOUT :
-					tg_req.dev->Cb_Cmd_Request(req,tg_req.cb_ptr);
-					break;
-				
-				case TgRequest::READ_ATTR :
-					tg_req.dev->Cb_ReadAttr_Request(req,tg_req.cb_ptr);
-					break;
-				
-				case TgRequest::WRITE_ATTR :
-				case TgRequest::WRITE_ATTR_SINGLE :
-					tg_req.dev->Cb_WriteAttr_Request(req,tg_req.cb_ptr);
-					break;
+					switch (tg_req.req_type)
+					{
+					case TgRequest::CMD_INOUT :
+						tg_req.dev->Cb_Cmd_Request(req,tg_req.cb_ptr);
+						break;
+
+					case TgRequest::READ_ATTR :
+						tg_req.dev->Cb_ReadAttr_Request(req,tg_req.cb_ptr);
+						break;
+
+					case TgRequest::WRITE_ATTR :
+					case TgRequest::WRITE_ATTR_SINGLE :
+						tg_req.dev->Cb_WriteAttr_Request(req,tg_req.cb_ptr);
+						break;
+					}
+
+					tg_req.dev->dec_asynch_counter(CALL_BACK);
+					asyn_p_table->remove_request(tg_req.dev,req);
 				}
-			
-				tg_req.dev->dec_asynch_counter(CALL_BACK);
-				asyn_p_table->remove_request(tg_req.dev,req);
+				catch (CORBA::BAD_INV_ORDER &e)
+				{
+					if (e.minor() != omni::BAD_INV_ORDER_RequestNotSentYet)
+						throw;
+				}			
 			}
 		}
 	}
@@ -1058,6 +1080,9 @@ void ApiUtil::attr_to_device(const AttributeValue *attr_value,
 					dev_attr->ext->StateSeq = new DevVarStateArray(max,len,tmp_state,false);
 				}
 				break;
+				
+			default:
+				break;
 		}
 	}	
 }
