@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: attr_proxy.cpp,v 1.7 2007/04/20 14:38:33 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: attr_proxy.cpp,v 1.8 2008/01/18 14:44:40 jensmeyer Exp $\n$Name:  $";
 //
 // attr_proxy.cpp 	- C++ source code file for TANGO attribute proxy api 
 //
@@ -1282,23 +1282,42 @@ void AttributeProxy::stop_poll()
 	dev_proxy->stop_poll_attribute(attr_name);
 }
 
+
 //-----------------------------------------------------------------------------
 //
 // AttributeProxy::subscribe_event - Subscribe to an event
+//												 Old interface for compatibility
 //
 //-----------------------------------------------------------------------------
 
-int AttributeProxy::subscribe_event (EventType event, CallBack *callback, const vector<string> &filters)
+int AttributeProxy::subscribe_event (EventType event, CallBack *callback, 
+                                    const vector<string> &filters)
+{
+	return subscribe_event (event, callback, filters, false);
+}
+
+
+//-----------------------------------------------------------------------------
+//
+// AttributeProxy::subscribe_event - Subscribe to an event
+//                                   Adds the statless flag for stateless
+//                                   event subscription.
+//
+//-----------------------------------------------------------------------------
+
+int AttributeProxy::subscribe_event (EventType event, CallBack *callback, 
+                                    const vector<string> &filters, bool stateless)
 {
   	if (ApiUtil::instance()->get_event_consumer() == NULL)
 	{
 		ApiUtil::instance()->create_event_consumer();
 	}
 
-	ApiUtil::instance()->get_event_consumer()->connect(dev_proxy);
+	//ApiUtil::instance()->get_event_consumer()->connect(dev_proxy);
 
 	
-	return ApiUtil::instance()->get_event_consumer()->subscribe_event(dev_proxy, attr_name, event, callback, filters);
+	return ApiUtil::instance()->get_event_consumer()->subscribe_event(dev_proxy, 
+	                           attr_name, event, callback, filters, stateless);
 }
 
 
