@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dbapi_class.cpp,v 3.1 2003/05/28 14:42:55 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dbapi_class.cpp,v 3.3 2008/02/29 13:21:23 taurel Exp $\n$Name:  $";
 //
 // dbapi_class.cpp - C++ source code file for TANGO dbapi class DbClass
 //
@@ -70,11 +70,29 @@ DbClass::~DbClass()
 
 void DbClass::get_property(DbData &db_data)
 {
+//
+// Try to get db server cache in case we are called during a DS
+// startup sequence
+//
+	
+	ApiUtil *au = ApiUtil::instance();
+	DbServerCache *dsc;
+	if (au->in_server() == true)
+	{
+		Tango::Util *tg = Tango::Util::instance();
+		dsc = tg->get_db_cache();
+	}
+	else
+		dsc = NULL;
+
+//
+// Call DB (or cache)
+//
+	
 	if (ext_dbase == true)
-		dbase->get_class_property(name, db_data);
+		dbase->get_class_property(name, db_data, dsc);
 	else
 	{
-		ApiUtil *au = ApiUtil::instance();
 		(au->get_db_vect())[db_ind]->get_class_property(name, db_data);
 	}
 }
@@ -87,12 +105,43 @@ void DbClass::get_property(DbData &db_data)
 
 void DbClass::put_property(DbData &db_data)
 {
-	if (ext_dbase == true)
-		dbase->put_class_property(name, db_data);
-	else
+
+//
+// Protect DS code againt a DB exception while the server is in its starting phase
+//
+
+	ApiUtil *au = ApiUtil::instance();
+	bool forget_except = false;
+	if (au->in_server() == true)
 	{
-		ApiUtil *au = ApiUtil::instance();
-		(au->get_db_vect())[db_ind]->put_class_property(name, db_data);
+		Tango::Util *tg = Tango::Util::instance();
+		if (tg->is_svr_starting() == true)
+		{
+			if (db_data.size() >= 2)
+			{
+				if ((db_data[0].name == POGO_TITLE) && (db_data[1].name == POGO_DESC))
+					forget_except = true;
+			}
+		}
+	}
+
+//
+// Call DB
+//
+
+	try
+	{
+		if (ext_dbase == true)
+			dbase->put_class_property(name, db_data);
+		else
+		{
+			(au->get_db_vect())[db_ind]->put_class_property(name, db_data);
+		}
+	}
+	catch (Tango::DevFailed &e)
+	{
+		if (forget_except == false)
+			throw;
 	}
 }
 
