@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dbapi_datum.cpp,v 3.4 2007/03/06 08:20:45 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dbapi_datum.cpp,v 3.5 2007/11/08 12:00:20 taurel Exp $\n$Name:  $";
 //
 // dbdatum.cpp 	- C++ source code file for TANGO dbapi class DbDatum
 //
@@ -11,6 +11,11 @@ static const char *RcsId = "$Id: dbapi_datum.cpp,v 3.4 2007/03/06 08:20:45 taure
 // version 	- 1.0
 //
 // $Log: dbapi_datum.cpp,v $
+// Revision 3.5  2007/11/08 12:00:20  taurel
+// - Add miscellaneous getter methods in the DbServerCache class
+// - The DbDatum::Size() method is now inline
+// - Fix bug in database re-connection timeout management
+//
 // Revision 3.4  2007/03/06 08:20:45  taurel
 // - Added 64 bits data types for 64 bits computer...
 //
@@ -204,17 +209,6 @@ DbDatum::~DbDatum()
 {
 }
 
-//-----------------------------------------------------------------------------
-//
-// DbDatum::size() - returns number of elements in DbDatum
-//
-//-----------------------------------------------------------------------------
-
-int DbDatum::size()
-{
-	return value_string.size();
-}
-
 //-----------------------------------------------------------------------------
 //
 // DbDatum::is_empty() - returns true or false if datum is empty
