@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dbapi_device.cpp,v 3.2 2004/07/07 08:39:56 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dbapi_device.cpp,v 3.3 2007/10/16 08:25:47 taurel Exp $\n$Name:  $";
 //
 // dbdevice.cpp - C++ source code file for TANGO dbapi class DbDevice
 //
@@ -149,11 +149,20 @@ void DbDevice::export_device(DbDevExportInfo &dev_export)
 
 void DbDevice::get_property(DbData &db_data)
 {
+	ApiUtil *au = ApiUtil::instance();
+	DbServerCache *dsc;
+	if (au->in_server() == true)
+	{
+		Tango::Util *tg = Tango::Util::instance();
+		dsc = tg->get_db_cache();
+	}
+	else
+		dsc = NULL;
+	
 	if (ext_dbase == true)
-		dbase->get_device_property(name, db_data);
+		dbase->get_device_property(name, db_data, dsc);
 	else
 	{
-		ApiUtil *au = ApiUtil::instance();
 		(au->get_db_vect())[db_ind]->get_device_property(name,db_data);
 	}
 	return;
