@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dbapi_history.cpp,v 3.1 2007/04/30 14:05:47 jlpons Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dbapi_history.cpp,v 3.2 2008/03/11 14:36:44 taurel Exp $\n$Name:  $";
 //
 // dbapi_history.cpp - C++ source code file for TANGO dbapi class DbHistory
 //
@@ -7,6 +7,9 @@ static const char *RcsId = "$Id: dbapi_history.cpp,v 3.1 2007/04/30 14:05:47 jlp
 // original 	- Feb 2007
 //
 // $Log: dbapi_history.cpp,v $
+// Revision 3.2  2008/03/11 14:36:44  taurel
+// - Apply patches from Frederic Picca about compilation with gcc 4.2
+//
 // Revision 3.1  2007/04/30 14:05:47  jlpons
 // Added new commands to the database client interface.
 //
@@ -145,7 +148,7 @@ void DbHistory::make_db_datum(vector<string> &values) {
 
   value.name = propname;
   value.value_string.resize(values.size());
-  for (int i=0; i<values.size(); i++)
+  for (unsigned int i=0; i<values.size(); i++)
     value.value_string[i] = values[i]; 
 
 }
