@@ -46,13 +46,17 @@ typedef vector<DbDatum> DbData;
 class DeviceAttributeExt
 {
 public:
-	DeviceAttributeExt():w_dim_x(0),w_dim_y(0),long_from_64(false) {};
+	DeviceAttributeExt():w_dim_x(0),w_dim_y(0) {};
+	DeviceAttributeExt & operator=(const DeviceAttributeExt &);
 	
-	DevErrorList_var	err_list;
-	long 			w_dim_x;
-	long			w_dim_y;
+	DevErrorList_var		err_list;
+	long 					w_dim_x;
+	long					w_dim_y;
 	
-	bool			long_from_64;			
+	DevVarLong64Array_var	Long64Seq;
+	DevVarULongArray_var	ULongSeq;
+	DevVarULong64Array_var	ULong64Seq;
+	DevVarStateArray_var	StateSeq;	
 };
 
 class ConnectionExt
@@ -61,11 +65,11 @@ public:
 	ConnectionExt():tr_reco(true),prev_failed(false),prev_failed_t0(0.0) {};
 	ConnectionExt & operator=(const ConnectionExt &);
 	
-	bool			tr_reco;
-	Tango::Device_3_var 	device_3;
+	bool				tr_reco;
+	Tango::Device_3_var device_3;
 	
-	bool			  prev_failed;
-	double		  prev_failed_t0;
+	bool			  	prev_failed;
+	double		  		prev_failed_t0;
 };
 
 class ApiUtilExt
@@ -78,12 +82,12 @@ public:
 
 struct _DevCommandInfo
 {
-	string cmd_name;
-	long cmd_tag;
-	long in_type;
-	long out_type;
-	string in_type_desc;
-	string out_type_desc;
+	string 		cmd_name;
+	long 		cmd_tag;
+	long 		in_type;
+	long 		out_type;
+	string 		in_type_desc;
+	string 		out_type_desc;
 };
 
 struct AttributeDimension
@@ -118,23 +122,23 @@ typedef _DeviceInfo DeviceInfo;
 
 struct _DeviceAttributeConfig
 {
-	string 		name;
+	string 			name;
 	AttrWriteType 	writable;
 	AttrDataFormat 	data_format;
-	int 		data_type;
-	int 		max_dim_x;
-	int 		max_dim_y;
-	string 		description;
-	string 		label;
-	string 		unit;
-	string 		standard_unit;
-	string 		display_unit;
-	string 		format;
-	string 		min_value;
-	string 		max_value;
-	string 		min_alarm;
-	string 		max_alarm;
-	string 		writable_attr_name;
+	int 			data_type;
+	int 			max_dim_x;
+	int 			max_dim_y;
+	string 			description;
+	string 			label;
+	string 			unit;
+	string 			standard_unit;
+	string 			display_unit;
+	string 			format;
+	string 			min_value;
+	string 			max_value;
+	string 			min_alarm;
+	string 			max_alarm;
+	string 			writable_attr_name;
 	vector<string> 	extensions;
 };
 	
@@ -153,33 +157,33 @@ typedef vector<AttributeInfo> AttributeInfoList;
 
 struct _AttributeAlarmInfo
 {
-	string		min_alarm;
-	string		max_alarm;
-	string 		min_warning;
-	string		max_warning;
-	string		delta_t;
-	string		delta_val;
+	string			min_alarm;
+	string			max_alarm;
+	string 			min_warning;
+	string			max_warning;
+	string			delta_t;
+	string			delta_val;
 	vector<string>	extensions;
 };
 
 struct _ChangeEventInfo
 {
-	string		rel_change;
-	string		abs_change;
+	string			rel_change;
+	string			abs_change;
 	vector<string>	extensions;
 };
 
 struct _PeriodicEventInfo
 {
-	string		period;
+	string			period;
 	vector<string>	extensions;
 };
 
 struct _ArchiveEventInfo
 {
-	string		archive_rel_change;
-	string		archive_abs_change;
-	string		archive_period;
+	string			archive_rel_change;
+	string			archive_abs_change;
+	string			archive_period;
 	vector<string>	extensions;
 };
 
@@ -203,6 +207,9 @@ struct _AttributeInfoEx : public AttributeInfo
 	AttributeEventInfo	events;
 	vector<string>		sys_extensions;
 	
+	_AttributeInfoEx & operator=(AttributeConfig_2 *);
+	_AttributeInfoEx & operator=(AttributeConfig_3 *);
+	
 	friend ostream &operator<<(ostream &,_AttributeInfoEx &);
 };
 
@@ -230,41 +237,50 @@ enum cb_sub_model
 #define 	CONNECTION_OK		1
 #define 	CONNECTION_NOTOK	0
 
-#define		PROT_SEP		"://"
+#define		PROT_SEP			"://"
 #define		TACO_PROTOCOL		"taco"
 #define		TANGO_PROTOCOL		"tango"
 
-#define		MODIFIER		'#'
-#define		DBASE_YES		"dbase=yes"
-#define		DBASE_NO		"dbase=no"
+#define		MODIFIER			'#'
+#define		DBASE_YES			"dbase=yes"
+#define		DBASE_NO			"dbase=no"
 
-#define		HOST_SEP		':'
-#define		PORT_SEP		'/'
+#define		HOST_SEP			':'
+#define		PORT_SEP			'/'
 #define		DEV_NAME_FIELD_SEP	'/'
-#define		RES_SEP			"->"
-#define		DEVICE_SEP		'/'
+#define		RES_SEP				"->"
+#define		DEVICE_SEP			'/'
 
-#define		FROM_IOR		"IOR"
-#define		NOT_USED		"Unused"
+#define		FROM_IOR			"IOR"
+#define		NOT_USED			"Unused"
 
+
+/****************************************************************************************
+ * 																						*
+ * 					The ApiUtil class													*
+ * 					-----------------													*
+ * 																						*
+ ***************************************************************************************/
+ 
 class ApiUtil
 {
 public:
 	TANGO_IMP_EXP static inline ApiUtil *instance()
 	{if (_instance == NULL)_instance = new ApiUtil();return _instance;}
 
-	CORBA::ORB_ptr	get_orb() {return CORBA::ORB::_duplicate(_orb);}
-	void		set_orb(CORBA::ORB_ptr orb_in) {_orb = orb_in;}
-	void 		create_orb();
-	int		get_db_ind();
-	int		get_db_ind(string &host,int port);
+	CORBA::ORB_ptr get_orb() {return CORBA::ORB::_duplicate(_orb);}
+	void set_orb(CORBA::ORB_ptr orb_in) {_orb = orb_in;}
+	void create_orb();
+	int	get_db_ind();
+	int	get_db_ind(string &host,int port);
 	vector<Database *> &get_db_vect() {return db_vect;}
-	bool		in_server() {return in_serv;}
-	AsynReq		*get_pasyn_table() {return asyn_p_table;}
-	void		create_event_consumer();
-	EventConsumer	*get_event_consumer() {return ext->event_consumer;}
+	bool in_server() {return in_serv;}
+	void in_server(bool serv) {in_serv = serv;}
+	AsynReq	*get_pasyn_table() {return asyn_p_table;}
+	void create_event_consumer();
+	EventConsumer *get_event_consumer() {return ext->event_consumer;}
 	
-	long 		pending_asynch_call(asyn_req_type ty)
+	size_t pending_asynch_call(asyn_req_type ty)
 	{if (ty==POLLING)return asyn_p_table->get_request_nb();
 	 else if (ty==CALL_BACK)return asyn_p_table->get_cb_request_nb();
 	 else return (asyn_p_table->get_request_nb()+asyn_p_table->get_cb_request_nb());}
@@ -273,7 +289,7 @@ public:
 	{if (_instance != NULL){delete _instance;_instance=NULL;}}
 	
 //
-//Asynchronous methods
+// Asynchronous methods
 //
 	
 	void get_asynch_replies();
@@ -289,20 +305,27 @@ protected:
 	
 	vector<Database *>	db_vect;
 	CORBA::ORB_ptr		_orb;
-	bool			in_serv;
+	bool				in_serv;
 	
 	cb_sub_model		auto_cb;
-	CbThreadCmd		cb_thread_cmd;
+	CbThreadCmd			cb_thread_cmd;
 	CallBackThread		*cb_thread_ptr;
 	
-	AsynReq			*asyn_p_table;
+	AsynReq				*asyn_p_table;
 		
 private:
-	TANGO_IMP static ApiUtil 		*_instance;
-	ApiUtilExt				*ext; 		// Class extension
+	TANGO_IMP static ApiUtil 	*_instance;
+	ApiUtilExt					*ext; 		// Class extension
 };
 
-
+/****************************************************************************************
+ * 																						*
+ * 					The DeviceData class												*
+ * 					--------------------												*
+ * 																						*
+ ***************************************************************************************/
+ 
+ 
 class DeviceData
 {
 
@@ -312,8 +335,8 @@ public :
 //
 	enum except_flags
 	{
-                isempty_flag,
-                wrongtype_flag,
+		isempty_flag,
+		wrongtype_flag,
 		numFlags
 	};
 
@@ -336,8 +359,10 @@ public :
 	void operator << (bool datum) {any <<= CORBA::Any::from_boolean(datum);}
 	void operator << (short datum) {any <<= datum;}
 	void operator << (unsigned short datum) {any <<= datum;}
-	void operator << (long datum) {any <<= datum;}
-	void operator << (unsigned long datum) {any <<= datum;}
+	void operator << (DevLong datum) {any <<= datum;}
+	void operator << (DevULong datum) {any <<= datum;}
+	void operator << (DevLong64 datum) {any <<= datum;}
+	void operator << (DevULong64 datum) {any <<= datum;}	
 	void operator << (float datum) {any <<= datum;}
 	void operator << (double datum) {any <<= datum;}
 	void operator << (char *&datum) {any <<= datum;}
@@ -346,14 +371,17 @@ public :
 	void operator << (vector<unsigned char>&);
 	void operator << (vector<string>&);
 	void operator << (vector<short>&);
-	void operator << (vector<unsigned short>&);
-	void operator << (vector<long>&);
-	void operator << (vector<unsigned long>&);
+	void operator << (vector<unsigned short>&);	
+	void operator << (vector<DevLong> &);
+	void operator << (vector<DevULong> &);
+	void operator << (vector<DevLong64> &);
+	void operator << (vector<DevULong64> &);
 	void operator << (vector<float>&);
 	void operator << (vector<double>&);
 	void operator << (DevState datum) {(any.inout()) <<= datum;}
-	void insert(vector<long>&, vector<string>&);
+	void insert(vector<DevLong>&, vector<string>&);
 	void insert(vector<double>&, vector<string>&);
+
 //
 // insert methods for TANGO CORBA sequence types
 //
@@ -361,7 +389,9 @@ public :
 	inline void operator << (DevVarShortArray* datum) { any.inout() <<= datum;}
 	inline void operator << (DevVarUShortArray* datum) { any.inout() <<= datum;}
 	inline void operator << (DevVarLongArray* datum) { any.inout() <<= datum;}
+	inline void operator << (DevVarLong64Array *datum) { any.inout() <<= datum;}
 	inline void operator << (DevVarULongArray* datum) { any.inout() <<= datum;}
+	inline void operator << (DevVarULong64Array* datum) { any.inout() <<= datum;}
 	inline void operator << (DevVarFloatArray* datum) { any.inout() <<= datum;}
 	inline void operator << (DevVarDoubleArray* datum) { any.inout() <<= datum;}
 	inline void operator << (DevVarStringArray* datum) { any.inout() <<= datum;}
@@ -372,7 +402,9 @@ public :
 	inline void operator << (DevVarShortArray &datum) { any.inout() <<= datum;}
 	inline void operator << (DevVarUShortArray datum) { any.inout() <<= datum;}
 	inline void operator << (DevVarLongArray &datum) { any.inout() <<= datum;}
+	inline void operator << (DevVarLong64Array &datum) { any.inout() <<= datum;}
 	inline void operator << (DevVarULongArray &datum) { any.inout() <<= datum;}
+	inline void operator << (DevVarULong64Array &datum) { any.inout() <<= datum;}
 	inline void operator << (DevVarFloatArray &datum) { any.inout() <<= datum;}
 	inline void operator << (DevVarDoubleArray &datum) { any.inout() <<= datum;}
 	inline void operator << (DevVarStringArray &datum) { any.inout() <<= datum;}
@@ -385,22 +417,27 @@ public :
 	bool operator >> (bool&);
 	bool operator >> (short&);
 	bool operator >> (unsigned short&);
-	bool operator >> (long&);
-	bool operator >> (unsigned long&);
+	bool operator >> (DevLong&);
+	bool operator >> (DevULong&);
+	bool operator >> (DevLong64&);
+	bool operator >> (DevULong64&);
 	bool operator >> (float&);
 	bool operator >> (double&);
 	bool operator >> (const char*&);
 	bool operator >> (string&);
+	
 	bool operator >> (vector<unsigned char>&);
 	bool operator >> (vector<string>&);
 	bool operator >> (vector<short>&);
 	bool operator >> (vector<unsigned short>&);
-	bool operator >> (vector<long>&);
-	bool operator >> (vector<unsigned long>&);
+	bool operator >> (vector<DevLong>&);
+	bool operator >> (vector<DevULong>&);
+	bool operator >> (vector<DevLong64>&);
+	bool operator >> (vector<DevULong64>&);
 	bool operator >> (vector<float>&);
 	bool operator >> (vector<double>&);
 	bool operator >> (DevState&);
-	bool extract(vector<long>&, vector<string>&);
+	bool extract(vector<DevLong>&, vector<string>&);
 	bool extract(vector<double>&, vector<string>&);
 //
 // extract methods for TANGO CORBA sequence types
@@ -409,7 +446,9 @@ public :
 	bool operator >> (const DevVarShortArray* &datum);
 	bool operator >> (const DevVarUShortArray* &datum);
 	bool operator >> (const DevVarLongArray* &datum);
+	bool operator >> (const DevVarLong64Array* &datum);
 	bool operator >> (const DevVarULongArray* &datum);
+	bool operator >> (const DevVarULong64Array* &datum);
 	bool operator >> (const DevVarFloatArray* &datum);
 	bool operator >> (const DevVarDoubleArray* &datum);
 	bool operator >> (const DevVarStringArray* &datum);
@@ -427,7 +466,13 @@ protected :
 };
 
 
-
+/****************************************************************************************
+ * 																						*
+ * 					The DeviceAttribute class											*
+ * 					-------------------------											*
+ * 																						*
+ ***************************************************************************************/
+ 
 class DeviceAttribute
 {
 
@@ -435,10 +480,10 @@ public :
 //
 // constructor methods
 //
-        enum except_flags
+	enum except_flags
 	{
-                isempty_flag,
-                wrongtype_flag,
+		isempty_flag,
+		wrongtype_flag,
 		failed_flag,
 		numFlags
 	};
@@ -450,79 +495,106 @@ public :
 	DeviceAttribute(AttributeValue);
 	
 	DeviceAttribute(string&, short);
-	DeviceAttribute(string&, long);
+	DeviceAttribute(string&, DevLong);
 	DeviceAttribute(string&, double);
 	DeviceAttribute(string&, string&);
 	DeviceAttribute(string&, float);
 	DeviceAttribute(string&, bool);
 	DeviceAttribute(string&, unsigned short);
 	DeviceAttribute(string&, unsigned char);
+	DeviceAttribute(string&, DevLong64);
+	DeviceAttribute(string&, DevULong);
+	DeviceAttribute(string&, DevULong64);
+	DeviceAttribute(string&, DevState);
 	
 	DeviceAttribute(string&, vector<short> &);
-	DeviceAttribute(string&, vector<long> &);
+	DeviceAttribute(string&, vector<DevLong> &);
 	DeviceAttribute(string&, vector<double> &);
 	DeviceAttribute(string&, vector<string> &);
 	DeviceAttribute(string&, vector<float> &);
 	DeviceAttribute(string&, vector<bool> &);
 	DeviceAttribute(string&, vector<unsigned short> &);
 	DeviceAttribute(string&, vector<unsigned char> &);
-	
+	DeviceAttribute(string&, vector<DevLong64> &);
+	DeviceAttribute(string&, vector<DevULong> &);
+	DeviceAttribute(string&, vector<DevULong64> &);
+	DeviceAttribute(string&, vector<DevState> &);
+		
 	DeviceAttribute(string&, vector<short> &,int,int);
-	DeviceAttribute(string&, vector<long> &,int,int);
+	DeviceAttribute(string&, vector<DevLong> &,int,int);
 	DeviceAttribute(string&, vector<double> &,int,int);
 	DeviceAttribute(string&, vector<string> &,int,int);
 	DeviceAttribute(string&, vector<float> &,int,int);
 	DeviceAttribute(string&, vector<bool> &,int,int);
 	DeviceAttribute(string&, vector<unsigned short> &,int,int);
 	DeviceAttribute(string&, vector<unsigned char> &,int,int);
-		
+	DeviceAttribute(string&, vector<DevLong64> &,int,int);
+	DeviceAttribute(string&, vector<DevULong> &,int,int);
+	DeviceAttribute(string&, vector<DevULong64> &,int,int);
+	DeviceAttribute(string&, vector<DevState> &,int,int);
+			
 	DeviceAttribute(const char *, short);
-	DeviceAttribute(const char *, long);
+	DeviceAttribute(const char *, DevLong);
 	DeviceAttribute(const char *, double);
 	DeviceAttribute(const char *, string&);
 	DeviceAttribute(const char *, float);
 	DeviceAttribute(const char *, bool);
 	DeviceAttribute(const char *, unsigned short);
 	DeviceAttribute(const char *, unsigned char);
-	
+	DeviceAttribute(const char *, DevLong64);
+	DeviceAttribute(const char *, DevULong);
+	DeviceAttribute(const char *, DevULong64);
+	DeviceAttribute(const char *, DevState);
+		
 	DeviceAttribute(const char *, vector<short> &);
-	DeviceAttribute(const char *, vector<long> &);
+	DeviceAttribute(const char *, vector<DevLong> &);
 	DeviceAttribute(const char *, vector<double> &);
 	DeviceAttribute(const char *, vector<string> &);
 	DeviceAttribute(const char *, vector<float> &);
 	DeviceAttribute(const char *, vector<bool> &);
 	DeviceAttribute(const char *, vector<unsigned short> &);
 	DeviceAttribute(const char *, vector<unsigned char> &);
-	
+	DeviceAttribute(const char *, vector<DevLong64> &);
+	DeviceAttribute(const char *, vector<DevULong> &);
+	DeviceAttribute(const char *, vector<DevULong64> &);
+	DeviceAttribute(const char *, vector<DevState> &);
+		
 	DeviceAttribute(const char *, vector<short> &,int,int);
-	DeviceAttribute(const char *, vector<long> &,int,int);
+	DeviceAttribute(const char *, vector<DevLong> &,int,int);
 	DeviceAttribute(const char *, vector<double> &,int,int);
 	DeviceAttribute(const char *, vector<string> &,int,int);
 	DeviceAttribute(const char *, vector<float> &,int,int);
 	DeviceAttribute(const char *, vector<bool> &,int,int);
 	DeviceAttribute(const char *, vector<unsigned short> &,int,int);
-	DeviceAttribute(const char *, vector<unsigned char> &,int,int);		
-	
+	DeviceAttribute(const char *, vector<unsigned char> &,int,int);
+	DeviceAttribute(const char *, vector<DevLong64> &,int,int);
+	DeviceAttribute(const char *, vector<DevULong> &,int,int);
+	DeviceAttribute(const char *, vector<DevULong64> &,int,int);
+	DeviceAttribute(const char *, vector<DevState> &,int,int);
+		
 	~DeviceAttribute();
 	
 	AttrQuality 		quality;
-	string 			name;
-	int 			dim_x;
-	int 			dim_y;
-	TimeVal 		time;
+	string 				name;
+	int 				dim_x;
+	int 				dim_y;
+	TimeVal 			time;
 	DeviceAttributeExt	*ext;		// Class extension
 		
 	string &get_name() {return name;}
 	AttrQuality &get_quality() {return quality;}
+	
 	int get_dim_x() {return dim_x;}
 	int get_dim_y() {return dim_y;}
-	TimeVal &get_date() {return time;}
+	int get_written_dim_x() {return ext->w_dim_x;}
+	int get_written_dim_y() {return ext->w_dim_y;}
 	AttributeDimension get_r_dimension();
 	AttributeDimension get_w_dimension();
 	long get_nb_read();
 	long get_nb_written();
-	int get_type();
 	
+	int get_type();
+	TimeVal &get_date() {return time;}	
 	void set_name(string &na) {name =  na;}
 	void set_name(const char *na) {string str(na);name = str;}
 	bool is_empty();
@@ -543,66 +615,114 @@ public :
 	DevVarFloatArray_var	FloatSeq;
 	DevVarBooleanArray_var	BooleanSeq;
 	DevVarUShortArray_var	UShortSeq;
-	DevVarCharArray_var	UCharSeq;
+	DevVarCharArray_var		UCharSeq;
 	
-	DevState		d_state;
-	bool			d_state_filled;
+//
+// For the state attribute
+//
+
+	DevState				d_state;
+	bool					d_state_filled;
 
 //
-// insert methods for native C++ types
+// insert methods for  C++ types
 //
 
 	void operator << (short);
-	void operator << (long);
+	void operator << (DevLong);
 	void operator << (double);
 	void operator << (string &);
 	void operator << (float);
 	void operator << (bool);
 	void operator << (unsigned short);
 	void operator << (unsigned char);
-	
+	void operator << (DevLong64);
+	void operator << (DevULong);
+	void operator << (DevULong64);
+	void operator << (DevState);
+		
 	void operator << (vector<short> &);
-	void operator << (vector<long> &);
+	void operator << (vector<DevLong> &);
 	void operator << (vector<double> &);
 	void operator << (vector<string> &);
 	void operator << (vector<float> &);
 	void operator << (vector<bool> &);
 	void operator << (vector<unsigned short> &);
 	void operator << (vector<unsigned char> &);
-	
+	void operator << (vector<DevLong64> &);
+	void operator << (vector<DevULong> &);
+	void operator << (vector<DevULong64> &);
+	void operator << (vector<DevState> &);
+		
 	void insert(vector<short> &,int,int);
-	void insert(vector<long> &,int,int);
+	void insert(vector<DevLong> &,int,int);
 	void insert(vector<double> &,int,int);
 	void insert(vector<string> &,int,int);
 	void insert(vector<float> &,int,int);
 	void insert(vector<bool> &,int,int);
 	void insert(vector<unsigned short> &,int,int);
 	void insert(vector<unsigned char> &,int,int);
-			
+	void insert(vector<DevLong64> &,int,int);
+	void insert(vector<DevULong> &,int,int);
+	void insert(vector<DevULong64> &,int,int);
+	void insert(vector<DevState> &,int,int);
+				
 //
-// extract methods for native C++ types
+// extract methods for  C++ types
 //
 
-	bool operator >> (DevState &);
-	
 	bool operator >> (short &);
-	bool operator >> (long &);
+	bool operator >> (DevLong &);
 	bool operator >> (double &);
 	bool operator >> (string&);
 	bool operator >> (float &);
 	bool operator >> (bool &);
 	bool operator >> (unsigned short &);
-	bool operator >> (unsigned char &);	
-
+	bool operator >> (unsigned char &);
+	bool operator >> (DevLong64 &);
+	bool operator >> (DevULong &);
+	bool operator >> (DevULong64 &);
+	bool operator >> (DevState &);
+	
 	bool operator >> (vector<string>&);
 	bool operator >> (vector<short>&);
-	bool operator >> (vector<long>&);
+	bool operator >> (vector<DevLong>&);
 	bool operator >> (vector<double>&);
 	bool operator >> (vector<float>&);
 	bool operator >> (vector<bool>&);
 	bool operator >> (vector<unsigned short>&);
 	bool operator >> (vector<unsigned char>&);
-		
+	bool operator >> (vector<DevLong64>&);
+	bool operator >> (vector<DevULong>&);
+	bool operator >> (vector<DevULong64>&);
+	bool operator >> (vector<DevState>&);
+	
+	
+	bool extract_read (vector<string>&);
+	bool extract_set  (vector<string>&);
+	bool extract_read (vector<short>&);
+	bool extract_set  (vector<short>&);
+	bool extract_read (vector<DevLong>&);
+	bool extract_set  (vector<DevLong>&);  
+	bool extract_read (vector<double>&);
+	bool extract_set  (vector<double>&);
+	bool extract_read (vector<float>&);
+	bool extract_set  (vector<float>&);    
+	bool extract_read (vector<bool>&);
+	bool extract_set  (vector<bool>&); 
+	bool extract_read (vector<unsigned short>&);
+	bool extract_set  (vector<unsigned short>&);
+	bool extract_read (vector<unsigned char>&);
+	bool extract_set  (vector<unsigned char>&);
+	bool extract_read (vector<DevLong64>&);
+	bool extract_set  (vector<DevLong64>&);
+	bool extract_read (vector<DevULong>&);
+	bool extract_set  (vector<DevULong>&);
+	bool extract_read (vector<DevULong64>&);
+	bool extract_set  (vector<DevULong64>&);
+	bool extract_read (vector<DevState>&);
+	bool extract_set  (vector<DevState>&);
+	
 //
 // extract methods for TANGO CORBA sequence types
 //
@@ -615,7 +735,11 @@ public :
 	bool operator >> (DevVarBooleanArray* &datum);
 	bool operator >> (DevVarUShortArray* &datum);
 	bool operator >> (DevVarCharArray* &datum);
-	
+	bool operator >> (DevVarLong64Array * &datum);
+	bool operator >> (DevVarULongArray * &datum);
+	bool operator >> (DevVarULong64Array * &datum);
+	bool operator >> (DevVarStateArray * &datum);
+		
 //
 // insert methods for TANGO CORBA sequence types by reference
 //
@@ -628,7 +752,11 @@ public :
 	void operator << (const DevVarBooleanArray &datum);
 	void operator << (const DevVarUShortArray &datum);
 	void operator << (const DevVarCharArray &datum);
-	
+	void operator << (const DevVarLong64Array &datum);
+	void operator << (const DevVarULongArray &datum);
+	void operator << (const DevVarULong64Array &datum);
+	void operator << (const DevVarStateArray &datum);
+		
 	void operator << (DevVarShortArray *datum);
 	void operator << (DevVarLongArray *datum);
 	void operator << (DevVarDoubleArray *datum);
@@ -637,7 +765,11 @@ public :
 	void operator << (DevVarBooleanArray *datum);
 	void operator << (DevVarUShortArray *datum);
 	void operator << (DevVarCharArray *datum);
-	
+	void operator << (DevVarLong64Array *datum);
+	void operator << (DevVarULongArray *datum);
+	void operator << (DevVarULong64Array *datum);
+	void operator << (DevVarStateArray *datum);
+		
 	void insert(const DevVarShortArray &datum,int,int);
 	void insert(const DevVarLongArray &datum,int,int);
 	void insert(const DevVarDoubleArray &datum,int,int);
@@ -646,7 +778,11 @@ public :
 	void insert(const DevVarBooleanArray &datum,int,int);
 	void insert(const DevVarUShortArray &datum,int,int);
 	void insert(const DevVarCharArray &datum,int,int);
-	
+	void insert(const DevVarLong64Array &datum,int,int);
+	void insert(const DevVarULongArray &datum,int,int);
+	void insert(const DevVarULong64Array &datum,int,int);
+	void insert(const DevVarStateArray &datum,int,int);
+		
 	void insert(DevVarShortArray *datum,int,int);
 	void insert(DevVarLongArray *datum,int,int);
 	void insert(DevVarDoubleArray *datum,int,int);
@@ -655,14 +791,30 @@ public :
 	void insert(DevVarBooleanArray *datum,int,int);
 	void insert(DevVarUShortArray *datum,int,int);
 	void insert(DevVarCharArray *datum,int,int);
-				
+	void insert(DevVarLong64Array *datum,int,int);
+	void insert(DevVarULongArray *datum,int,int);
+	void insert(DevVarULong64Array *datum,int,int);
+	void insert(DevVarStateArray *datum,int,int);
+					
 	friend ostream &operator<<(ostream &,DeviceAttribute &);
 
 protected :	
 	bitset<numFlags> 	exceptions_flags;
+	void del_mem(int);
+	bool check_for_data();
+	bool check_wrong_type_exception();
+	int  check_set_value_size(int seq_length);
 	
 };
 
+
+/****************************************************************************************
+ * 																						*
+ * 					The xxxDataHistory classes											*
+ * 					--------------------------											*
+ * 																						*
+ ***************************************************************************************/
+ 
 class DeviceDataHistory: public DeviceData
 {
 
@@ -689,12 +841,12 @@ public :
 	const DevErrorList &errors() {return err.in();}	
 	
 protected:
-	bool 			fail;
-	TimeVal 		time;
+	bool 				fail;
+	TimeVal 			time;
 	DevErrorList_var 	err;
 		
 	DevCmdHistoryList 	*seq_ptr;
-	int 			*ref_ctr_ptr;
+	int 				*ref_ctr_ptr;
 
 private:	
 	DeviceDataHistoryExt	*ext_hist;		// Class extension
@@ -735,38 +887,48 @@ private:
 	DeviceAttributeHistoryExt	*ext_hist;	// Class extension		
 };
 
+
+/****************************************************************************************
+ * 																						*
+ * 					The Connection class												*
+ * 					--------------------												*
+ * 																						*
+ ***************************************************************************************/
+ 
 class Connection 
 {
 protected :
-	bool 	dbase_used;			// Dev. with database
-	bool 	from_env_var;			// DB from TANGO_HOST
+	bool 				dbase_used;			// Dev. with database
+	bool 				from_env_var;		// DB from TANGO_HOST
 	
-	string 	host;				// DS host (if dbase_used=false)
-	string 	port;				// DS port (if dbase_used=false)
-	int 	port_num;			// DS port (as number)
+	string 				host;				// DS host (if dbase_used=false)
+	string 				port;				// DS port (if dbase_used=false)
+	int 				port_num;			// DS port (as number)
 	
-	string 	db_host;			// DB host
-	string 	db_port;			// DB port
-	int 	db_port_num;			// DB port (as number)
+	string 				db_host;			// DB host
+	string 				db_port;			// DB port
+	int 				db_port_num;		// DB port (as number)
 	
-	string 	ior;
-	long 	pasyn_ctr;
-	long	pasyn_cb_ctr;
+	string 				ior;
+	long 				pasyn_ctr;
+	long				pasyn_cb_ctr;
 		
-	Tango::Device_var device;
+	Tango::Device_var 	device;
 	Tango::Device_2_var device_2;
 	
-	int 	timeout;
-	int 	timeout_changed;
+	int 				timeout;
+	int 				timeout_changed;
 	
-	int connection_state;
-	int version;
-	Tango::DevSource source;
+	int 				connection_state;
+	int 				version;
+	Tango::DevSource 	source;
 
 	virtual string get_corba_name()=0;
 	virtual string build_corba_name()=0;
 
-	ConnectionExt			*ext; 	// Class extension
+	ConnectionExt		*ext; 	// Class extension
+	
+	DeviceData redo_synch_cmd(TgRequest &);
 		
 public :
 	virtual string dev_name()=0;
@@ -829,10 +991,12 @@ public :
 	
 };
 
-//
-// DeviceProxy - device proxy object for implementing generic high-level
-//               interface for TANGO device objects
-//
+/****************************************************************************************
+ * 																						*
+ * 					The DeviceProxy class												*
+ * 					--------------------												*
+ * 																						*
+ ***************************************************************************************/
 
 class DeviceProxy: public Tango::Connection
 {
@@ -848,6 +1012,12 @@ private :
 
 	void connect_to_adm_device();
 	void polling_cmd_except(Tango::DevFailed *,string &,const char *);
+	
+	void retrieve_read_args(TgRequest &,vector<string> &);
+	DeviceAttribute *redo_synch_read_call(TgRequest &);
+	vector<DeviceAttribute> *redo_synch_reads_call(TgRequest &);
+	void redo_synch_write_call(TgRequest &);
+	void write_attribute(const AttributeValueList &);
 
 	enum read_attr_type
 	{
@@ -945,6 +1115,7 @@ public :
 	
 	void write_attribute(DeviceAttribute&);
 	void write_attributes(vector<DeviceAttribute>&);
+	
 //
 // history methods
 //
@@ -1031,34 +1202,40 @@ public :
 //
 // Event methods
 //
-	int subscribe_event(const string &attr_name, EventType event, CallBack *, const vector<string> &filters);
+	int subscribe_event(const string &attr_name, EventType event, CallBack *, 
+	                   const vector<string> &filters);
+	int subscribe_event(const string &attr_name, EventType event, CallBack *, 
+	                   const vector<string> &filters, bool stateless);
+							 
 	void unsubscribe_event(int event_id);
 };
 
-//
-// AttributeProxy - attribute proxy object for implementing high-level
-//                  interface for TANGO attributes
-//
+/****************************************************************************************
+ * 																						*
+ * 					The AttributeProxy class											*
+ * 					--------------------												*
+ * 																						*
+ ***************************************************************************************/
 
 class AttributeProxy
 {
 private :
-	string 			attr_name;
-	string 			device_name;
+	string 				attr_name;
+	string 				device_name;
 	Tango::DeviceProxy 	*dev_proxy;
 	Tango::DbAttribute 	*db_attr;
-        bool    		dbase_used;	// Dev. with database
-        bool    		from_env_var;   // DB from TANGO_HOST
+	bool    			dbase_used;		// Dev. with database
+	bool    			from_env_var;   // DB from TANGO_HOST
 
-        string  		host;           // DS host (if dbase_used=false)
-        string  		port;           // DS port (if dbase_used=false)
-        int     		port_num;       // DS port (as number)
+	string  			host;           // DS host (if dbase_used=false)
+	string  			port;           // DS port (if dbase_used=false)
+	int     			port_num;       // DS port (as number)
 
-        string  		db_host;        // DB host
-        string  		db_port;        // DB port
-        int     		db_port_num;    // DB port (as number)
+	string  			db_host;        // DB host
+	string  			db_port;        // DB port
+	int     			db_port_num;    // DB port (as number)
 
-	AttributeProxyExt	*ext;     	// Class extension
+	AttributeProxyExt	*ext;     		// Class extension
 
 	void real_constructor(string &);
 	void ctor_from_dp(const DeviceProxy *,string &);
@@ -1141,10 +1318,30 @@ public :
 // Event methods
 //
 
-	int subscribe_event(EventType event, CallBack *, const vector<string> &filters);
-	void unsubscribe_event(int ev_id) {dev_proxy->unsubscribe_event(ev_id);}
+	int subscribe_event (EventType event, CallBack *, 
+								const vector<string> &filters);
+	int subscribe_event (EventType event, CallBack *, 
+	                    	const vector<string> &filters, bool stateless);
+														  
+	void unsubscribe_event (int ev_id) {dev_proxy->unsubscribe_event(ev_id);}
 };
 
+
+
+// 					Small utility classes
+//					---------------------
+ 
+
+class AutoConnectTimeout
+{
+public:
+	AutoConnectTimeout(unsigned int to) {omniORB::setClientConnectTimeout((CORBA::ULong)to);}
+	~AutoConnectTimeout() {omniORB::setClientConnectTimeout(0);}
+};
+
+
+
+
 //
 // minor is also defined (Linux) in sysmacros.h. We want the compiler to
 // use the SystemException::minor() method !!
@@ -1153,13 +1350,6 @@ public :
 #ifdef minor
 #undef minor
 #endif
-	
-/*
-#ifdef WIN32 \
-		if (timeout_changed == 1) \
-			connection_state = CONNECTION_NOTOK; \
-#endif \
-*/
 
 //
 // Some extension classes
