@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: devapi_data.cpp,v 3.10 2007/03/06 08:20:45 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: devapi_data.cpp,v 3.12 2008/03/25 15:47:25 taurel Exp $\n$Name:  $";
 
 //
 // devapi_data.cpp 	- C++ source code file for TANGO devapi class DeviceData
@@ -8,6 +8,12 @@ static const char *RcsId = "$Id: devapi_data.cpp,v 3.10 2007/03/06 08:20:45 taur
 // original 		- March 2001
 //
 // log			- $Log: devapi_data.cpp,v $
+// log			- Revision 3.12  2008/03/25 15:47:25  taurel
+// log			- - Fix some warnings for gcc 4.2 (Thank's to F Picca from Soleil)
+// log			-
+// log			- Revision 3.11  2008/03/11 14:36:44  taurel
+// log			- - Apply patches from Frederic Picca about compilation with gcc 4.2
+// log			-
 // log			- Revision 3.10  2007/03/06 08:20:45  taurel
 // log			- - Added 64 bits data types for 64 bits computer...
 // log			-
@@ -386,6 +392,9 @@ int DeviceData::get_type()
 			case CORBA::tk_string:
 				data_type = Tango::DEVVAR_STRINGARRAY;
 				break;
+				
+			default:
+				break;
 			}
 			break;
 			
@@ -402,6 +411,9 @@ int DeviceData::get_type()
 			case CORBA::tk_double:
 				data_type = Tango::DEVVAR_DOUBLESTRINGARRAY;
 				break;
+
+			default:
+				break;
 			}
 			break;
 			
@@ -409,6 +421,9 @@ int DeviceData::get_type()
 			data_type = Tango::DEV_STATE;
 			break;
 			
+		default:
+			break;
+			
 		}
 		
 		CORBA::release(tc);
@@ -1914,6 +1929,9 @@ ostream &operator<<(ostream &o_str,DeviceData &dd)
 				dd.any.inout() >>= str_arr;
 				o_str << *str_arr;
 				break;
+				
+			default:
+				break;
 			}
 			break;
 			
@@ -1936,6 +1954,9 @@ ostream &operator<<(ostream &o_str,DeviceData &dd)
 				o_str << dbstr_arr->dvalue << endl;
 				o_str << dbstr_arr->svalue;
 				break;
+				
+			default:
+				break;
 			}
 			break;
 			
@@ -1945,6 +1966,9 @@ ostream &operator<<(ostream &o_str,DeviceData &dd)
 			o_str << Tango::DevStateName[tmp_state];
 			break;
 			
+		default:
+			break;
+			
 		}
 		
 		CORBA::release(tc);
