@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: devapi_datahist.cpp,v 3.10 2007/03/06 08:20:45 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: devapi_datahist.cpp,v 3.11 2008/03/11 14:36:44 taurel Exp $\n$Name:  $";
 
 //
 // devapi_datahist.cpp 	- C++ source code file for TANGO devapi class 
@@ -9,6 +9,9 @@ static const char *RcsId = "$Id: devapi_datahist.cpp,v 3.10 2007/03/06 08:20:45
 // original 		- June 2002
 //
 // $Log: devapi_datahist.cpp,v $
+// Revision 3.11  2008/03/11 14:36:44  taurel
+// - Apply patches from Frederic Picca about compilation with gcc 4.2
+//
 // Revision 3.10  2007/03/06 08:20:45  taurel
 // - Added 64 bits data types for 64 bits computer...
 //
@@ -457,6 +460,9 @@ DeviceAttributeHistory::DeviceAttributeHistory(int n,DevAttrHistoryList_var &seq
 			tmp_uch = (const_cast<DevVarCharArray *>(tmp_seq_uch))->get_buffer((CORBA::Boolean)true);
 			UCharSeq = new DevVarCharArray(max,len,tmp_uch,true);
 			break;
+			
+		default:
+			break;
 		}
 	}	
 
@@ -575,6 +581,9 @@ DeviceAttributeHistory::DeviceAttributeHistory(int n,DevAttrHistoryList_3_var &s
 			tmp_uch = (const_cast<DevVarCharArray *>(tmp_seq_uch))->get_buffer((CORBA::Boolean)true);
 			UCharSeq = new DevVarCharArray(max,len,tmp_uch,true);
 			break;
+			
+		default:
+			break;
 		}
 	}	
 
