@@ -8,6 +8,31 @@
 //
 // author(s) :          N.Leclercq
 //
+// $Revision: 3.13 $
+//
+// $Log: group.h,v $
+// Revision 3.13  2008/03/04 13:55:04  nleclercq
+// Fixed a pb in Group::next_req_id
+//
+// Revision 1.1  2008/02/28 10:36:10  leclercq
+// Added 5.5.2 patches
+//
+// Revision 3.12  2007/03/29 14:30:15  tiagocoutinho
+// - Bug fix by Nicolas Leclercq regarding groups with groups
+//
+// Revision 3.11  2007/03/06 08:20:45  taurel
+// - Added 64 bits data types for 64 bits computer...
+//
+// Revision 3.10  2006/10/24 13:11:41  bourtemb
+// Add the possibility to set the timeout for each DeviceProxy of the group.
+// Add set_timeout_millis() methods in Group, GroupElement and GroupDeviceElement
+// classes.
+// Add an optional timeout parameter to GroupElementFactory::instanciate() method
+// Add an optional timeout parameter to GroupElement::Add(), Group::Add() and
+// GroupDeviceElement methods.
+// Add a constructor to GroupDeviceElement class with a timeout parameter.
+//
+//
 // copyleft :           European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -245,7 +270,7 @@ public:
     return result;
   }
   //- data exctractor method for DevVarLongStringArray
-  bool extract (std::vector<long>& vl, std::vector<std::string>& vs);
+  bool extract (std::vector<DevLong>& vl, std::vector<std::string>& vs);
   //- data exctractor method for DevVarDoubleStringArray
   bool extract (std::vector<double>& vd, std::vector<std::string>& vs);
 private:
@@ -459,8 +484,9 @@ class GroupElementFactory
 
   friend class Tango::Group;
 
-  //- instanciatethe GroupElement which name matches the specified pattern
-  static GroupElements instanciate (const std::string& p);
+  //- instanciatethe GroupElement which name matches the specified pattern with the specified timeout
+  //- timeout = -1 => do not change the timeout
+  static GroupElements instanciate (const std::string& p, int timeout = -1);
 
   //- forbidden methods
   GroupElementFactory();
@@ -477,10 +503,10 @@ public:
   //- Group management methods 
   //---------------------------------------------
   //- 
-  virtual void add (const std::string& s);
-  //-
-  virtual void add (const std::vector<std::string>& sl);
+  virtual void add (const std::string& s, int timeout_ms = -1);
   //-
+  virtual void add (const std::vector<std::string>& sl, int timeout_ms = -1);
+  //- 
   virtual void remove (const std::string& s, bool forward = true);
   //-
   virtual void remove (const std::vector<std::string>& sl, bool forward = true);
@@ -501,6 +527,8 @@ public:
   //- 
   virtual bool ping (bool forward = true) = 0;
   //-
+  virtual void set_timeout_millis (int timeout_ms) = 0;
+  //-
   virtual long command_inout_asynch (const std::string& c, bool forget = false, bool forward = true, long reserved = -1) = 0;
   //- 
   virtual long command_inout_asynch (const std::string& c,  const DeviceData& d, bool forget = false, bool forward = true, long reserved = -1) = 0;
@@ -622,12 +650,12 @@ public:
 
   //- Group management methods 
   //---------------------------------------------
-  //- 
-  virtual void add (Group* group);
   //-
-  virtual void add (const std::string& pattern);
+  virtual void add (Group* group, int timeout_ms = -1);
   //-
-  virtual void add (const std::vector<std::string>& patterns);
+  virtual void add (const std::string& pattern, int timeout_ms = -1);
+  //-
+  virtual void add (const std::vector<std::string>& patterns, int timeout_ms = -1);
   //-
   virtual void remove (const std::string& pattern, bool forward = true);
   //-
@@ -653,6 +681,8 @@ public:
   //- 
   virtual bool ping (bool forward = true);
   //-
+  virtual void set_timeout_millis (int timeout_ms);
+  //-
   GroupCmdReplyList command_inout (const std::string& c, bool forward = true);
   //-
   GroupCmdReplyList command_inout (const std::string& c, const DeviceData& d, bool forward = true);
@@ -673,7 +703,7 @@ public:
       omni_mutex_lock guard(elements_mutex);
     #endif
     long gsize = get_size_i(forward);
-    if (gsize != d.size()) {
+    if (gsize != static_cast<long>(d.size())) {
       TangoSys_OMemStream desc;
 		  desc << "the size of the input argument list must equal the number of device in the group" 
            << " [expected:" 
@@ -690,21 +720,17 @@ public:
       reserved = next_req_id();
     }
     Tango::DeviceData dd;
-    if (forward) 
-    {
-      GroupElements te = get_hiearchy();
-      for (unsigned int i = 0; i < te.size(); i++) {
-        dd << d[i];
-        te[i]->command_inout_asynch(c, dd, forget, false, reserved);
+    for (unsigned int i = 0, j = 0; i < elements.size(); i++) {
+      if (elements[i]->is_device()) {
+        dd << d[j++];
+        elements[i]->command_inout_asynch(c, dd, forget, false, reserved);
       }
-    } 
-    else 
-    {
-      for (unsigned int i = 0, j = 0; i < elements.size(); i++) {
-        if (elements[i]->is_device()) {
-          dd << d[j++];
-          elements[i]->command_inout_asynch(c, dd, forget, false, reserved);
-        }
+      else if (forward) {
+        Tango::Group * g = reinterpret_cast<Tango::Group*>(elements[i]);
+        long gsize = g->get_size_i(forward);
+        std::vector<T> sub_d(d.begin() + j,  d.begin() + j + gsize);
+        reinterpret_cast<Tango::Group*>(elements[i])->command_inout_asynch(c, sub_d, forget, false, reserved);
+        j += gsize;
       }
     }
     if (forget == false) {
@@ -744,7 +770,7 @@ public:
     #endif
     GroupReplyList rl;
     long gsize = get_size_i(forward);
-    if (gsize != d.size()) {
+    if (gsize != static_cast<long>(d.size())) {
       TangoSys_OMemStream desc;
 		  desc << "the size of the input argument list must equal the number of device in the group" 
            << " [expected:" 
@@ -761,21 +787,17 @@ public:
       reserved = next_req_id();
     }
     DeviceAttribute da(const_cast<string&>(a),  0.);
-    if (forward) 
-    {
-      GroupElements te = get_hiearchy();
-      for (unsigned int i = 0; i < te.size(); i++) {
-        da << d[i];
-        te[i]->write_attribute_asynch(da, false, reserved);
+    for (unsigned int i = 0, j = 0; i < elements.size(); i++) {
+      if (elements[i]->is_device()) {
+       	da << d[j++];
+        elements[i]->write_attribute_asynch(da, false, reserved);
       }
-    } 
-    else 
-    {
-      for (unsigned int i = 0, j = 0; i < elements.size(); i++) {
-        if (elements[i]->is_device()) {
-          da << d[j++];
-          elements[i]->write_attribute_asynch(da, false, reserved);
-        }
+      else if (forward) {
+        Tango::Group * g = reinterpret_cast<Tango::Group*>(elements[i]);
+        long gsize = g->get_size_i(forward);
+        std::vector<T> sub_d(d.begin() + j,  d.begin() + j + gsize);
+        reinterpret_cast<Tango::Group*>(elements[i])->write_attribute_asynch(a, sub_d, false, reserved);
+        j += gsize;
       }
     }
     push_async_request(reserved, forward);
@@ -884,6 +906,8 @@ public:
   //-
   virtual long get_size (bool forward = true);
   //-
+  virtual void set_timeout_millis (int timeout);
+  //-
 
   //- Some of the following public methods should be protected but C++ 
   //- does not allow to execute a protected method using an instance of 
@@ -898,6 +922,8 @@ public:
 private:
   //- ctor: creates an GroupDeviceElement named <name>
   GroupDeviceElement (const std::string& name);
+  //- ctor: creates a GroupDeviceElement named <name> with timeout set to timeout_ms milliseconds
+  GroupDeviceElement (const std::string& name, int timeout_ms);
   //- dtor: release resources
   virtual ~GroupDeviceElement();
   //- build connection to the device (may throw DevFailed)
