@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: proxy_asyn.cpp,v 3.10 2007/04/20 14:38:33 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: proxy_asyn.cpp,v 3.12 2008/03/25 15:47:25 taurel Exp $\n$Name:  $";
 
 //
 // cpp 	- C++ source code file for TANGO api Connection and DeviceProxy
@@ -9,6 +9,12 @@ static const char *RcsId = "$Id: proxy_asyn.cpp,v 3.10 2007/04/20 14:38:33 taure
 // original 	- August 2002
 //
 // $Log: proxy_asyn.cpp,v $
+// Revision 3.12  2008/03/25 15:47:25  taurel
+// - Fix some warnings for gcc 4.2 (Thank's to F Picca from Soleil)
+//
+// Revision 3.11  2008/03/11 14:36:45  taurel
+// - Apply patches from Frederic Picca about compilation with gcc 4.2
+//
 // Revision 3.10  2007/04/20 14:38:33  taurel
 // - Ported to Windows 64 bits x64 architecture
 //
@@ -259,6 +265,7 @@ long Connection::command_inout_asynch(string &command,bool faf)
 DeviceData Connection::command_inout_reply(long id)
 {
 
+	DeviceData data_out;
 //
 // Retrieve request object
 //
@@ -290,186 +297,182 @@ DeviceData Connection::command_inout_reply(long id)
 						       desc.str(),
 						       (const char *)"Connection::command_inout_reply");
 	}
-	else
-	{
-			
-		DeviceData data_out;
+
 //
 // Check if the reply is an exception
 //
 
-		CORBA::Environment_ptr env = req.request->env();
-		if (!CORBA::is_nil(env) && (env->exception() == NULL))
-		{
-		
+	CORBA::Environment_ptr env = req.request->env();
+	if (!CORBA::is_nil(env) && (env->exception() == NULL))
+	{
+	
 //
 // Get received value
 //
 
-			const CORBA::Any *received;
-			CORBA::Any &dii_any = req.request->return_value();
-			dii_any >>= received;
-			CORBA::Any *server_any = new CORBA::Any(*received);
-			
-			data_out.any = server_any;			
-		}
-		else
-		{
+		const CORBA::Any *received;
+		CORBA::Any &dii_any = req.request->return_value();
+		dii_any >>= received;
+		CORBA::Any *server_any = new CORBA::Any(*received);
 		
+		data_out.any = server_any;			
+	}
+	else
+	{
+	
 //
 // Retrieve exception and re-throw it.
 //
-		
-			CORBA::Exception *ex_ptr = env->exception();
+	
+		CORBA::Exception *ex_ptr = env->exception();
 
 //
 // Special treatement for timeout exception (TRANSIENT with specific minor code)
 //
-			
-			CORBA::TRANSIENT *tra;
-			if ((tra = CORBA::TRANSIENT::_downcast(ex_ptr)) != NULL)
-			{
-				if (tra->minor() == omni::TRANSIENT_CallTimedout)
-				{		
-					CORBA::NVList_ptr req_arg = req.request->arguments();
-					const char *cmd;
-					CORBA::NamedValue_ptr nv = req_arg->item(0);
-					*(nv->value()) >>= cmd;
-					char *tmp = CORBA::string_dup(cmd);
+		
+		CORBA::TRANSIENT *tra;
+		if ((tra = CORBA::TRANSIENT::_downcast(ex_ptr)) != NULL)
+		{
+			if (tra->minor() == omni::TRANSIENT_CallTimedout)
+			{		
+				CORBA::NVList_ptr req_arg = req.request->arguments();
+				const char *cmd;
+				CORBA::NamedValue_ptr nv = req_arg->item(0);
+				*(nv->value()) >>= cmd;
+				char *tmp = CORBA::string_dup(cmd);
 
-					char *cb_excep_mess = Tango::Except::print_CORBA_SystemException(tra);
-	
-					TangoSys_OMemStream desc;
-					desc << "Timeout (" << timeout << " mS) exceeded on device " << dev_name();
-					desc << ", command " << tmp << ends;
-					CORBA::string_free(tmp);
-			
-					ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-					pasyn_ctr--;	
-					
-					ApiCommExcept::re_throw_exception(cb_excep_mess,
-						  (const char *)"API_DeviceTimedOut",
-						  desc.str(),
-						  (const char *)"Connection::command_inout_reply()");
-				}
+				char *cb_excep_mess = Tango::Except::print_CORBA_SystemException(tra);
+
+				TangoSys_OMemStream desc;
+				desc << "Timeout (" << timeout << " mS) exceeded on device " << dev_name();
+				desc << ", command " << tmp << ends;
+				CORBA::string_free(tmp);
+		
+				ApiUtil::instance()->get_pasyn_table()->remove_request(id);
+				pasyn_ctr--;	
+				
+				ApiCommExcept::re_throw_exception(cb_excep_mess,
+					  (const char *)"API_DeviceTimedOut",
+					  desc.str(),
+					  (const char *)"Connection::command_inout_reply()");
 			}
-			
-			
-			CORBA::UnknownUserException *unk_ex;
-			if ((unk_ex = CORBA::UnknownUserException::_downcast(ex_ptr)) != NULL)
-			{
-			
+		}
+		
+		
+		CORBA::UnknownUserException *unk_ex;
+		if ((unk_ex = CORBA::UnknownUserException::_downcast(ex_ptr)) != NULL)
+		{
+		
 //
 // It is a UserUnknownException exception. This means that the
 // server has sent a DevFailed exception
 //
 
-				const Tango::DevFailed *serv_ex;
-				unk_ex->exception() >>= serv_ex;
-				Tango::DevFailed ex(*serv_ex);
+			const Tango::DevFailed *serv_ex;
+			unk_ex->exception() >>= serv_ex;
+			Tango::DevFailed ex(*serv_ex);
 
-				CORBA::NVList_ptr req_arg = req.request->arguments();
-				const char *cmd;
-				CORBA::NamedValue_ptr nv = req_arg->item(0);
-				*(nv->value()) >>= cmd;
-				char *tmp = CORBA::string_dup(cmd);
+			CORBA::NVList_ptr req_arg = req.request->arguments();
+			const char *cmd;
+			CORBA::NamedValue_ptr nv = req_arg->item(0);
+			*(nv->value()) >>= cmd;
+			char *tmp = CORBA::string_dup(cmd);
 
-				TangoSys_OMemStream desc;
-				desc << "Failed to execute command_inout_asynch on device " << dev_name();
-				desc << ", command " << cmd << ends;
-				CORBA::string_free(tmp);
-			
-				ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-				pasyn_ctr--;
-			
-	               		Except::re_throw_exception(ex,
-						   (const char*)"API_CommandFailed",
-                        			   desc.str(),
-						   (const char*)"Connection::command_inout_reply()");
-					
-	
-			}
+			TangoSys_OMemStream desc;
+			desc << "Failed to execute command_inout_asynch on device " << dev_name();
+			desc << ", command " << cmd << ends;
+			CORBA::string_free(tmp);
+		
+			ApiUtil::instance()->get_pasyn_table()->remove_request(id);
+			pasyn_ctr--;
+		
+			Except::re_throw_exception(ex,
+					   (const char*)"API_CommandFailed",
+					   desc.str(),
+					   (const char*)"Connection::command_inout_reply()");
+				
 
-			CORBA::SystemException *sys_ex;
-			if ((sys_ex = CORBA::SystemException::_downcast(ex_ptr)) != NULL)
-			{
+		}
 
-				connection_state = CONNECTION_NOTOK;
-			
+		CORBA::SystemException *sys_ex;
+		if ((sys_ex = CORBA::SystemException::_downcast(ex_ptr)) != NULL)
+		{
+
+			connection_state = CONNECTION_NOTOK;
+		
 //
 // Re-throw nearly all CORBA system exceptions
 //
 
-				CORBA::NVList_ptr req_arg = req.request->arguments();
-				const char *cmd;
-				CORBA::NamedValue_ptr nv = req_arg->item(0);
-				*(nv->value()) >>= cmd;
-				char *tmp = CORBA::string_dup(cmd);
+			CORBA::NVList_ptr req_arg = req.request->arguments();
+			const char *cmd;
+			CORBA::NamedValue_ptr nv = req_arg->item(0);
+			*(nv->value()) >>= cmd;
+			char *tmp = CORBA::string_dup(cmd);
 
-				char *cb_excep_mess = Tango::Except::print_CORBA_SystemException(sys_ex);
-				
+			char *cb_excep_mess = Tango::Except::print_CORBA_SystemException(sys_ex);
+			
 //
 // Check if the exception was a connection exception
 // In this case, try to ping the device.
 // If successfull, just returns otherwise, throw the first exception
 //
 
-				string ex(cb_excep_mess);
-				string::size_type pos = ex.find("TRANSIENT_ConnectFailed");
-				if (pos != string::npos)
+			string ex(cb_excep_mess);
+			string::size_type pos = ex.find("TRANSIENT_ConnectFailed");
+			if (pos != string::npos)
+			{
+				try
 				{
-					try
-					{
-						DeviceProxy *dev_ptr = static_cast<DeviceProxy *>(this);
-						dev_ptr->ping();
-						CORBA::string_free(tmp);
+					DeviceProxy *dev_ptr = static_cast<DeviceProxy *>(this);
+					dev_ptr->ping();
+					CORBA::string_free(tmp);
 
 //
 // If we arrive here, this means we have to do a re-try with a synchronous
 // request
 //
 
-						DeviceData dd_out = redo_synch_cmd(req);
+					DeviceData dd_out = redo_synch_cmd(req);
 
 //
 // Remove request from request global table.
 //
 
-						ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-						pasyn_ctr--;
-					
-						return dd_out;
-						
-					}
-					catch (Tango::DevFailed &e) {}
-				}
-
-				TangoSys_OMemStream desc;
-				desc << "Failed to execute command_inout_asynch on device " << dev_name();
-				desc << ", command " << cmd << ends;
-				CORBA::string_free(tmp);
-			
-				ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-				pasyn_ctr--;
-			
-	               		ApiCommExcept::re_throw_exception(cb_excep_mess,
-						          (const char*)"API_CommunicationFailed",
-                        			          desc.str(),
-						          (const char*)"Connection::command_inout_reply()");
-					
-	
-			}		
-		}
+					ApiUtil::instance()->get_pasyn_table()->remove_request(id);
+					pasyn_ctr--;
 				
+					return dd_out;
+					
+				}
+				catch (Tango::DevFailed &e) {}
+			}
+
+			TangoSys_OMemStream desc;
+			desc << "Failed to execute command_inout_asynch on device " << dev_name();
+			desc << ", command " << cmd << ends;
+			CORBA::string_free(tmp);
+		
+			ApiUtil::instance()->get_pasyn_table()->remove_request(id);
+			pasyn_ctr--;
+		
+			ApiCommExcept::re_throw_exception(cb_excep_mess,
+						  (const char*)"API_CommunicationFailed",
+						  desc.str(),
+						  (const char*)"Connection::command_inout_reply()");
+				
+
+		}		
+	}
+			
 //
 // Remove request from request global table.
 //
 
-		ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-		pasyn_ctr--;
-					
-		return data_out;
-	}
+	ApiUtil::instance()->get_pasyn_table()->remove_request(id);
+	pasyn_ctr--;
+				
+	return data_out;
 }
 
 //-----------------------------------------------------------------------------
@@ -938,6 +941,8 @@ vector<DeviceAttribute> *DeviceProxy::read_attributes_reply(long id)
 		
 		return dev_attr;
 	}
+	
+	return NULL;
 }
 
 //-----------------------------------------------------------------------------
@@ -1082,7 +1087,7 @@ DeviceAttribute *DeviceProxy::read_attribute_reply(long id)
 		
 		return dev_attr;
 	}
-
+	return NULL;
 }
 
 
