@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: proxy_asyn_cb.cpp,v 3.6 2005/01/13 08:36:37 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: proxy_asyn_cb.cpp,v 3.8 2007/04/20 14:38:33 taurel Exp $\n$Name:  $";
 
 //
 // cpp 	- C++ source code file for TANGO api class and asynchronous callback
@@ -9,6 +9,15 @@ static const char *RcsId = "$Id: proxy_asyn_cb.cpp,v 3.6 2005/01/13 08:36:37 tau
 // original 	- January 2003
 //
 // $Log: proxy_asyn_cb.cpp,v $
+// Revision 3.8  2007/04/20 14:38:33  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
+// Revision 3.7  2007/04/16 14:55:16  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
 // Revision 3.6  2005/01/13 08:36:37  taurel
 // - Merge trunk with Release_5_0 from brach Release_5_branch
 //
@@ -104,7 +113,7 @@ static const char *RcsId = "$Id: proxy_asyn_cb.cpp,v 3.6 2005/01/13 08:36:37 tau
 #include <tango.h>
 
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	#include <sys/timeb.h>
 #else
 	#include <sys/time.h>
@@ -394,7 +403,7 @@ void Connection::Cb_Cmd_Request(CORBA::Request_ptr req,Tango::CallBack *cb_ptr)
 				errors[0].reason = CORBA::string_dup("API_CorbaException");
 				errors[0].origin = CORBA::string_dup("Connection::Cb_Cmd_Request()");
 
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 				string st = desc.str();
 				errors[1].desc = CORBA::string_dup(st.c_str());
 #else
@@ -432,7 +441,7 @@ void Connection::Cb_Cmd_Request(CORBA::Request_ptr req,Tango::CallBack *cb_ptr)
 			errors.length(nb_err + 1);
 			errors[nb_err].severity = Tango::ERR;
 
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 			string st = desc.str();
 			errors[nb_err].desc = CORBA::string_dup(st.c_str());
 #else
@@ -468,7 +477,7 @@ void Connection::Cb_Cmd_Request(CORBA::Request_ptr req,Tango::CallBack *cb_ptr)
 			errors[0].reason = CORBA::string_dup("API_CorbaException");
 			errors[0].origin = CORBA::string_dup("Connection::Cb_Cmd_Request()");
 
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 			string st = desc.str();
 			errors[1].desc = CORBA::string_dup(st.c_str());
 #else
@@ -578,7 +587,7 @@ void Connection::Cb_ReadAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb_
 					(*dev_attr)[i].ext->err_list.inout().length(nb_except + 1);
 					(*dev_attr)[i].ext->err_list[nb_except].reason = CORBA::string_dup("API_AttributeFailed");
 					(*dev_attr)[i].ext->err_list[nb_except].origin = CORBA::string_dup("Connection::Cb_ReadAttr_Request");
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 					string st = desc.str();
 					(*dev_attr)[i].ext->err_list[nb_except].desc = CORBA::string_dup(st.c_str());
 #else
@@ -638,7 +647,7 @@ void Connection::Cb_ReadAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb_
 				errors[0].reason = CORBA::string_dup("API_CorbaException");
 				errors[0].origin = CORBA::string_dup("Connection::Cb_ReadAttr_Request()");
 
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 				string st = desc.str();
 				errors[1].desc = CORBA::string_dup(st.c_str());
 #else
@@ -680,7 +689,7 @@ void Connection::Cb_ReadAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb_
 			errors.length(nb_err + 1);
 			errors[nb_err].severity = Tango::ERR;
 
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 			string st = desc.str();
 			errors[nb_err].desc = CORBA::string_dup(st.c_str());
 #else
@@ -720,7 +729,7 @@ void Connection::Cb_ReadAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb_
 			errors[0].reason = CORBA::string_dup("API_CorbaException");
 			errors[0].origin = CORBA::string_dup("Connection::Cb_ReadAttr_Request()");
 
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 			string st = desc.str();
 			errors[1].desc = CORBA::string_dup(st.c_str());
 #else
@@ -812,7 +821,7 @@ void Connection::Cb_WriteAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb
 				err_3.errors[0].severity = Tango::ERR;
 				err_3.errors[0].reason = CORBA::string_dup("API_CorbaException");
 				err_3.errors[0].origin = CORBA::string_dup("Connection::Cb_WriteAttr_Request()");
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 				string st = desc.str();
 				err_3.errors[1].desc = CORBA::string_dup(st.c_str());
 #else
@@ -857,7 +866,7 @@ void Connection::Cb_WriteAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb
 
 				err_3.errors.length(1);
 				err_3.errors[0].severity = Tango::ERR;
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 				string st = desc.str();
 				err_3.errors[0].desc = CORBA::string_dup(st.c_str());
 #else
@@ -886,7 +895,7 @@ void Connection::Cb_WriteAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb
 			long nb_err = errors.length();
 			errors.length(nb_err + 1);
 			errors[nb_err].severity = Tango::ERR;
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 			string st = desc.str();
 			errors[nb_err].desc = CORBA::string_dup(st.c_str());
 #else
@@ -925,7 +934,7 @@ void Connection::Cb_WriteAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb
 			err_3.errors[0].severity = Tango::ERR;
 			err_3.errors[0].reason = CORBA::string_dup("API_CorbaException");
 			err_3.errors[0].origin = CORBA::string_dup("Connection::Cb_WriteAttr_Request()");
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 			string st = desc.str();
 			err_3.errors[1].desc = CORBA::string_dup(st.c_str());
 #else
@@ -1017,7 +1026,7 @@ void Connection::get_asynch_replies(long call_timeout)
 //
 
 			long nb = call_timeout / 20;
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 			struct timespec to_wait,inter;		
 			to_wait.tv_sec = 0;
 			to_wait.tv_nsec = 20000000;
@@ -1025,7 +1034,7 @@ void Connection::get_asynch_replies(long call_timeout)
 				
 			while ((nb > 0) && (pasyn_cb_ctr != 0))
 			{
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 				Sleep(20);
 #else		
 				nanosleep(&to_wait,&inter);
