@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: attrdesc.cpp,v 3.7 2006/04/27 13:44:51 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: attrdesc.cpp,v 3.9 2007/04/16 14:56:36 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -16,9 +16,18 @@ static const char *RcsId = "$Id: attrdesc.cpp,v 3.7 2006/04/27 13:44:51 taurel E
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.7 $
+// $Revision: 3.9 $
 //
 // $Log: attrdesc.cpp,v $
+// Revision 3.9  2007/04/16 14:56:36  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
+// Revision 3.8  2007/03/06 08:18:03  taurel
+// - Added 64 bits data types for 64 bits computer...
+//
 // Revision 3.7  2006/04/27 13:44:51  taurel
 // - Fix compatibility bug introduced in previous release by moving new
 // class data members in their corresponding extension classes
@@ -291,6 +300,8 @@ void Attr::check_type()
 		unsuported = false;
 	else if (type == Tango::DEV_LONG)
 		unsuported = false;
+	else if (type == Tango::DEV_LONG64)
+		unsuported = false;
 	else if (type == Tango::DEV_DOUBLE)
 		unsuported = false;
 	else if (type == Tango::DEV_STRING)
@@ -303,6 +314,12 @@ void Attr::check_type()
 		unsuported = false;
 	else if (type == Tango::DEV_UCHAR)
 		unsuported = false;
+	else if (type == Tango::DEV_ULONG)
+		unsuported = false;
+	else if (type == Tango::DEV_ULONG64)
+		unsuported = false;
+	else if (type == Tango::DEV_STATE)
+		unsuported = false;
 
 	if (unsuported == true)
 	{
