@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: blackbox.cpp,v 3.11 2007/04/20 14:40:24 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: blackbox.cpp,v 3.13 2008/03/25 15:00:22 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -16,9 +16,16 @@ static const char *RcsId = "$Id: blackbox.cpp,v 3.11 2007/04/20 14:40:24 taurel
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.11 $
+// $Revision: 3.13 $
 //
 // $Log: blackbox.cpp,v $
+// Revision 3.13  2008/03/25 15:00:22  taurel
+// - Small bug preventing Visual C++ to compile
+//
+// Revision 3.12  2008/03/14 11:53:52  taurel
+// - Add a check if the calling thread has been created by omni_thread
+// (For thread created by Python)
+//
 // Revision 3.11  2007/04/20 14:40:24  taurel
 // - Ported to Windows 64 bits x64 architecture
 //
@@ -709,7 +716,11 @@ void BlackBox::inc_indexes()
 
 void BlackBox::get_client_host()
 {
-	omni_thread::value_t *ip = omni_thread::self()->get_value(key);
+	omni_thread *th_id = omni_thread::self();
+	if (th_id == NULL)
+		th_id = omni_thread::create_dummy();
+	
+	omni_thread::value_t *ip = th_id->get_value(key);
 	if (ip == NULL)
 		strcpy(box[insert_elt].host_ip_str,"polling");
 	else
