@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: device_3.cpp,v 3.25 2007/06/05 15:52:29 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: device_3.cpp,v 3.29 2008/03/20 07:38:46 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -15,9 +15,24 @@ static const char *RcsId = "$Id: device_3.cpp,v 3.25 2007/06/05 15:52:29 taurel
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 3.25 $
+// $Revision: 3.29 $
 //
 // $Log: device_3.cpp,v $
+// Revision 3.29  2008/03/20 07:38:46  taurel
+// - Last commit before Release 6.1 ?
+//
+// Revision 3.28  2008/03/11 14:38:25  taurel
+// - Apply patches from Frederic Picca about compilation with gcc 4.2
+//
+// Revision 3.27  2008/02/06 15:51:55  jensmeyer
+// Changed the settings of the set value dimensions for WRITE attributes.
+// Now read and set dimensions are the same for write only attributes.
+//
+// Revision 3.26  2007/10/16 08:22:51  taurel
+// - Add management of the TC connection establishment timeout for DB access
+// - Add DB server cache in DS used during DS startup sequence
+// - Comment out the sleep time during DS startup sequence
+//
 // Revision 3.25  2007/06/05 15:52:29  taurel
 // - Clarify one error message
 //
@@ -946,8 +961,18 @@ void Device_3Impl::read_attributes_no_except(const Tango::DevVarStringArray& nam
 						}
 						else
 						{
-							(*back)[index].w_dim.dim_x = 0;
-							(*back)[index].w_dim.dim_y = 0;
+							if ( w_type == Tango::WRITE)
+							{
+								// for write only attributes read and set value are the same!
+								(*back)[index].w_dim.dim_x = att.get_x();
+								(*back)[index].w_dim.dim_y = att.get_y();
+							}
+							else
+							{
+								// Tango::Read : read only attributes
+								(*back)[index].w_dim.dim_x = 0;
+								(*back)[index].w_dim.dim_y = 0;
+							}
 						}
 					}
 					catch (Tango::DevFailed &e)
@@ -1049,7 +1074,7 @@ void Device_3Impl::read_attributes_from_cache(const Tango::DevVarStringArray& na
 
 	bool found;
 	vector<long> poll_period;
-	long not_polled_attr = 0;
+	unsigned long not_polled_attr = 0;
 	
 	if (non_polled.size() != 0)
 	{
@@ -1863,7 +1888,7 @@ throw(Tango::DevFailed, CORBA::SystemException)
 
 	polled_attr->get_attr_history(n,back,att.get_data_type());
 
-	cout4 << "Leaving Device_3Impl::command_inout_history_3 method" << endl;				
+	cout4 << "Leaving Device_3Impl::command_inout_history_3 method" << endl;
 	return back;
 }
 
@@ -1913,7 +1938,31 @@ throw (Tango::DevFailed, CORBA::SystemException)
 	back->dev_class = CORBA::string_dup(device_class->get_name().c_str());
 	back->server_id = CORBA::string_dup(tango_ptr->get_ds_name().c_str());
 	back->server_version = DevVersion;
-	back->doc_url = CORBA::string_dup(device_class->get_doc_url().c_str());
+	
+//
+// Build the complete info sent in the doc_url string
+//
+
+	string doc_url("Doc URL = ");
+	doc_url = doc_url + device_class->get_doc_url();
+	string &cvs_tag = device_class->get_cvs_tag();
+	if (cvs_tag.size() != 0)
+	{
+		doc_url = doc_url + "\nCVS Tag = ";
+		doc_url = doc_url + cvs_tag;
+	}
+	string &cvs_location = device_class->get_cvs_location();
+	if (cvs_location.size() != 0)
+	{
+		doc_url = doc_url + "\nCVS Location = ";
+		doc_url = doc_url + cvs_location;
+	}
+	back->doc_url = CORBA::string_dup(doc_url.c_str());
+	
+//
+// Set the device type
+//
+
 	back->dev_type = CORBA::string_dup(device_class->get_type().c_str());
 		
 //
