@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: deviceclass.cpp,v 1.21 2007/05/22 12:48:40 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: deviceclass.cpp,v 1.26 2008/03/11 14:38:25 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -13,9 +13,29 @@ static const char *RcsId = "$Id: deviceclass.cpp,v 1.21 2007/05/22 12:48:40 taur
 //
 // author(s) :		E.Taurel
 //
-// $Revision: 1.21 $
+// $Revision: 1.26 $
 //
 // $Log: deviceclass.cpp,v $
+// Revision 1.26  2008/03/11 14:38:25  taurel
+// - Apply patches from Frederic Picca about compilation with gcc 4.2
+//
+// Revision 1.25  2008/01/25 15:44:51  taurel
+// - Some changes in the Db cache
+// - A lighter system to shutdown DS in case of dynamic attribute
+//
+// Revision 1.24  2007/12/12 10:17:18  taurel
+// - Db calls during DS startup has a separate timeout and some retries
+//
+// Revision 1.23  2007/11/08 12:03:44  taurel
+// - Start implementing user interceptors
+// - Fix bug in poll thread pproperty management when removing polling object
+// - Set a database timeout to 6 sec
+//
+// Revision 1.22  2007/10/16 08:22:51  taurel
+// - Add management of the TC connection establishment timeout for DB access
+// - Add DB server cache in DS used during DS startup sequence
+// - Comment out the sleep time during DS startup sequence
+//
 // Revision 1.21  2007/05/22 12:48:40  taurel
 // - Clarified an error message
 //
@@ -217,30 +237,25 @@ void DeviceClass::get_class_system_resource()
 	Tango::Util *tg = Tango::Util::instance();
 	if (tg->_UseDb == true)
 	{
+		Database *db = tg->get_database();
 		DbData db_data;
 		
 		db_data.push_back(DbDatum("doc_url"));
 		db_data.push_back(DbDatum("cvs_tag"));
 		db_data.push_back(DbDatum("cvs_location"));
 
-		bool retry = true;
-		while (retry == true)
-		{		
-			try
-			{
-				db_class->get_property(db_data);
-				retry = false;
-			}
-			catch (Tango::CommunicationFailed &) {}
-			catch (Tango::DevFailed &)
-			{
-				TangoSys_OMemStream o;
-				o << "Database error while trying to retrieve properties for class " << name.c_str() << ends;
-					
-				Except::throw_exception((const char *)"API_DatabaseAccess",
-						o.str(),
-						(const char *)"DeviceClass::get_class_system_resource");
-			}
+		try
+		{
+			db->get_class_property(name,db_data,tg->get_db_cache());
+		}
+		catch (Tango::DevFailed &)
+		{
+			TangoSys_OMemStream o;
+			o << "Database error while trying to retrieve properties for class " << name.c_str() << ends;
+
+			Except::throw_exception((const char *)"API_DatabaseAccess",
+					o.str(),
+					(const char *)"DeviceClass::get_class_system_resource");
 		}
 		
 		if (db_data[1].is_empty() == false)
@@ -251,10 +266,9 @@ void DeviceClass::get_class_system_resource()
 		if (db_data[0].is_empty() == true)
 		{
 			cout4 << "doc_url property for class " << name << " is not defined in database" << endl;
-			Database *db = tg->get_database();
 			try
 			{
-				db->get_class_property("Default",db_data);
+				db->get_class_property("Default",db_data,tg->get_db_cache());
 			}
 			catch (Tango::DevFailed &)
 			{			
@@ -636,9 +650,39 @@ DeviceClass::~DeviceClass()
 		Tango::Util *tg = Tango::Util::instance();
 		PortableServer::POA_ptr r_poa = tg->get_poa();
 
-		for (i = 0;i < device_list.size();i++)
+		unsigned long nb_dev = device_list.size();
+		for (i = 0;i < nb_dev;i++)
 		{	
-			delete_dev(i,tg,r_poa);		
+				
+//
+// Clear vectors used to memorize info used to clean db
+// in case of devices with dyn attr removed during device
+// destruction
+//
+
+			tg->get_polled_dyn_attr_names().clear();
+			tg->get_full_polled_att_list().clear();
+			tg->get_all_dyn_attr_names().clear();
+			tg->get_dyn_att_dev_name().clear();
+
+//
+// Delete device
+//
+					
+			delete_dev(0,tg,r_poa);
+					
+//
+// Clean-up db (dyn attribute)
+//
+					
+
+			if (tg->get_polled_dyn_attr_names().size() != 0)
+				tg->clean_attr_polled_prop();			
+			if (tg->get_all_dyn_attr_names().size() != 0)
+				tg->clean_dyn_attr_prop();
+
+			vector<DeviceImpl *>::iterator it = device_list.begin();
+			device_list.erase(it);		
 		}
 		device_list.clear();
 		CORBA::release(r_poa);
@@ -968,6 +1012,8 @@ void DeviceClass::export_device(DeviceImpl *dev,const char *corba_obj_name)
 //
 
 		bool retry = true;
+		long db_retries = DB_START_PHASE_RETRIES;
+		
 		while (retry == true)
 		{
 			try
@@ -975,7 +1021,18 @@ void DeviceClass::export_device(DeviceImpl *dev,const char *corba_obj_name)
 				tg->get_database()->export_device(exp);
 				retry = false;
 			}
-			catch (Tango::CommunicationFailed &) {}
+			catch (Tango::CommunicationFailed &)
+			{
+				cerr << "CommunicationFailed while exporting device " << dev->get_name() << endl;
+				if (tg->is_svr_starting() == true)
+				{
+					db_retries--;
+					if (db_retries == 0)
+						throw;
+				}
+				else
+					throw;
+			}
 		}
 		
 		CORBA::release(orb_ptr);
@@ -1012,7 +1069,6 @@ CORBA::Any *DeviceClass::command_handler(DeviceImpl *device,string &command,cons
 	CORBA::Any *ret = NULL;
 	vector<Command *>::iterator i_cmd;
 	
-	bool with_default_cmd = false;
 	string command_lower(command);
 
 	cout4 << "Entering DeviceClass::command_handler() method" << endl;
