@@ -8,9 +8,24 @@
 //
 // author(s) :		E.Taurel
 //
-// $Revision: 1.7 $
+// $Revision: 1.11 $
 //
 // $Log: deviceclass.h,v $
+// Revision 1.11  2008/03/20 07:38:46  taurel
+// - Last commit before Release 6.1 ?
+//
+// Revision 1.10  2008/01/23 08:31:57  jensmeyer
+// Modified dllexport of the DeviceClass to avoid warnings when compiling
+// the Tango library.
+//
+// Revision 1.9  2007/08/22 13:00:48  jensmeyer
+// Added dllexport for the device class under windows to build a class DLL.
+//
+// Revision 1.8  2007/05/11 07:29:29  taurel
+// - Added device_destroyer() methods. These method are doing the opposite of
+// the device_factory() one. They remove (and delete) a device from the list
+// of available device in the class. They do not mmodify the DB
+//
 // Revision 1.7  2006/02/20 08:16:52  taurel
 // - Add a new method in DeviceClass to delete a device at run-time
 //
@@ -92,10 +107,16 @@ public:
  * device class
  *
  * $Author: taurel $
- * $Revision: 1.7 $
+ * $Revision: 1.11 $
  */
- 
-class DeviceClass
+
+class
+#ifdef _TG_WINDOWS_
+	#ifndef _TANGO_LIB
+		__declspec(dllexport)
+	#endif
+#endif
+		DeviceClass
 {
 	friend class Tango::AutoTangoMonitor;
 public:
@@ -155,7 +176,7 @@ public:
  * Create device(s).
  *
  * In the DeviceClass class, this method is pure abstract and must be defined
- * in sub-class. Its rule is to create all the calss devices and to store them
+ * in sub-class. Its rule is to create all the class devices and to store them
  * in a vector of device
  *
  * @param dev_list The device name list
@@ -178,6 +199,30 @@ public:
 	
 	virtual void device_name_factory(vector<string> &list) {};
 	
+/**
+ * Delete device.
+ *
+ * The rule of this method is to delete a device from the running
+ * device belonging to the Tango class. It does change anything in the
+ * database
+ *
+ * @param list Reference to the device name list
+ */	
+	
+	void device_destroyer(const string &);
+	
+/**
+ * Delete device.
+ *
+ * The rule of this method is to delete a device from the running
+ * device belonging to the Tango class. It does change anything in the
+ * database
+ *
+ * @param list Reference to the device name list
+ */	
+	
+	void device_destroyer(const char *);
+	
 //@}
 	
 /**@name Get/Set object members.
@@ -323,6 +368,9 @@ public:
 	string &get_cvs_tag() {return ext->cvs_tag;}
 	string &get_cvs_location() {return ext->cvs_location;}
 	
+	void set_cvs_tag(string &str) {ext->cvs_tag=str;}
+	void set_cvs_location(string &str) {ext->cvs_location=str;}
+	
 	void add_device(DeviceImpl *dev) {device_list.push_back(dev);}
 	void delete_dev(long idx,Tango::Util *tg,PortableServer::POA_ptr r_poa);
 
