@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dserver.cpp,v 3.23 2007/04/20 14:40:32 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dserver.cpp,v 3.29 2008/01/25 15:44:51 taurel Exp $\n$Name:  $";
 
 //+=============================================================================
 //
@@ -14,9 +14,33 @@ static const char *RcsId = "$Id: dserver.cpp,v 3.23 2007/04/20 14:40:32 taurel E
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.23 $
+// $Revision: 3.29 $
 //
 // $Log: dserver.cpp,v $
+// Revision 3.29  2008/01/25 15:44:51  taurel
+// - Some changes in the Db cache
+// - A lighter system to shutdown DS in case of dynamic attribute
+//
+// Revision 3.28  2008/01/18 07:58:12  taurel
+// - Protect the unregister_server call against any kind of exception
+//
+// Revision 3.27  2008/01/09 13:03:47  jensmeyer
+// Added a correct clean-up when exiting a device server which
+// is also used as an event client.
+// Added a EventConsumer::disconnect_from_notifd method as for
+// the EventSupplier.
+//
+// Revision 3.26  2007/12/19 15:42:43  taurel
+// - Add some cleanup when quitting applis or device server (notifd objects)
+//
+// Revision 3.25  2007/10/26 11:31:52  taurel
+// - Simply remove some log message sin source code
+//
+// Revision 3.24  2007/10/16 08:22:51  taurel
+// - Add management of the TC connection establishment timeout for DB access
+// - Add DB server cache in DS used during DS startup sequence
+// - Comment out the sleep time during DS startup sequence
+//
 // Revision 3.23  2007/04/20 14:40:32  taurel
 // - Ported to Windows 64 bits x64 architecture
 //
@@ -98,184 +122,6 @@ static const char *RcsId = "$Id: dserver.cpp,v 3.23 2007/04/20 14:40:32 taurel E
 // Revision 3.9.2.3  2004/09/27 09:09:07  taurel
 // - Changes to allow reading state and/or status as attributes
 //
-// Revision 3.9.2.2  2004/09/15 06:46:28  taurel
-// - Added four new types for attributes (boolean, float, unsigned short and unsigned char)
-// - It is also possible to read state and status as attributes
-// - Fix bug in Database::get_class_property() method (missing ends insertion)
-// - Fix bug in admin device DevRestart command (device name case problem)
-//
-// Revision 3.9.2.1  2004/08/19 07:44:04  taurel
-// - Replace server low level database access call by Database class method call
-// - Split device monitor in 3 : 1 to protect harware access, 1 to protect cache access and one mutex for device black box
-//
-// Revision 3.9  2004/07/07 08:40:11  taurel
-//
-// - Fisrt commit after merge between Trunk and release 4 branch
-// - Add EventData copy ctor, asiignement operator and dtor
-// - Add Database and DeviceProxy::get_alias() method
-// - Add AttributeProxy ctor from "device_alias/attribute_name"
-// - Exception thrown when subscribing two times for exactly yhe same event
-//
-// Revision 3.8  2004/06/15 12:38:22  taurel
-// - Last commit before merge with release 4 branch
-//
-// Revision 3.7  2003/09/02 13:08:14  taurel
-// Add memorized attribute feature (only for SCALAR and WRITE/READ_WRITE attribute)
-//
-// Revision 3.6  2003/07/03 07:40:51  taurel
-// - Change in Tango IDL file : Implement a new way to tranfer data for read_attribute and write_attribute CORBA operation
-// - Handle this new IDL release in DeviceProxy class
-// - New exception methods in DeviceAttribute class
-// - New way to get data out of DeviceAttribute object
-// - Fix bugs in DeviceProxy copy constructor and assignement operator
-// - Change some method names in DeviceDataHistory and DeviceAttributeHistory classes
-// - Change the implementation of the DeviceProxy::write_attribute() method to avoid DeviceAttribute copying
-// - Clean-up how a server is killed via a CTRL-C or a dserver device kill command
-// - Add a server_cleanup() method in the Util class
-// - Win32 : Update debug menu in the server graphical window to support logging feature
-// - Win32 : Display library CVS tag in the "Help->About" sub-window
-//
-// Revision 3.5.2.7  2004/03/09 16:36:36  taurel
-// - Added HP aCC port (thanks to Claudio from Elettra)
-// - Some last small bugs fixes
-//
-// Revision 3.5.2.6  2004/03/02 07:41:56  taurel
-// - Fix compiler warnings (gcc used with -Wall)
-// - Fix bug in DbDatum insertion operator fro vectors
-// - Now support "modulo" as periodic filter
-//
-// Revision 3.5.2.5  2004/02/18 15:06:17  taurel
-// Now the DevRestart command immediately restart device event (if any). Previously, it was possible to wait up to 200 secondes before they
-// restart
-//
-// Revision 3.5.2.4  2004/02/06 11:58:51  taurel
-// - Many changes in the event system
-//
-// Revision 3.5.2.3  2003/11/28 17:04:13  taurel
-// Changes to handle tango lib build as a Windows DLL and used in a device
-// server
-// - Modify how the device server is killed with the Dserver kill command
-//
-// Revision 3.5.2.1  2003/09/30 11:50:43  taurel
-// Add some changes foreseen for release 4.1 and already implemented on
-// the trunck into this release 4.0 branch
-//
-// Revision 3.5  2003/05/28 14:55:09  taurel
-// Add the include (conditionally) of the include files generated by autoconf
-//
-// Revision 3.4  2003/05/16 08:46:16  taurel
-// Many changes for release 3.0.1. The most important ones are :
-// - Timeout are backs
-// - Multiple db servers (change in TANGO_HOST syntax)
-// - Added methods to print DeviceData, DeviceDataHistory, DeviceAttribute and DeviceAttributeHistory instances
-// - Attributes name stored in blackbox
-// - Remove check if a class is created without any device
-// - It's now possible to create a DeviceProxy from its alias name
-// - Command, attribute names are case insensitive
-// - Change parameters of some DeviceProxy logging methods
-// - Change parameters of DeviceProxy asynchronous replies calls
-// - New serialization model in device server (no serialization model)
-// - Win32 (2000) device server service does not exit at loggoff anymore
-// - Miscellaneous bug fixes
-//
-// Revision 3.3  2003/04/24 10:18:09  nleclercq
-// Minor modifications to make some compilers happy
-//
-// Revision 3.2  2003/04/08 15:06:17  taurel
-// - Names of read/written attributes are now stored in blackbox
-// - Admin device RestartServer command is now done in its own thread with
-//   changing CORBA POA manager state
-//
-// Revision 3.1  2003/04/02 12:26:04  taurel
-// Miscellaneous changes to :
-//  - Modify TANGO_HOST env. variable syntax to be able to have several db servers
-//  - Asynchronous calls are now done on device_2 or device depending on the device IDL version
-//  - Timeout are bcks (omniORB 4.0.1)
-//  - Fix bug when killing a device server via the kill command of the admin device
-//    (Solaris only)
-//  - Cleanup device server code to have only one ORB and one db connection within a server
-//
-// Revision 3.0  2003/03/25 16:43:01  taurel
-// Many changes for Tango release 3.0 including
-// - Added full logging features
-// - Added asynchronous calls
-// - Host name of clients now stored in black-box
-// - Three serialization model in DS
-// - Fix miscellaneous bugs
-// - Ported to gcc 3.2
-// - Added ApiUtil::cleanup() and destructor methods
-// - Some internal cleanups
-// - Change the way how TangoMonitor class is implemented. It's a recursive
-//   mutex
-//
-// Revision 2.11  2003/03/11 17:55:52  nleclercq
-// Switch from log4cpp to log4tango
-//
-// Revision 2.10  2003/02/17 14:57:40  taurel
-// Added the new Tango logging stuff (Thanks Nicolas from Soleil)
-//
-// Revision 2.9  2003/01/09 12:03:16  taurel
-// - Ported to gcc 3.2
-// - Added ApiUtil::cleanup() and ApiUtil::~ApiUtil() methods
-// - Replace some ORB * by ORB_ptr
-// - Use CORBA::ORB::is_nil() instead of comparing to NULL
-//
-// Revision 2.8  2002/12/16 12:07:18  taurel
-// No change in code at all but only forgot th emost important line in
-// list of updates in the previous release :
-// - Change underlying ORB from ORBacus to omniORB
-//
-// Revision 2.7  2002/12/16 10:16:21  taurel
-// - New method get_device_list() in Util class
-// - Util::get_class_list takes DServer device into account
-// - Util::get_device_by_name() takes DServer device into account
-// - Util::get_device_list_by_class() takes DServer device into account
-// - New parameter to the attribute::set_value() method to enable CORBA to free
-// memory allocated for the attribute
-//
-// Revision 2.6  2002/10/17 07:43:06  taurel
-// Fix bug in history stored by the polling thread :
-// - We need one copy of the attribute data to build an history!!! It is true
-// also for command which return data created by the DeviceImpl::create_xxx
-// methods. Chnage in pollring.cpp/pollring.h/dserverpoll.cpp/pollobj.cpp
-// and pollobj.h
-//
-// Revision 2.5  2002/10/15 11:27:19  taurel
-// Fix bugs in device.cpp file :
-// - Protect the state and status CORBA attribute with the device monitor
-// Add the "TgLibVers" string as a #define in tango_config.h
-//
-// Revision 2.4  2002/08/12 15:06:54  taurel
-// Several big fixes and changes
-//   - Remove HP-UX specific code
-//   - Fix bug in polling alogorithm which cause the thread to enter an infinite
-//     loop (pollthread.cpp)
-//   - For bug for Win32 device when trying to set attribute config
-//     (attribute.cpp)
-//
-// Revision 2.3  2002/07/02 15:22:24  taurel
-// Miscellaneous small changes/bug fixes for Tango CPP release 2.1.0
-//     - classes reference documentation now generated using doxygen instead of doc++
-//     - A little file added to the library which summarizes version number.
-//       The RCS/CVS "ident" command will now tells you that release library x.y.z is composed
-//       by C++ client classes set release a.b and C++ server classes set release c.d
-//     - Fix incorrect field setting for DevFailed exception re-thrown from a CORBA exception
-//     - It's now not possible to poll the Init command
-//     - It's now possible to define a default class doc. per control system
-//       instance (using property)
-//     - The test done to check if attribute value has been set before it is
-//       returned to caller is done only if the attribute quality is set to VALID
-//     - The JTCInitialize object is now stored in the Util
-//     - Windows specific : The tango.h file now also include winsock.h
-//
-// Revision 2.2  2002/04/30 10:50:41  taurel
-// Don't check alarm on attribute if attribute quality factor is INVALID
-//
-// Revision 2.1  2002/04/29 12:24:03  taurel
-// Fix bug in attribute::set_value method and on the check against min and max value when writing attributes
-//
-// Revision 2.0  2002/04/09 14:45:10  taurel
-// See Tango WEB pages for list of changes
 //
 //
 // copyleft :           European Synchrotron Radiation Facility
@@ -289,6 +135,7 @@ static const char *RcsId = "$Id: dserver.cpp,v 3.23 2007/04/20 14:40:32 taurel E
 #endif
 
 #include <tango.h>
+#include <eventsupplier.h>
 
 #include <new>
 #include <algorithm>
@@ -444,7 +291,7 @@ void DServer::init_device()
 					Tango::DbDatum na;
 					try
 					{
-						na = db->get_device_name(tg->get_ds_name(),class_list[i]->get_name());
+						na = db->get_device_name(tg->get_ds_name(),class_list[i]->get_name(),tg->get_db_cache());
 					}
 					catch (Tango::DevFailed &)
 					{				
@@ -726,9 +573,39 @@ void DServer::delete_devices()
 				unsigned long loop;
 
 				vector<DeviceImpl *> &devs = class_list[i]->get_device_list();
-				for (loop = 0;loop < devs.size();loop++)
+				unsigned long nb_dev = devs.size();
+				for (loop = 0;loop < nb_dev;loop++)
 				{	
-					class_list[i]->delete_dev(loop,tg,r_poa);		
+				
+//
+// Clear vectors used to memorize info used to clean db
+// in case of devices with dyn attr removed durind device
+// destruction
+//
+
+					tg->get_polled_dyn_attr_names().clear();
+					tg->get_full_polled_att_list().clear();
+					tg->get_all_dyn_attr_names().clear();
+					tg->get_dyn_att_dev_name().clear();
+
+//
+// Delete device
+//
+					
+					class_list[i]->delete_dev(0,tg,r_poa);
+					
+//
+// Clean-up db (dyn attribute)
+//
+					
+
+					if (tg->get_polled_dyn_attr_names().size() != 0)
+						tg->clean_attr_polled_prop();			
+					if (tg->get_all_dyn_attr_names().size() != 0)
+						tg->clean_dyn_attr_prop();
+
+					vector<DeviceImpl *>::iterator it = devs.begin();
+					devs.erase(it);	
 				}
 				devs.clear();
 				CORBA::release(r_poa);
@@ -1362,11 +1239,17 @@ void *KillThread::run_undetached(void *ptr)
 	
 	omni_thread::self()->set_value(key_py_data,new PyData());
 
+//
+// Mark the server as shutting down
+//
+
+	Tango::Util *tg = Tango::Util::instance();
+	tg->set_svr_shutting_down(true);
+
 //
 //  Send kill command to the polling thread
 // 
 
-	Tango::Util *tg = Tango::Util::instance();
 	TangoMonitor &mon = tg->get_poll_monitor();
 	PollThCmd &shared_cmd = tg->get_poll_shared_cmd();
 
@@ -1386,7 +1269,11 @@ void *KillThread::run_undetached(void *ptr)
 // Unregister server device(s) from database
 //
 
-	tg->unregister_server();
+	try
+	{
+		tg->unregister_server();
+	}
+	catch(...) {}
 
 //
 // Delete devices (except the admin one)
@@ -1397,7 +1284,6 @@ void *KillThread::run_undetached(void *ptr)
 		AutoPyLock PyLo;
 		tg->get_dserver_device()->delete_devices();
 	}
-	
 
 //
 // Write file database for server started with the -file option
@@ -1409,13 +1295,28 @@ void *KillThread::run_undetached(void *ptr)
 		delete db_ptr;
 	}
 
+// 	Stop the KeepAliveThread and the EventConsumer Thread when 
+//  	they have been started to receive events.
+
+	EventConsumer *ec = ApiUtil::instance()->get_event_consumer();
+	if (ec != NULL)
+		ec->disconnect_from_notifd();
+
+//
+// If the server has used event, disconnect from the notifd
+// structured push consumer
+//
+
+	EventSupplier *ev = tg->get_event_supplier();
+	if (ev != NULL)
+		ev->disconnect_from_notifd();
+						
 //
 // Shutdown the ORB
 //
 
 	CORBA::ORB_ptr orb = Tango::Util::instance()->get_orb();
 	orb->shutdown(true);
-//	CORBA::release(orb);
 
 //
 // Cleanup logging stuffs
