@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dserverclass.cpp,v 3.12 2007/04/20 14:40:33 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dserverclass.cpp,v 3.14 2008/03/20 07:39:24 taurel Exp $\n$Name:  $";
 
 //+=============================================================================
 //
@@ -14,9 +14,15 @@ static const char *RcsId = "$Id: dserverclass.cpp,v 3.12 2007/04/20 14:40:33 tau
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.12 $
+// $Revision: 3.14 $
 //
 // $Log: dserverclass.cpp,v $
+// Revision 3.14  2008/03/20 07:39:24  taurel
+// - Last commit before Release 6.1 ?
+//
+// Revision 3.13  2007/12/12 10:17:18  taurel
+// - Db calls during DS startup has a separate timeout and some retries
+//
 // Revision 3.12  2007/04/20 14:40:33  taurel
 // - Ported to Windows 64 bits x64 architecture
 //
@@ -1338,6 +1344,7 @@ void DServerClass::command_factory()
 
 void DServerClass::device_factory(const Tango::DevVarStringArray *devlist_ptr)
 {
+	Tango::Util *tg = Tango::Util::instance();
 	
 	for (unsigned long i = 0;i < devlist_ptr->length();i++)
 	{
@@ -1362,6 +1369,18 @@ void DServerClass::device_factory(const Tango::DevVarStringArray *devlist_ptr)
 			export_device(device_list.back());
 		else
 			export_device(device_list.back(),(*devlist_ptr)[i]);
+			
+//
+// After the export of the admin device, the server is marked as started
+// and the database server connection timeout is set to the classical
+// timeout value (Except for db server itself)
+//
+
+		tg->set_svr_starting(false);
+		Database *db = tg->get_database();
+		if (db != NULL)
+			db->set_timeout_millis(CLNT_TIMEOUT);
+			
 	}
 }
 
