@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dserverpoll.cpp,v 3.16 2007/05/17 07:59:07 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dserverpoll.cpp,v 3.20 2008/01/08 14:38:18 taurel Exp $\n$Name:  $";
 
 //+=============================================================================
 //
@@ -14,9 +14,26 @@ static const char *RcsId = "$Id: dserverpoll.cpp,v 3.16 2007/05/17 07:59:07 taur
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 3.16 $
+// $Revision: 3.20 $
 //
 // $Log: dserverpoll.cpp,v $
+// Revision 3.20  2008/01/08 14:38:18  taurel
+// - strcasecmp() is not named like this on Windows !!
+//
+// Revision 3.19  2007/11/08 12:03:44  taurel
+// - Start implementing user interceptors
+// - Fix bug in poll thread pproperty management when removing polling object
+// - Set a database timeout to 6 sec
+//
+// Revision 3.18  2007/10/26 11:35:44  taurel
+// - Fix bug in synchronization with polling thread for start_polling, stop_polling and rem_obj_polling
+// - String comparaison now case insensitive when storing polling properties in db
+//
+// Revision 3.17  2007/10/16 08:23:37  taurel
+// - Add management of the TC connection establishment timeout for DB access
+// - Add DB server cache in DS used during DS startup sequence
+// - Comment out the sleep time during DS startup sequence
+//
 // Revision 3.16  2007/05/17 07:59:07  taurel
 // - The polling is not configured via a separate thread any more. The polling thread add_obj_polling method has been modified to support a parameter telling to the polling thread when it has to polled the object.
 // Add device name in monitor print message
@@ -832,7 +849,9 @@ void DServer::add_obj_polling(const Tango::DevVarLongStringArray *argin,
 	{
 		if (poll_list[i]->get_type() == type)
 		{
-			if (poll_list[i]->get_name() == obj_name)
+			string name_lower = poll_list[i]->get_name();
+			transform(name_lower.begin(),name_lower.end(),name_lower.begin(),::tolower);
+			if (name_lower == obj_name)
 			{
 				TangoSys_OMemStream o;
 				if (type == Tango::POLL_CMD)
@@ -976,7 +995,7 @@ void DServer::add_obj_polling(const Tango::DevVarLongStringArray *argin,
 				vector<string> &cmd_list = dev->get_polled_cmd();
 				for (i = 0;i < cmd_list.size();i = i+2)
 				{
-					if (cmd_list[i] == obj_name)
+					if (TG_strcasecmp(cmd_list[i].c_str(),obj_name.c_str()) == 0)
 					{
 						cmd_list[i + 1] = upd_str;
 						break;
@@ -1011,7 +1030,7 @@ void DServer::add_obj_polling(const Tango::DevVarLongStringArray *argin,
 				vector<string> &attr_list = dev->get_polled_attr();
 				for (i = 0;i < attr_list.size();i = i+2)
 				{
-					if (attr_list[i] == obj_name)
+					if (TG_strcasecmp(attr_list[i].c_str(),obj_name.c_str()) == 0)
 					{
 						attr_list[i + 1] = upd_str;
 						break;
@@ -1403,7 +1422,7 @@ void DServer::rem_obj_polling(const Tango::DevVarStringArray *argin,
 			omni_mutex_lock sync(mon);
 			if (shared_cmd.cmd_pending == true)
 			{
-				mon.signal();
+				mon.wait();
 			}
 			shared_cmd.cmd_pending = true;
 			if (tmp_upd == 0)
@@ -1418,10 +1437,10 @@ void DServer::rem_obj_polling(const Tango::DevVarStringArray *argin,
 
 			cout4 << "Cmd sent to polling thread" << endl;
 
-	//
-	// Wait for thread to execute command except if the command is
-	// requested by the polling thread itself
-	//
+//
+// Wait for thread to execute command except if the command is
+// requested by the polling thread itself
+//
 
 			omni_thread *th = omni_thread::self();
 			int th_id = th->id();
@@ -1443,9 +1462,9 @@ void DServer::rem_obj_polling(const Tango::DevVarStringArray *argin,
 		cout4 << "Thread cmd normally executed" << endl;
 	}
 	else
-		{
+	{
 		cout4 << "Polling thread is no longer running!!!!" << endl;
-		}
+	}
 
 //
 // Remove the object from the polled object list
@@ -1550,7 +1569,10 @@ void DServer::rem_obj_polling(const Tango::DevVarStringArray *argin,
 		{	
 			DbData send_data;
 			send_data.push_back(db_info);
-			dev->get_db_device()->put_property(send_data);
+			if (db_info.size() == 0)
+				dev->get_db_device()->delete_property(send_data);
+			else
+				dev->get_db_device()->put_property(send_data);
 		
 			cout4 << "Database polling properties updated" << endl;
 		}
@@ -1584,7 +1606,7 @@ void DServer::stop_polling()
 		omni_mutex_lock sync(mon);
 		if (shared_cmd.cmd_pending == true)
 		{
-			mon.signal();
+			mon.wait();
 		}
 		shared_cmd.cmd_pending = true;
 		shared_cmd.cmd_code = POLL_STOP;
@@ -1642,7 +1664,7 @@ void DServer::start_polling()
 		omni_mutex_lock sync(mon);
 		if (shared_cmd.cmd_pending == true)
 		{
-			mon.signal();
+			mon.wait();
 		}
 		shared_cmd.cmd_pending = true;
 		shared_cmd.cmd_code = POLL_START;
