@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dserversignal.cpp,v 3.9 2007/04/20 14:40:33 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dserversignal.cpp,v 3.12 2008/04/04 14:25:02 jensmeyer Exp $\n$Name:  $";
 
 //+=============================================================================
 //
@@ -14,9 +14,18 @@ static const char *RcsId = "$Id: dserversignal.cpp,v 3.9 2007/04/20 14:40:33 tau
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.9 $
+// $Revision: 3.12 $
 //
 // $Log: dserversignal.cpp,v $
+// Revision 3.12  2008/04/04 14:25:02  jensmeyer
+// Added the necessary signal definitions for FreeBSD __freebsd__
+//
+// Revision 3.11  2008/03/27 16:25:00  jensmeyer
+// Changed errno definition for Windows.
+//
+// Revision 3.10  2008/02/28 12:29:31  jensmeyer
+// Added signal definitions for MacOSX (__darwin__).
+//
 // Revision 3.9  2007/04/20 14:40:33  taurel
 // - Ported to Windows 64 bits x64 architecture
 //
@@ -213,7 +222,9 @@ static const char *RcsId = "$Id: dserversignal.cpp,v 3.9 2007/04/20 14:40:33 tau
 #include <new>
 #include <dserversignal.h>
 
+#ifndef _TG_WINDOWS_
 extern int errno;
+#endif
 
 namespace Tango
 {
@@ -297,8 +308,6 @@ DServerSignal::DServerSignal():TangoMonitor("signal")
 	sig_name[SIGUSR1] = "SIGUSR1";
 	sig_name[SIGUSR2] = "SIGUSR2";
 	sig_name[SIGCHLD] = "SIGCHLD";
-	sig_name[SIGCLD] = "SIGCLD";
-	sig_name[SIGPWR] = "SIGPWR";
 	sig_name[SIGVTALRM] = "SIGVTALRM";
 	sig_name[SIGPROF] = "SIGPROF";
 	sig_name[SIGIO] = "SIGIO";
@@ -308,17 +317,29 @@ DServerSignal::DServerSignal():TangoMonitor("signal")
 	sig_name[SIGCONT] = "SIGCONT";
 	sig_name[SIGTTIN] = "SIGTTIN";
 	sig_name[SIGTTOU] = "SIGTTOU";
-	sig_name[SIGURG] = "SIGURG";
-	
+	sig_name[SIGURG]  = "SIGURG";
+	sig_name[SIGSYS]  = "SIGSYS";
 #ifdef linux
 	sig_name[SIGXCPU] = "SIGXCPU";
 	sig_name[SIGXFSZ] = "SIGXFSZ";
+	sig_name[SIGCLD]  = "SIGCLD";
+	sig_name[SIGPWR]  = "SIGPWR";
 #else
 #ifdef sun
-#else 
-	sig_name[SIGEMT] = "SIGEMT";
-	sig_name[SIGSYS] = "SIGSYS";
+	sig_name[SIGCLD]  = "SIGCLD";
+	sig_name[SIGPWR]  = "SIGPWR";
 	sig_name[SIGLOST] = "SIGLOST";
+	sig_name[SIGEMT]  = "SIGEMT";
+#else
+#ifdef __darwin__
+	sig_name[SIGEMT]  = "SIGEMT";
+	sig_name[SIGINFO] = "SIGINFO";
+#else
+#ifdef __freebsd__
+	sig_name[SIGXCPU] = "SIGXCPU";
+	sig_name[SIGXFSZ] = "SIGXFSZ";
+#endif /* __freebsd__ */
+#endif /* __darwin__ */
 #endif /* sun */
 #endif /* linux */
 #endif /* _TG_WINDOWS_ */
