@@ -11,9 +11,18 @@
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.2 $
+// $Revision: 3.5 $
 //
 // $Log: dserversignal.h,v $
+// Revision 3.5  2008/04/04 14:25:02  jensmeyer
+// Added the necessary signal definitions for FreeBSD __freebsd__
+//
+// Revision 3.4  2008/02/28 12:29:31  jensmeyer
+// Added signal definitions for MacOSX (__darwin__).
+//
+// Revision 3.3  2007/04/20 14:40:34  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
 // Revision 3.2  2006/01/20 08:22:29  taurel
 // - Added necessary changes to support Device server written in Python
 //
@@ -156,7 +165,8 @@ namespace Tango
 //
 //=============================================================================
 
-#if ((defined sun) || defined (WIN32))
+#if ((defined sun) || defined (_TG_WINDOWS_) || \
+    (defined __darwin__) || (defined __freebsd__)) 
 #define		_NSIG		NSIG
 #endif
 
@@ -206,7 +216,7 @@ public :
 		
 		TangoSys_Pid my_pid;
 		bool th_data_created;
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 		pthread_t my_thread;
 #endif
 		void *run_undetached(void *);
@@ -223,7 +233,7 @@ protected :
 	bool				sig_to_remove;
 	int				inst_sig;
 	int 				rem_sig;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	static HANDLE			win_ev;
 	static int			win_signo;
 #endif
@@ -236,7 +246,7 @@ private:
 	vector<DeviceClass *>::iterator find_class(long, DeviceClass *);
 	vector<DeviceClass *>::iterator find_delayed_class(long, DeviceClass *);
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	static inline bool auto_signal(long s)
 	{
 		if ((s==SIGINT) || (s==SIGTERM) || (s==SIGABRT) || (s==SIGBREAK))
@@ -265,7 +275,7 @@ private:
 #endif
 
 
-#if (defined __linux)
+#if (defined __linux || defined __darwin__ || defined __freebsd__)
 	static inline bool auth_signal(long s)
 	{
 		if ((s==SIGUSR1) || (s==SIGUSR2))
@@ -277,7 +287,7 @@ private:
 #if ((defined sun) || (defined __hpux))
 	static inline bool auth_signal(long s) {return true;}
 #endif
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	static inline bool auth_signal(long s) {return true;}
 #endif
 	
