@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: event.cpp,v 1.21 2007/06/21 09:05:54 jensmeyer Exp $";
+static const char *RcsId = "$Id: event.cpp,v 1.32 2008/04/02 16:03:56 taurel Exp $";
 
 ////////////////////////////////////////////////////////////////////////////////
 ///
@@ -14,9 +14,50 @@ static const char *RcsId = "$Id: event.cpp,v 1.21 2007/06/21 09:05:54 jensmeyer
 ///
 ///		original : 7 April 2003
 ///
-///		$Revision: 1.21 $
+///		$Revision: 1.32 $
 ///
 ///		$Log: event.cpp,v $
+///		Revision 1.32  2008/04/02 16:03:56  taurel
+///		- Fix compatibility bug in the Database::get_device_name() method
+///		
+///		Revision 1.31  2008/03/11 14:38:25  taurel
+///		- Apply patches from Frederic Picca about compilation with gcc 4.2
+///		
+///		Revision 1.30  2008/01/23 08:33:49  jensmeyer
+///		Modified unsubscribe_event() not to take the callback monitor
+///		before erasing the callback from the map.
+///		
+///		Revision 1.29  2008/01/18 14:25:30  jensmeyer
+///		Added the stateless event subscription in the EventConsumer.
+///		The subscribe_event method has an option now to subscribe for
+///		events in stateless way.
+///		
+///		Revision 1.28  2008/01/10 13:40:43  jensmeyer
+///		Cleaned-up comments with old thread synchronisation mechanism
+///		
+///		Revision 1.27  2008/01/09 13:03:47  jensmeyer
+///		Added a correct clean-up when exiting a device server which
+///		is also used as an event client.
+///		Added a EventConsumer::disconnect_from_notifd method as for
+///		the EventSupplier.
+///		
+///		Revision 1.26  2007/12/19 15:42:43  taurel
+///		- Add some cleanup when quitting applis or device server (notifd objects)
+///		
+///		Revision 1.25  2007/12/17 12:54:48  taurel
+///		- Remove an unused local variable
+///		
+///		Revision 1.24  2007/11/22 15:32:03  jensmeyer
+///		Modified the mutexes to allow calling subscribe_event() and unsubscribe_event() in callback methods of other events.
+///		
+///		Revision 1.23  2007/11/13 13:19:27  jensmeyer
+///		Corrected leavefunc() which is executed at client exit to shutdown the event client threads.
+///		
+///		Revision 1.22  2007/10/16 08:23:37  taurel
+///		- Add management of the TC connection establishment timeout for DB access
+///		- Add DB server cache in DS used during DS startup sequence
+///		- Comment out the sleep time during DS startup sequence
+///		
 ///		Revision 1.21  2007/06/21 09:05:54  jensmeyer
 ///		Corrected the creation of the administration device name during
 ///		event subscription for devices witout database
@@ -217,7 +258,7 @@ namespace Tango {
 
 /************************************************************************/
 /*		       															*/		
-/* 			leavfunc function 											*/
+/* 		(client_)leavfunc function 										*/
 /*			-----------------											*/
 /*		       															*/
 /* This function will be executed at process exit or when the main      */
@@ -228,52 +269,33 @@ namespace Tango {
 /* It also destroys the heartbeat filters								*/
 /*																		*/
 /************************************************************************/
-
+void client_leavefunc()
+{
+	leavefunc();
+}
 
 void leavefunc()
 {
-//	cout << "Leaving............" << endl;
+	// flag to avoid calling client_leavefunc twice
+	static bool already_executed = false;
+
 	EventConsumer *ev = ApiUtil::instance()->get_event_consumer();
 	
-	if (ev != NULL)
+	if (ev != NULL && already_executed == false)
 	{
-	
-//
-// Clean-up heartbeat filters in notification service
-//
-
-		ev->cleanup_heartbeat_filters();
-
-//
-// Shut-down the KeepAliveThread and wait for it to exit
-//
-
-		{
-			omni_mutex_lock sync(ev->cmd);
-			
-			ev->cmd.cmd_pending = true;
-			ev->cmd.cmd_code = EXIT_TH;
-			
-			ev->cmd.cond.signal();
-		}
-				
+		ev->disconnect_from_notifd();			
+		
+		//
+		// Shut-down the ORB and wait for the thread to exit
+		//
+		
 		int *rv;
-		ev->keep_alive_thread->join((void **)&rv);
-			
-//
-// Destroy DeviceProxy object stored in map container
-//
-
-		ev->cleanup_EventChannel_map();		
-			
-//
-// Shut-down the ORB and wait for its thread to exit
-//
 		ev->orb_->shutdown(true);
-
 		ev->join((void **)&rv);
-	}
 	
+		already_executed = true;
+	}
+		
 }
 
 
@@ -307,7 +329,9 @@ EventConsumer::EventConsumer(ApiUtil *ptr) : omni_thread((void *)ptr)
 
 #ifndef _USRDLL
 	if (ptr->in_server() == false)
+		{
 		atexit(leavefunc);
+		}
 #endif
 
 	start_undetached();
@@ -339,6 +363,9 @@ EventConsumer *EventConsumer::create()
 
 void *EventConsumer::run_undetached(void *arg)
 {
+	// initialise the unique event id for the client;
+	subscribe_event_id = 0;
+	
 	cmd.cmd_pending = false;
 	keep_alive_thread = new EventConsumerKeepAliveThread(cmd);
 	keep_alive_thread->start();
@@ -361,6 +388,43 @@ void *EventConsumer::run_undetached(void *arg)
 	return (void *)NULL;
 }
 
+//+----------------------------------------------------------------------------
+//
+// method : 		EventConsumer::disconnect_from_notifd()
+// 
+// description : 	Method to stop the keep alive thread and to
+//                disconnect from all used notifd event channels
+//
+//-----------------------------------------------------------------------------
+void EventConsumer::disconnect_from_notifd()
+{
+
+	cout3 << "calling Tango::EventConsumer::disconnect_from_notifd() \n";
+	
+	//
+	// Shut-down the KeepAliveThread and wait for it to exit
+	//
+	
+	{
+		omni_mutex_lock sync(cmd);
+			
+		cmd.cmd_pending = true;
+		cmd.cmd_code = EXIT_TH;
+			
+		cmd.cond.signal();
+	}
+				
+		int *rv;
+		keep_alive_thread->join((void **)&rv);
+			
+	//
+	// Destroy DeviceProxy object stored in map container
+	//
+
+	cleanup_EventChannel_map();			
+}
+
+
 void EventConsumer::disconnect_structured_push_consumer()
 {
 	cout3 << "calling Tango::EventConsumer::disconnect_structured_push_consumer() \n";
@@ -409,7 +473,6 @@ void EventConsumer::connect(DeviceProxy *device_proxy)
 	std::map<std::string,EventChannelStruct>::iterator ipos;
 
 	{
-		//omni_mutex_lock lo(channel_map_mut);
 		ipos = channel_map.find(channel_name);
 
 		if (ipos == channel_map.end())
@@ -451,7 +514,6 @@ void EventConsumer::connect(DeviceProxy *device_proxy)
 
 void EventConsumer::connect_event_channel(string &channel_name,Database *db,bool reconnect)
 {
-	CORBA::Any send;
 	CORBA::Any_var received;
 	const DevVarLongStringArray *dev_import_list;
 	
@@ -467,15 +529,12 @@ void EventConsumer::connect_event_channel(string &channel_name,Database *db,bool
 
 	if (db != NULL)
 	{
-        send <<= channel_name.c_str();
 		try 
 		{
-        	received = db->command_inout("DbImportEvent",send);
+			received = db->import_event(channel_name);
 		}
 		catch (...) 
 		{
-//		cerr << channel_name << " has no event channel defined in the database - maybe the server is not running ... " << endl;
-//		channel_exported = 0;
 			TangoSys_OMemStream o;
 
 			o << channel_name;
@@ -1035,6 +1094,9 @@ void EventConsumer::attr_to_device(const AttributeValue *attr_value,
 						dev_attr->ext->StateSeq = new DevVarStateArray(max,len,tmp_state,false);
 					}
 					break;
+					
+				default:
+					break;
 				}
 		}
 	}	
@@ -1057,23 +1119,20 @@ void EventConsumer::push_structured_event(const CosNotification::StructuredEvent
 	string domain_name(event.header.fixed_header.event_type.domain_name);
 	string event_type(event.header.fixed_header.event_type.type_name);
 	string event_name(event.header.fixed_header.event_name);	
-
-	// only reading from the maps
-	ReaderLock r (map_modification_lock);
 	
 	if (event_name == "heartbeat")
 	{
-      std::map<std::string,EventChannelStruct>::iterator ipos;
-
-		//omni_mutex_lock lo(channel_map_mut); 
-      ipos = channel_map.find(domain_name);
+		// only reading from the maps
+      map_modification_lock.readerIn();
+		
+		std::map<std::string,EventChannelStruct>::iterator ipos;
+		ipos = channel_map.find(domain_name);
  
       if (ipos != channel_map.end())
       {
 			try 
 			{
 				ipos->second.channel_monitor->get_monitor();
-				//channel_map[domain_name].last_heartbeat = time(NULL);			
 				ipos->second.last_heartbeat = time(NULL);
 				ipos->second.channel_monitor->rel_monitor();	
 			}
@@ -1082,40 +1141,47 @@ void EventConsumer::push_structured_event(const CosNotification::StructuredEvent
 				cerr << "Tango::EventConsumer::push_structured_event() timeout on channel monitor of " << ipos->first << endl;
 			}
 		}
+		
+		map_modification_lock.readerOut();
 	}
+	
 	else
 	{
-		string attr_event_name = domain_name + "." + event_name;
-	
+		map_modification_lock.readerIn();
+		bool map_lock = true;
+		
+		string attr_event_name = domain_name + "." + event_name;		
 		map<std::string,EventCallBackStruct>::iterator ipos;
 
-		//cb_map_mut.lock();	
 		ipos = event_callback_map.find(attr_event_name);
 	
 		if (ipos != event_callback_map.end())
 		{
 			try 
 			{
-				ipos->second.callback_monitor->get_monitor();			
-			
+				ipos->second.callback_monitor->get_monitor();
+				
+				CallBack *callback;
+				callback = ipos->second.callback;	
+				map_modification_lock.readerOut();
+				map_lock = false;	
+				
 				AttributeValue *attr_value = NULL;
 				AttributeValue_3 *attr_value_3 = NULL;
 				AttributeConfig_2 *attr_conf_2 = NULL;
 				AttributeConfig_3 *attr_conf_3 = NULL;
 				AttributeInfoEx *attr_info_ex = NULL;
-				CallBack *callback;
 				DevErrorList errors;
 				bool ev_attr_conf = false;
 				const DevErrorList *err_ptr;
-				//callback = event_callback_map[attr_event_name].callback;
-				callback = ipos->second.callback;
+				
 				
 				if (callback != NULL)
 				{
 
-				//
-				// Check if the event transmit error
-				//
+//
+// Check if the event transmit error
+//
 
 					DeviceAttribute *dev_attr = NULL;
 					CORBA::TypeCode_var ty = event.remainder_of_body.type();
@@ -1172,9 +1238,9 @@ void EventConsumer::push_structured_event(const CosNotification::StructuredEvent
 						event.remainder_of_body >>= err_ptr;
 						errors = *err_ptr;
 
-	//
-	// We need to find which type of event we have received
-	//
+//
+// We need to find which type of event we have received
+//
 
 						string::size_type pos = attr_event_name.find('.');
 						string att_type = attr_event_name.substr(pos + 1);
@@ -1182,9 +1248,9 @@ void EventConsumer::push_structured_event(const CosNotification::StructuredEvent
 							ev_attr_conf = true;
 					}			
 
-	//
-	// Fire the user callback
-	//
+//
+// Fire the user callback
+//
 
 					if (ev_attr_conf == false)
 					{
@@ -1193,7 +1259,6 @@ void EventConsumer::push_structured_event(const CosNotification::StructuredEvent
 															  event_name,
 															  dev_attr,
 															  errors);
-						//cb_map_mut.unlock();
 						try
 						{
 							callback->push_event(event_data);
@@ -1214,7 +1279,6 @@ void EventConsumer::push_structured_event(const CosNotification::StructuredEvent
 															  event_name,
 															  attr_info_ex,
 															  errors);
-						//cb_map_mut.unlock();
 						try
 						{
 							callback->push_event(event_data);
@@ -1235,36 +1299,47 @@ void EventConsumer::push_structured_event(const CosNotification::StructuredEvent
 			}
 			catch (...)
 			{
+				// free the map lock if not already done
+				if ( map_lock == true )
+					{
+					map_modification_lock.readerOut();
+					}
+				
 				cerr << "Tango::EventConsumer::push_structured_event() timeout on callback monitor of " << ipos->first << endl;
 			}				
 		}
-			//else
-			//{
-				//cb_map_mut.unlock();
-			//}
+	else
+		{
+		// even if nothing was found in the map, free the lock
+		map_modification_lock.readerOut();
+		}	
 	}
 
 }
 
+
 //+----------------------------------------------------------------------------
 //
 // method : 		EventConsumer::subscribe_event()
 // 
-// description : 	Method to subscribe to an event
+// description : 	Method to subscribe to an event. 
+//                Can be called in a stateless way, that it even works 
+//                when the attribute is not available.
 //
-// argument : in :	device :
-//			attribute :
-//			event :
-//			callback :
-//			filters :
+// argument : in :	device : The device handle
+//			         attribute : The name of the attribute
+//			             event : The type of event to subscribe for
+//			          callback : A pointer to the callback object
+//			           filters : Eventual event filter strings
+//                stateless : Flag to enable the stateless connection when set to true
 //
 //-----------------------------------------------------------------------------
-
-int EventConsumer::subscribe_event(DeviceProxy *device,
+int EventConsumer::subscribe_event (DeviceProxy *device,
 				   const string &attribute,
 				   EventType event,
 				   CallBack *callback,
-				   const vector<string> &filters)
+				   const vector<string> &filters,
+					bool stateless)
 {
 	if ((device == NULL) || (callback == NULL))
 	{		
@@ -1272,17 +1347,11 @@ int EventConsumer::subscribe_event(DeviceProxy *device,
                        	(const char*)"Device or callback pointer NULL",
                        	(const char*)"EventConsumer::subscribe_event()");
 	}
-	
-	static int subscribe_event_id = 0; // unique event id
 
 	string device_name = device->dev_name();
 	cout3 << "Tango::EventConsumer::subscribe_event(" << device_name << "," << attribute <<"," << event << ")\n";
 
-//
-// Build callback map key
-//
-
-	string event_name;
+	string event_name;	
 	switch (event) 
 	{
 		case CHANGE_EVENT : event_name = "change";
@@ -1307,27 +1376,114 @@ int EventConsumer::subscribe_event(DeviceProxy *device,
 		case ATTR_CONF_EVENT : event_name = "attr_conf";
 					      break;
 	}
+
+	//
+	// Build callback map key
+	//
 	
 	string att_name_lower = attribute;
 	transform(att_name_lower.begin(),att_name_lower.end(),att_name_lower.begin(),::tolower);
 	string callback_key = device_name + "/" + att_name_lower + "." + event_name;
 
 	// only reading from the maps
-	map_modification_lock.readerIn();
-
-	if (event_callback_map.find(callback_key) != event_callback_map.end())
 	{
-		map_modification_lock.readerOut();
+		ReaderLock r (map_modification_lock);
 		
-		TangoSys_OMemStream o;
+		if (event_callback_map.find(callback_key) != event_callback_map.end())
+		{
+			map_modification_lock.readerOut();
+		
+			TangoSys_OMemStream o;
 
-		o << "Already connected to event " << callback_key << ends;
-		EventSystemExcept::throw_exception((const char*)"API_MethodArgument",
+			o << "Already connected to event " << callback_key << ends;
+			EventSystemExcept::throw_exception((const char*)"API_MethodArgument",
                        	o.str(),
                        	(const char*)"EventConsumer::subscribe_event()");
+		}
 	}
+				
+	try
+	{
+		int event_id = connect_event (device, attribute, event, callback, 
+		                             filters, event_name);
+		return event_id;		
+	}
+		
+	catch (Tango::DevFailed &e)
+	{
+		// if the stateless flag is not true, rethrow the exception
+		if ( stateless == false )
+		{
+			throw;
+		}
+		
+		// when the subscribe event has not worked, store the connection data in a vector of not
+		// yet connected events.
+		// Retry to connect in the next heartbeat period.
+		
+		EventNotConnected conn_params;
+		conn_params.device     = device;
+		conn_params.attribute  = attribute;
+		conn_params.event_type = event;
+		conn_params.event_name = event_name;
+		conn_params.callback   = callback;
+		conn_params.filters    = filters;
+		conn_params.last_heartbeat = time(NULL);
+		
+		
+		// protect the vector as the other maps!
+		WriterLock w(map_modification_lock);
+		
+		// create and save the unique event ID
+		subscribe_event_id++;	
+		conn_params.event_id = subscribe_event_id;
+	
+		event_not_connected.push_back (conn_params);	
+		return subscribe_event_id;
+	}
+}
 
 
+
+
+
+//+----------------------------------------------------------------------------
+//
+// method : 		EventConsumer::connect_event()
+// 
+// description : 	Method to connect to an event and to
+//                return an approriate exception if the 
+//                connection fails.
+//
+// argument : in :	device : The device handle
+//			         attribute : The name of the attribute
+//			             event : The type of event to subscribe for
+//			          callback : A pointer to the callback object
+//			           filters : Eventual event filter strings
+//                event_id  : the unique event ID
+//
+//-----------------------------------------------------------------------------
+
+int EventConsumer::connect_event(DeviceProxy *device,
+				   const string &attribute,
+				   EventType event,
+				   CallBack *callback,
+				   const vector<string> &filters,
+					string &event_name,
+					int event_id)
+{
+	int ret_event_id = event_id;
+	string device_name = device->dev_name();
+	cout3 << "Tango::EventConsumer::connect_event(" << device_name << "," << attribute <<"," << event << ")\n";
+
+//
+// Build callback map key
+//
+	
+	string att_name_lower = attribute;
+	transform(att_name_lower.begin(),att_name_lower.end(),att_name_lower.begin(),::tolower);
+	string callback_key = device_name + "/" + att_name_lower + "." + event_name;
+
 //
 // Inform server that we want to subscribe (we cannot use the asynchronous fire-and-forget
 // request so as not to block the client because it does not reconnect if the device is
@@ -1350,6 +1506,8 @@ int EventConsumer::subscribe_event(DeviceProxy *device,
 	bool allocated = false;
 	string channel_name;
 	
+	map_modification_lock.readerIn();
+	
 	ipos = device_channel_map.find(device_name);
 	if (ipos == device_channel_map.end())
 	{
@@ -1377,7 +1535,7 @@ int EventConsumer::subscribe_event(DeviceProxy *device,
 			o << "Check that device server is running..." << ends;
 			Except::throw_exception((const char *)"API_BadConfigurationProperty",
 				       	 	o.str(),
-				        	(const char *)"EventConsumer::subscribe_event()");
+				         	(const char *)"EventConsumer::subscribe_event()");
 		}
 	}
 	else
@@ -1419,8 +1577,9 @@ int EventConsumer::subscribe_event(DeviceProxy *device,
 		map_modification_lock.readerOut();
 		WriterLock w(map_modification_lock);
 
-		cout3 << "device " << device_name << " is not connected, going to connect to the event channel !\n";
+		cout3 << "device " << device_name << " is not connected, going to connect to the event channel !\n";		
 		connect(device);
+		
 		ipos = device_channel_map.find(device_name);
 		if (ipos == device_channel_map.end())
 		{
@@ -1434,7 +1593,6 @@ int EventConsumer::subscribe_event(DeviceProxy *device,
 
 		channel_name = device_channel_map[device_name];		
 		{
-			//omni_mutex_lock lock_cha(channel_map_mut);
 			channel_map[channel_name].last_subscribed = time(NULL);
 		}
 	}
@@ -1457,7 +1615,6 @@ int EventConsumer::subscribe_event(DeviceProxy *device,
 	
 	try
 	{
-		//omni_mutex_lock lock_channel(channel_map_mut);
 		channel_map[channel_name].channel_monitor->get_monitor();
     	ffp    = channel_map[channel_name].eventChannel->default_filter_factory();  
     	filter = ffp->create_filter("EXTENDED_TCL");
@@ -1516,7 +1673,6 @@ int EventConsumer::subscribe_event(DeviceProxy *device,
 		
     	CosNotifyFilter::ConstraintInfoSeq_var dummy = filter->add_constraints(exp);
 		{		
-			//omni_mutex_lock lo(channel_map_mut);
     		filter_id = channel_map[channel_name].structuredProxyPushSupplier->add_filter(filter);
 		}
 
@@ -1528,15 +1684,21 @@ int EventConsumer::subscribe_event(DeviceProxy *device,
 		new_event_callback.callback = callback;
 		new_event_callback.filter_id = filter_id;
 		new_event_callback.filter_constraint = constraint_expr;
-		subscribe_event_id++;
-		new_event_callback.id = subscribe_event_id;
+		
+		// if an event ID was passwd to the method, reuse it!
+		
+		if ( ret_event_id <= 0 )
+		{
+			subscribe_event_id++;
+			ret_event_id = subscribe_event_id;
+		}
+		new_event_callback.id = ret_event_id;
 				
 		// create a callback monitor
 		new_event_callback.callback_monitor = new TangoMonitor();
 		// set the timeout for the callback monitor to 500ms not to block the event consumer for to long.
 		new_event_callback.callback_monitor->timeout(500);
 		
-		//omni_mutex_lock lock_cb(cb_map_mut);
 		event_callback_map[callback_key] = new_event_callback;
   	}
   	catch(CosNotifyFilter::InvalidConstraint &)
@@ -1611,7 +1773,6 @@ int EventConsumer::subscribe_event(DeviceProxy *device,
 						      err);
 		{
 			event_callback_map[callback_key].callback_monitor->get_monitor();
-			//omni_mutex_lock lock_cb(cb_map_mut);
 			try
 			{
 				callback->push_event(event_data);
@@ -1650,7 +1811,6 @@ int EventConsumer::subscribe_event(DeviceProxy *device,
 						      err);
 		{
 			event_callback_map[callback_key].callback_monitor->get_monitor();
-			//omni_mutex_lock lock_cb(cb_map_mut);
 			try
 			{
 				callback->push_event(event_data);
@@ -1665,7 +1825,7 @@ int EventConsumer::subscribe_event(DeviceProxy *device,
 		delete event_data;
 	}
 
-	return subscribe_event_id;
+	return ret_event_id;
 }
 
 
@@ -1681,20 +1841,21 @@ int EventConsumer::subscribe_event(DeviceProxy *device,
 
 void EventConsumer::unsubscribe_event(int event_id)
 {
-	std::map<std::string,EventCallBackStruct>::iterator epos;
-	bool found=false;
-
-	WriterLock w(map_modification_lock);
-	//omni_mutex_lock lock_cb(cb_map_mut);
 	
-	for (epos = event_callback_map.begin(); epos != event_callback_map.end(); epos++)
+	WriterLock w(map_modification_lock);
+
+//
+// First remove the callback entry from the callback map
+//
+	
+	std::map<std::string,EventCallBackStruct>::iterator epos;
+	for (epos = event_callback_map.begin(); epos != event_callback_map.end(); ++epos)
 	{
 		if(epos->second.id == event_id)
 		{
 			//cout << "Tango::EventConsumer::unsubscribe_event() - found event id " << event_id << " going to remove_filter()\n";
 			try
 			{
-				//omni_mutex_lock lock_cha(channel_map_mut);
 				CosNotifyFilter::Filter_var f = channel_map[epos->second.channel_name].structuredProxyPushSupplier->get_filter(epos->second.filter_id);
 				channel_map[epos->second.channel_name].structuredProxyPushSupplier->remove_filter(epos->second.filter_id);
 				f->destroy();
@@ -1705,46 +1866,103 @@ void EventConsumer::unsubscribe_event(int event_id)
 					(const char*)"Failed to unsubscribe event, caught exception while calling remove_filter() or destroy() (hint: check the Notification daemon is running ",
 					(const char*)"EventConsumer::unsubscribe_event()");    
 			}
+
+			string deleted_channel_name = epos->second.channel_name;			
 			event_callback_map.erase(epos);
-			found=true;
-			break;
+
+//
+// Check if there is another callback using the same channel
+//
+			
+			std::map<std::string,EventCallBackStruct>::iterator cb_pos;
+			bool channel_used_elsewhere = false;
+			for (cb_pos = event_callback_map.begin(); cb_pos != event_callback_map.end(); ++cb_pos)
+			{
+				if (cb_pos->second.channel_name == deleted_channel_name)
+				{
+					channel_used_elsewhere = true;
+					break;
+				}
+			}
+
+//
+// This channel is not used anymore in the app, remove its entry in the channel maps
+//
+			
+			if (channel_used_elsewhere == false)
+			{
+				std::map<std::string,EventChannelStruct>::iterator chan_pos;
+				for (chan_pos = channel_map.begin(); chan_pos != channel_map.end(); ++chan_pos)
+				{
+					if (chan_pos->first == deleted_channel_name)
+					{
+						if (chan_pos->second.adm_device_proxy != NULL)
+						{
+							try
+							{
+								CosNotifyFilter::Filter_var f = chan_pos->second.structuredProxyPushSupplier->get_filter(chan_pos->second.heartbeat_filter_id);
+								chan_pos->second.structuredProxyPushSupplier->remove_filter(chan_pos->second.heartbeat_filter_id);
+								f->destroy();
+
+								chan_pos->second.structuredProxyPushSupplier->disconnect_structured_push_supplier();
+							}
+							catch (...)
+							{
+								EventSystemExcept::throw_exception((const char*)"API_EventNotFound",
+									(const char*)"Failed to unsubscribe event, caught exception while calling remove_filter() or destroy() on the heartbeat filter (hint: check the Notification daemon is running ",
+									(const char*)"EventConsumer::unsubscribe_event()");    
+							}
+							
+							delete chan_pos->second.adm_device_proxy;
+							if (chan_pos->second.channel_monitor != NULL)
+								delete chan_pos->second.channel_monitor;
+						}
+			
+						channel_map.erase(chan_pos);
+						break;
+					}
+				}
+				
+				std::map<std::string,std::string>::iterator dev_pos;
+				for (dev_pos = device_channel_map.begin(); dev_pos != device_channel_map.end(); ++dev_pos)
+				{
+					if (dev_pos->second == deleted_channel_name)
+					{
+						device_channel_map.erase(dev_pos);
+						break;
+					}
+				}
+			}
+			
+			return;
 		}
 	}
-	if (!found)
+	
+	// check also the vector of not yet connected events
+		
+	if ( event_not_connected.size() > 0 )
 	{
-		EventSystemExcept::throw_exception((const char*)"API_EventNotFound",
+		std::vector<EventNotConnected>::iterator vpos;
+		for (vpos =  event_not_connected.begin(); 
+			 vpos != event_not_connected.end(); vpos++)
+		{
+			if ( vpos->event_id == event_id)
+			{
+			   // delete element from vector when subscribe worked
+				
+			   event_not_connected.erase(vpos);
+			   return;
+			}
+		}	
+	}
+		
+	// nothing was found!
+		
+	EventSystemExcept::throw_exception((const char*)"API_EventNotFound",
 			(const char*)"Failed to unsubscribe event, the event id specified does not correspond with any known one",
 			(const char*)"EventConsumer::unsubscribe_event()");    
-	}
-	
 }
 
-//+----------------------------------------------------------------------------
-//
-// method : 		EventConsumer::cleanup_heartbeat_filters()
-// 
-// description : 	Method to destroy the filter created in the
-//			notification service for the heartbeat event.
-//
-//-----------------------------------------------------------------------------
-
-void EventConsumer::cleanup_heartbeat_filters()
-{
-	std::map<std::string,EventChannelStruct>::iterator ipos;
-
-	//channel_map_mut.lock();
-	for (ipos = channel_map.begin(); ipos != channel_map.end(); ipos++)
-	{
-		ipos->second.channel_monitor->get_monitor();
-		
-		CosNotifyFilter::Filter_var f = ipos->second.structuredProxyPushSupplier->get_filter(ipos->second.heartbeat_filter_id);
-		ipos->second.structuredProxyPushSupplier->remove_filter(ipos->second.heartbeat_filter_id);
-		f->destroy();
-		
-		ipos->second.channel_monitor->rel_monitor();
-	}
-	//channel_map_mut.unlock();
-}
 
 //+----------------------------------------------------------------------------
 //
@@ -1757,15 +1975,35 @@ void EventConsumer::cleanup_heartbeat_filters()
 
 void EventConsumer::cleanup_EventChannel_map()
 {
+	std::map<std::string,EventCallBackStruct>::iterator epos;
+	for (epos = event_callback_map.begin(); epos != event_callback_map.end(); epos++)
+	{
+		CosNotifyFilter::Filter_var f = channel_map[epos->second.channel_name].structuredProxyPushSupplier->get_filter(epos->second.filter_id);
+		channel_map[epos->second.channel_name].structuredProxyPushSupplier->remove_filter(epos->second.filter_id);
+		f->destroy();
+	}
+	
 	std::map<std::string,EventChannelStruct>::iterator ipos;
 	{
-		//omni_mutex_lock lo(channel_map_mut);
 		for (ipos = channel_map.begin(); ipos != channel_map.end(); ipos++)
 		{
 			if (ipos->second.adm_device_proxy != NULL)
 			{
 				ipos->second.channel_monitor->get_monitor();
 				
+				// Destroy the filter created in the
+				// notification service for the heartbeat event
+				
+			   	CosNotifyFilter::Filter_var f = ipos->second.structuredProxyPushSupplier->get_filter(ipos->second.heartbeat_filter_id);
+				ipos->second.structuredProxyPushSupplier->remove_filter(ipos->second.heartbeat_filter_id);
+				f->destroy();
+				
+				// disconnect the pushsupplier to stop receiving events
+				//cout << "EventConsumer::cleanup_EventChannel_map(): Disconnect push supplier!" << endl;
+				ipos->second.structuredProxyPushSupplier->disconnect_structured_push_supplier();
+							
+				// Release the connection to the device server administration device
+				
 				delete ipos->second.adm_device_proxy;
 				ipos->second.adm_device_proxy = NULL;
 				
@@ -1930,7 +2168,6 @@ bool EventConsumerKeepAliveThread::reconnect_to_channel(EvChanIte &ipos,EventCon
 
 	cout3 << "Entering KeepAliveThread::reconnect()" << endl;
 
-	//event_consumer->cb_map_mut.lock();
 	for (epos = event_consumer->event_callback_map.begin(); epos != event_consumer->event_callback_map.end(); epos++)
 	{
 		if ((epos->second.channel_name == ipos->first) && (epos->second.callback != NULL))
@@ -1956,8 +2193,6 @@ bool EventConsumerKeepAliveThread::reconnect_to_channel(EvChanIte &ipos,EventCon
 		}
 	}
 
-	//event_consumer->cb_map_mut.unlock();
-	
 	return ret;
 }
 
@@ -1982,7 +2217,6 @@ void EventConsumerKeepAliveThread::reconnect_to_event(EvChanIte &ipos,EventConsu
 
 	cout3 << "Entering KeepAliveThread::reconnect_to_event()" << endl;
 
-	// event_consumer->cb_map_mut.lock();
 	for (epos = event_consumer->event_callback_map.begin(); epos != event_consumer->event_callback_map.end(); epos++)
 	{
 		if ((epos->second.channel_name == ipos->first) && (epos->second.callback != NULL))
@@ -2010,8 +2244,6 @@ void EventConsumerKeepAliveThread::reconnect_to_event(EvChanIte &ipos,EventConsu
 			}
 		}
 	}
-	//event_consumer->cb_map_mut.unlock();
-	
 }
 
 //+----------------------------------------------------------------------------
@@ -2172,27 +2404,150 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 // Re-subscribe
 //
  
- 		{
+		// lock the maps only for reading
+		event_consumer->map_modification_lock.readerIn();
+			
+		now = time(NULL);
+			
+		//
+		// check the list of not yet connected events and try to subscribe
+		//
+			
+		if ( event_consumer->event_not_connected.size() > 0 )
+			{
+			std::vector<EventNotConnected>::iterator vpos;
+			for (vpos = event_consumer->event_not_connected.begin(); 
+				 vpos != event_consumer->event_not_connected.end();
+				 vpos++ )
+				{
+				
+				// check wether it is necessary to try to subscribe again!
+				if ( (now - vpos->last_heartbeat) >= (EVENT_HEARTBEAT_PERIOD) )
+					{
+					try
+						{
+						// try to subscribe
+						
+						event_consumer->map_modification_lock.readerOut();
+						event_consumer->connect_event (vpos->device,vpos->attribute,vpos->event_type, 
+																					vpos->callback, 
+																					vpos->filters,
+																					vpos->event_name,
+																					vpos->event_id);	
+						event_consumer->map_modification_lock.readerIn();
+							
+						// delete element from vector when subscribe worked
+						vpos = event_consumer->event_not_connected.erase(vpos);
+						if ( vpos == event_consumer->event_not_connected.end() )
+							{
+							break;
+							}
+						}
+		
+					catch (Tango::DevFailed &e) 
+						{
+						event_consumer->map_modification_lock.readerIn();
+						
+						// subscribe has not worked, try again in the next hearbeat period
+						vpos->last_heartbeat = now;
+						
+						// The event can still not be connected.
+						// Send the return error message as event to the client application.
+						//
+						// push an event with the error message!
+						
+						DevErrorList err;
+						err.length(0);
+						string domain_name = vpos->device->dev_name() + "/" + vpos->attribute;
+						err = e.errors;
+											
+						//
+						// For attribute data event
+						//
+						
+						if ((vpos->event_name == "change") ||
+	    					(vpos->event_name == "quality") || 
+	    					(vpos->event_name == "archive") ||
+	    					(vpos->event_name == "user_event"))
+							{						
+							DeviceAttribute da;
+							EventData *event_data = new EventData(vpos->device,
+													      			domain_name,
+													      			vpos->event_name,
+													      			&da,
+													      			err);
+							try
+								{
+								vpos->callback->push_event(event_data);
+								}
+							catch (...)
+								{
+								cerr << "EventConsumerKeepAliveThread::run_undetached() exception in callback method of " << domain_name << endl;
+								}
+						
+							event_data->attr_value = NULL;							
+							delete event_data;						
+							}
+						
+						//
+						// For attribute configuration event
+						//
+							
+						else if (vpos->event_name == CONF_TYPE_EVENT)
+							{
+							AttributeInfoEx aie;
+							AttrConfEventData *event_data = new AttrConfEventData(vpos->device,
+													      			domain_name,
+													      			vpos->event_name,
+													      			&aie,
+													      			err);
+							try
+								{
+								vpos->callback->push_event(event_data);
+								}
+							catch (...)
+								{
+								cerr << "EventConsumerKeepAliveThread::run_undetached() exception in callback method of " << domain_name << endl;
+								}
+							
+							event_data->attr_conf = NULL;
+							delete event_data;				
+							}
+						}
+					
+					catch (...)
+						{
+						event_consumer->map_modification_lock.readerIn();
+						
+						// subscribe has not worked, try again in the next hearbeat period
+						vpos->last_heartbeat = now;
+						
+						cout << "During the event subscription an exception was send which is not a Tango::DevFailed exception!" << endl;	
+						}						
+					}
+				}
+			}
+		event_consumer->map_modification_lock.readerOut();
+		
+		
+		// check for all other event reconnections
+			
+		{
 			// lock the maps only for reading
-			ReaderLock r (event_consumer->map_modification_lock);
-			//cout << "keep alive taking reader lock!" << endl;
+			ReaderLock r (event_consumer->map_modification_lock);	
 			
 			std::map<std::string,EventChannelStruct>::iterator ipos;
 			std::map<std::string,EventCallBackStruct>::iterator epos;
-			now = time(NULL);
 
-			//event_consumer->channel_map_mut.lock();
 			for (ipos = event_consumer->channel_map.begin(); ipos != event_consumer->channel_map.end(); ipos++)
 			{
 				try
 				{
 					// lock the event cahnnel
 					ipos->second.channel_monitor->get_monitor();
-					//cout << "keep alive taking channel monitor for " << ipos->first << endl;
 
 					if ((now - ipos->second.last_subscribed) > EVENT_RESUBSCRIBE_PERIOD/3)
 					{
-						//event_consumer->cb_map_mut.lock();
 						for (epos = event_consumer->event_callback_map.begin(); epos != event_consumer->event_callback_map.end(); epos++)
 						{
 							if (epos->second.channel_name == ipos->first )
@@ -2201,31 +2556,27 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 								{
 									// lock the callback 
 									epos->second.callback_monitor->get_monitor();
-									//cout << "keep alive taking callback monitor : resubscribe : " << epos->first<< endl;
 
-       							DeviceData subscriber_in;
-       							vector<string> subscriber_info;
-       							subscriber_info.push_back(epos->second.device->dev_name());
-       							subscriber_info.push_back(epos->second.attr_name);
-       							subscriber_info.push_back("subscribe");
-       							subscriber_info.push_back(epos->second.event_name);
-       							subscriber_in << subscriber_info;
-      							ipos->second.adm_device_proxy->command_inout("EventSubscriptionChange",subscriber_in);
+									DeviceData subscriber_in;
+									vector<string> subscriber_info;
+									subscriber_info.push_back(epos->second.device->dev_name());
+									subscriber_info.push_back(epos->second.attr_name);
+									subscriber_info.push_back("subscribe");
+									subscriber_info.push_back(epos->second.event_name);
+									subscriber_in << subscriber_info;
+									ipos->second.adm_device_proxy->command_inout("EventSubscriptionChange",subscriber_in);
 
 									ipos->second.last_subscribed = time(NULL);
         							epos->second.last_subscribed = time(NULL);
 
 									epos->second.callback_monitor->rel_monitor();
-									//cout << "keep alive release callback monitor : resubscribe" << endl;
 								}
 								catch (...) 
 								{
-								epos->second.callback_monitor->rel_monitor();
-								cerr << "EventConsumerKeepAliveThread::run_undetached() exception during periodic resubscription of " << epos->first << endl;
+									epos->second.callback_monitor->rel_monitor();
 								};	
 							}
 						}
-						//event_consumer->cb_map_mut.unlock();
 					}
 
 		//
@@ -2321,7 +2672,6 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 						DeviceAttribute *dev_attr = NULL;
 						AttributeInfoEx *dev_attr_conf = NULL;
 
-						//event_consumer->cb_map_mut.lock();
 						for (epos = event_consumer->event_callback_map.begin(); epos != event_consumer->event_callback_map.end(); epos++)
 						{
 							if ((epos->second.channel_name == ipos->first) && (epos->second.callback != NULL))
@@ -2330,7 +2680,6 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 								try
 								{
 									epos->second.callback_monitor->get_monitor();
-									//cout << "keep alive get callback monitor : event : " << epos->first << endl;
 
 									if (epos->second.filter_ok == false)
 									{
@@ -2413,7 +2762,7 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 
 										try 
 										{
-       									ipos->second.adm_device_proxy->command_inout("EventSubscriptionChange",subscriber_in);
+											ipos->second.adm_device_proxy->command_inout("EventSubscriptionChange",subscriber_in);
 											ipos->second.heartbeat_skipped = false;
         									ipos->second.last_subscribed = time(NULL);
 										}
@@ -2529,7 +2878,6 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 									}
 									// release callback monitor
 									epos->second.callback_monitor->rel_monitor();
-									//cout << "keep alive release callback monitor : event" << endl;
 								}
 								catch (...)
 								{
@@ -2537,7 +2885,6 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 								}	
 							}
 						}
-						//event_consumer->cb_map_mut.unlock();
 					}
 					else
 					{
@@ -2547,14 +2894,12 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 
 					// release channel monitor
 					ipos->second.channel_monitor->rel_monitor();
-					//cout << "keep alive release channel monitor " << endl;	
 				}
 				catch (...)
 				{
 					cerr << "EventConsumerKeepAliveThread::run_undetached() timeout on callback monitor of " << epos->first << endl;
 				}			
 			}
-			//event_consumer->channel_map_mut.unlock();
 		}
 	}
 	
