@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: eventsupplier.cpp,v 1.15 2007/04/20 14:41:33 taurel Exp $";
+static const char *RcsId = "$Id: eventsupplier.cpp,v 1.21 2008/03/11 14:38:25 taurel Exp $";
 
 ////////////////////////////////////////////////////////////////////////////////
 ///
@@ -14,9 +14,31 @@ static const char *RcsId = "$Id: eventsupplier.cpp,v 1.15 2007/04/20 14:41:33 ta
 ///
 ///		original : 29 June 2004
 ///
-///		$Revision: 1.15 $
+///		$Revision: 1.21 $
 ///
 ///		$Log: eventsupplier.cpp,v $
+///		Revision 1.21  2008/03/11 14:38:25  taurel
+///		- Apply patches from Frederic Picca about compilation with gcc 4.2
+///		
+///		Revision 1.20  2008/01/15 12:32:43  taurel
+///		- Simply remove some print
+///		
+///		Revision 1.19  2007/12/19 15:42:43  taurel
+///		- Add some cleanup when quitting applis or device server (notifd objects)
+///		
+///		Revision 1.18  2007/12/12 10:17:18  taurel
+///		- Db calls during DS startup has a separate timeout and some retries
+///		
+///		Revision 1.17  2007/11/08 12:03:44  taurel
+///		- Start implementing user interceptors
+///		- Fix bug in poll thread pproperty management when removing polling object
+///		- Set a database timeout to 6 sec
+///		
+///		Revision 1.16  2007/10/16 08:23:37  taurel
+///		- Add management of the TC connection establishment timeout for DB access
+///		- Add DB server cache in DS used during DS startup sequence
+///		- Comment out the sleep time during DS startup sequence
+///		
 ///		Revision 1.15  2007/04/20 14:41:33  taurel
 ///		- Ported to Windows 64 bits x64 architecture
 ///		
@@ -158,7 +180,8 @@ EventSupplier::EventSupplier(CORBA::ORB_var _orb,
 EventSupplier *EventSupplier::create(CORBA::ORB_var _orb,
 				     string server_name,
 				     Database *db,
-				     string &host_name) 
+				     string &host_name,
+				     Util *tg) 
 {
 	cout4 << "calling Tango::EventSupplier::create() \n";
 
@@ -176,7 +199,7 @@ EventSupplier *EventSupplier::create(CORBA::ORB_var _orb,
 //
 
 	NotifService ns;	
-	connect_to_notifd(ns,_orb,server_name,db,host_name);
+	connect_to_notifd(ns,_orb,server_name,db,host_name,tg);
 	
 //
 // EventSupplier singleton does not exist, create it
@@ -199,7 +222,7 @@ void EventSupplier::connect()
 	}
 	catch(const CosEventChannelAdmin::AlreadyConnected&)
 	{
-       		cerr << "Tango::EventSupplier::connect() caught AlreadyConnected exception" << endl;
+		cerr << "Tango::EventSupplier::connect() caught AlreadyConnected exception" << endl;
 	}
 
 }
@@ -228,7 +251,8 @@ void EventSupplier::subscription_change(const CosNotification::EventTypeSeq& add
 void EventSupplier::connect_to_notifd(NotifService &ns,CORBA::ORB_var &_orb,
 				      string &server_name,
 				      Database *db,
-				      string &host_name)
+				      string &host_name,
+				      Util *tg)
 {
 	CosNotifyChannelAdmin::EventChannelFactory_var _eventChannelFactory;
 	CosNotifyChannelAdmin::EventChannel_var _eventChannel;
@@ -242,19 +266,23 @@ void EventSupplier::connect_to_notifd(NotifService &ns,CORBA::ORB_var &_orb,
 	string factory_ior;
 	string factory_name;
 	factory_name = "notifd/factory/" + host_name;
-        CORBA::Any send;
 	CORBA::Any_var received;
 	string d_name = "DServer/";
 	d_name = d_name + server_name;
-        const DevVarLongStringArray *dev_import_list;
-	
+	const DevVarLongStringArray *dev_import_list;
+
 	if (Util::_FileDb == false)
 	{
-
-        	send <<= factory_name.c_str();
-		try 
+		try
 		{
-        		received = db->command_inout("DbImportEvent",send);
+			if (tg->get_db_cache() != NULL)
+			{
+				dev_import_list = tg->get_db_cache()->import_notifd_event();
+			}
+			else
+			{
+				received = db->import_event(factory_name);
+			}
 		}
 		catch (...) 
 		{
@@ -264,10 +292,9 @@ void EventSupplier::connect_to_notifd(NotifService &ns,CORBA::ORB_var &_orb,
 // for this server event channel, clear it.
 //
 
-        		send <<= d_name.c_str();
 			try
 			{
-				db->command_inout("DbUnExportEvent",send);
+				db->unexport_event(d_name);
 			}
 			catch (...) {}
 				
@@ -284,9 +311,9 @@ void EventSupplier::connect_to_notifd(NotifService &ns,CORBA::ORB_var &_orb,
 				(const char*)"Failed to import the EventChannelFactory from the Tango database",
 				(const char*)"EventSupplier::create()");    
 		}
-		
-        	received.inout() >>= dev_import_list;
- 
+
+		if (tg->get_db_cache() == NULL)
+			received.inout() >>= dev_import_list;
 		factory_ior = string((dev_import_list->svalue)[1]);	
 	}
 	else
@@ -320,10 +347,10 @@ void EventSupplier::connect_to_notifd(NotifService &ns,CORBA::ORB_var &_orb,
 	try
 	{
 		CORBA::Object *event_factory_obj;
-    		event_factory_obj = _orb -> string_to_object(factory_ior.c_str());
+    	event_factory_obj = _orb -> string_to_object(factory_ior.c_str());
 #ifndef _TG_WINDOWS_
-    		if (event_factory_obj -> _non_existent())
-			event_factory_obj = CORBA::Object::_nil();
+    	if (event_factory_obj -> _non_existent())
+		event_factory_obj = CORBA::Object::_nil();
 #endif /* _TG_WINDOWS_ */
 
 //
@@ -356,10 +383,9 @@ void EventSupplier::connect_to_notifd(NotifService &ns,CORBA::ORB_var &_orb,
 
 		if (Util::_FileDb == false)
 		{
-        		send <<= d_name.c_str();
 			try
 			{
-				db->command_inout("DbUnExportEvent",send);
+				db->unexport_event(d_name);
 			}
 			catch (...) {}
 		}
@@ -388,11 +414,16 @@ void EventSupplier::connect_to_notifd(NotifService &ns,CORBA::ORB_var &_orb,
 
 	if (Util::_FileDb == false)
 	{
-        	send <<= d_name.c_str();
-		//cout << d_name.c_str() << endl;
 		try 
 		{
-        		received = db->command_inout("DbImportEvent",send);
+			if (tg->get_db_cache() != NULL)
+			{
+				dev_import_list = tg->get_db_cache()->DbServerCache::import_adm_event();
+			}
+			else
+			{
+				received = db->import_event(d_name);
+			}
 		}
 		catch (...) 
 		{
@@ -407,18 +438,20 @@ void EventSupplier::connect_to_notifd(NotifService &ns,CORBA::ORB_var &_orb,
 
 			channel_exported = 0;
 		}
+
 		if (channel_exported != 0)
 		{
-        		received.inout() >>= dev_import_list;
-        		channel_ior = string((dev_import_list->svalue)[1]);
-        		channel_exported = dev_import_list->lvalue[0];
+			if (tg->get_db_cache() == NULL)
+				received.inout() >>= dev_import_list;
+        	channel_ior = string((dev_import_list->svalue)[1]);
+        	channel_exported = dev_import_list->lvalue[0];
 		
 //
 // check if the channel is exported on this host, if not assume it
 // is an old channel and we need to recreate it on the local host
 //
 
-        		channel_host = string((dev_import_list->svalue)[3]);
+        	channel_host = string((dev_import_list->svalue)[3]);
 			if (channel_host !=  host_name)
 				channel_exported = 0;
 		}
@@ -428,13 +461,13 @@ void EventSupplier::connect_to_notifd(NotifService &ns,CORBA::ORB_var &_orb,
 
 	if (channel_exported)
 	{
-        	CORBA::Object *event_channel_obj;
-        	event_channel_obj = _orb -> string_to_object(channel_ior.c_str());
+        CORBA::Object *event_channel_obj;
+        event_channel_obj = _orb -> string_to_object(channel_ior.c_str());
 
 		try
 		{
-        		if (event_channel_obj -> _non_existent())
-                		event_channel_obj = CORBA::Object::_nil();
+        	if (event_channel_obj -> _non_existent())
+                event_channel_obj = CORBA::Object::_nil();
 	
 			_eventChannel = CosNotifyChannelAdmin::EventChannel::_nil();
  			_eventChannel = CosNotifyChannelAdmin::EventChannel::_narrow(event_channel_obj);
@@ -463,7 +496,7 @@ void EventSupplier::connect_to_notifd(NotifService &ns,CORBA::ORB_var &_orb,
 
 		try
 		{
-	    		_eventChannel = _eventChannelFactory -> create_channel(initialQoS,
+	    	_eventChannel = _eventChannelFactory -> create_channel(initialQoS,
 						      initialAdmin,
 						      channelId);
 			cout4 << "Tango::EventSupplier::create() channel for server " << d_name << " created\n";
@@ -472,18 +505,19 @@ void EventSupplier::connect_to_notifd(NotifService &ns,CORBA::ORB_var &_orb,
 
 			if (Util::_FileDb == false)
 			{
-        			Tango::DevVarStringArray *dev_export_list = new Tango::DevVarStringArray;
-        			dev_export_list->length(5);
-        			(*dev_export_list)[0] = CORBA::string_dup(d_name.c_str());
-        			(*dev_export_list)[1] = CORBA::string_dup(ior_string.c_str());
-        			(*dev_export_list)[2] = CORBA::string_dup(host_name.c_str());
-        			ostringstream ostream;
-        			ostream << getpid() << ends;
-        			(*dev_export_list)[3] = CORBA::string_dup(ostream.str().c_str());
-        			(*dev_export_list)[4] = CORBA::string_dup("1");
-        			send <<= dev_export_list;
-                		received = db->command_inout("DbExportEvent",send);
-                		cout4 << "successfully  exported event channel to Tango database !\n";
+        		Tango::DevVarStringArray *eve_export_list = new Tango::DevVarStringArray;
+        		eve_export_list->length(5);
+        		(*eve_export_list)[0] = CORBA::string_dup(d_name.c_str());
+    			(*eve_export_list)[1] = CORBA::string_dup(ior_string.c_str());
+    			(*eve_export_list)[2] = CORBA::string_dup(host_name.c_str());
+    			ostringstream ostream;
+    			ostream << getpid() << ends;
+    			(*eve_export_list)[3] = CORBA::string_dup(ostream.str().c_str());
+    			(*eve_export_list)[4] = CORBA::string_dup("1");
+
+				db->export_event(eve_export_list);
+
+        		cout4 << "successfully  exported event channel to Tango database !\n";
 			}
 			else
 			{
@@ -525,15 +559,15 @@ void EventSupplier::connect_to_notifd(NotifService &ns,CORBA::ORB_var &_orb,
 // We'll use the channel's default Supplier admin
 //
 
-        CosNotifyChannelAdmin::SupplierAdmin_var 
+	CosNotifyChannelAdmin::SupplierAdmin_var 
 		_supplierAdmin = _eventChannel -> default_supplier_admin();
 	if (CORBA::is_nil(_supplierAdmin))
 	{
-        	cerr << "Could not get CosNotifyChannelAdmin::SupplierAdmin" << endl;
+        cerr << "Could not get CosNotifyChannelAdmin::SupplierAdmin" << endl;
 		EventSystemExcept::throw_exception((const char*)"API_NotificationServiceFailed",
 			(const char*)"Failed to get the default supplier admin from the notification daemon (hint: make sure the notifd process is running on this host)",
 			(const char*)"EventSupplier::create()");    
-    	}
+    }
 
 //
 // Obtain a Proxy Consumer
@@ -547,16 +581,14 @@ void EventSupplier::connect_to_notifd(NotifService &ns,CORBA::ORB_var &_orb,
 	CosNotifyChannelAdmin::ProxyConsumer_var _proxyConsumer;
 	try
 	{
-	        _proxyConsumer =
-		_supplierAdmin -> obtain_notification_push_consumer(
-		    CosNotifyChannelAdmin::STRUCTURED_EVENT, _proxyId);
+	    _proxyConsumer = _supplierAdmin -> obtain_notification_push_consumer(CosNotifyChannelAdmin::STRUCTURED_EVENT, _proxyId);
 		if (CORBA::is_nil(_proxyConsumer))
 		{
-        		cerr << "Could not get CosNotifyChannelAdmin::ProxyConsumer" << endl;
+        	cerr << "Could not get CosNotifyChannelAdmin::ProxyConsumer" << endl;
 			EventSystemExcept::throw_exception((const char*)"API_NotificationServiceFailed",
 				(const char*)"Failed to obtain a Notification push consumer, make sure the notifd process is running on this host",
 				(const char*)"EventSupplier::create()");    
-    		}
+    	}
 	}
 	catch(const CosNotifyChannelAdmin::AdminLimitExceeded&)
 	{
@@ -571,8 +603,8 @@ void EventSupplier::connect_to_notifd(NotifService &ns,CORBA::ORB_var &_orb,
 		CosNotifyChannelAdmin::StructuredProxyPushConsumer::_narrow(_proxyConsumer);
 	if (CORBA::is_nil(_structuredProxyPushConsumer))
 	{
-       		cerr << "Tango::EventSupplier::create() could not get CosNotifyChannelAdmin::StructuredProxyPushConsumer" << endl;
-    	}
+       	cerr << "Tango::EventSupplier::create() could not get CosNotifyChannelAdmin::StructuredProxyPushConsumer" << endl;
+    }
 	
 //
 // Init returned value
@@ -735,7 +767,7 @@ void EventSupplier::reconnect_notifd()
 		connect_to_notifd(ns,orb_,
 				  tg->get_ds_name(),
 				  db,
-				  tg->get_host_name());
+				  tg->get_host_name(),tg);
 		
 		supplierAdmin = ns.SupAdm;
 		proxyId = ns.pID;
@@ -888,7 +920,7 @@ bool EventSupplier::detect_change_3(Attribute &attr,
 			if (except != NULL)
 			{
 				if ( attr.ext->prev_change_event.err == true )
-				{	
+				{
 					if ( Except::compare_exception (*except, attr.ext->prev_change_event.except) == true )
 					{
 						force_change = false;
@@ -1481,6 +1513,9 @@ bool EventSupplier::detect_change_3(Attribute &attr,
 						}
 					}
 					break;
+					
+				default:
+					break;
 				}
 			}
 		}	
@@ -1633,7 +1668,7 @@ void EventSupplier::detect_and_push_archive_event_3(DeviceImpl *device_impl,
 
 		cout3 << "EventSupplier::detect_and_push_archive_event(): called for attribute " << attr_name << endl;
 
-		double now_ms, ms_since_last_periodic, ms_since_last_event;
+		double now_ms, ms_since_last_periodic;
 #ifdef _TG_WINDOWS_
         struct _timeb           now_win;
 #endif
@@ -2138,6 +2173,24 @@ void EventSupplier::push_event_3(DeviceImpl *device_impl,
 	}
 
 }
-	
+
+//+----------------------------------------------------------------------------
+//
+// method : 		EventSupplier::disconnect_from_notifd()
+// 
+// description : 	Method to disconnect the DS from the notifd event channel
+//
+//-----------------------------------------------------------------------------
+
+void EventSupplier::disconnect_from_notifd()
+{
+	try
+	{
+		structuredProxyPushConsumer->disconnect_structured_push_consumer();
+	}
+	catch(CORBA::Exception &e) {}
+}
+
+
 } /* End of Tango namespace */
 
