@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: except.cpp,v 3.8 2007/06/06 09:37:58 jensmeyer Exp $\n$Name:  $";
+static const char *RcsId = "$Id: except.cpp,v 3.10 2008/03/11 14:38:25 taurel Exp $\n$Name:  $";
 
 //+=============================================================================
 //
@@ -11,9 +11,15 @@ static const char *RcsId = "$Id: except.cpp,v 3.8 2007/06/06 09:37:58 jensmeyer
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.8 $
+// $Revision: 3.10 $
 //
 // $Log: except.cpp,v $
+// Revision 3.10  2008/03/11 14:38:25  taurel
+// - Apply patches from Frederic Picca about compilation with gcc 4.2
+//
+// Revision 3.9  2007/11/23 09:47:06  taurel
+// - Fix linker problem introduced by the fix on re_throw_exception()
+//
 // Revision 3.8  2007/06/06 09:37:58  jensmeyer
 // Added null pointer check for all corba system exceptions.
 //
@@ -221,6 +227,7 @@ namespace Tango
 {
 
 char Except::mess[256];
+omni_mutex Except::the_mutex;
 
 //+----------------------------------------------------------------------------
 //
@@ -754,7 +761,7 @@ bool Except::compare_exception(Tango::DevFailed &ex1, Tango::DevFailed &ex2)
 {
 	// check the length of the exception stack
 	
-	long nb_err = ex1.errors.length();
+	unsigned long nb_err = ex1.errors.length();
 	if ( nb_err != ex2.errors.length() )
 		{
 		return false;
@@ -762,7 +769,7 @@ bool Except::compare_exception(Tango::DevFailed &ex1, Tango::DevFailed &ex2)
 	
 	// check all exceptions in the stack
 	
-	for (long i=0; i<nb_err; i++)
+	for (unsigned long i=0; i<nb_err; i++)
 		{
 		// check the severity
 		if ( ex1.errors[i].severity != ex2.errors[i].severity )
