@@ -8,9 +8,13 @@
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.3 $
+// $Revision: 3.4 $
 //
 // $Log: except.h,v $
+// Revision 3.4  2007/11/23 07:17:57  taurel
+// - Fix threading bug in re_throw_exception() when used with a
+// CORBA::SystemException
+//
 // Revision 3.3  2007/03/06 08:19:43  taurel
 // - Added 64 bits data types for 64 bits computer...
 //
@@ -154,7 +158,7 @@ namespace Tango
  * static methods
  *
  * $Author: taurel $
- * $Revision: 3.3 $
+ * $Revision: 3.4 $
  */
  
 class Except
@@ -1811,6 +1815,7 @@ public:
 //@}
 
 	static char *print_CORBA_SystemException(const CORBA::SystemException *);
+	static omni_mutex the_mutex;
 
 protected:
 	static char mess[256];
