@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: multiattribute.cpp,v 3.12 2007/04/16 14:57:42 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: multiattribute.cpp,v 3.16 2008/03/11 14:38:25 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -14,9 +14,24 @@ static const char *RcsId = "$Id: multiattribute.cpp,v 3.12 2007/04/16 14:57:42 t
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 3.12 $
+// $Revision: 3.16 $
 //
 // $Log: multiattribute.cpp,v $
+// Revision 3.16  2008/03/11 14:38:25  taurel
+// - Apply patches from Frederic Picca about compilation with gcc 4.2
+//
+// Revision 3.15  2008/01/30 10:28:47  jensmeyer
+// Corrected bug in get_w_attr_by_name method.
+// The method returned without exception also for non writabe attributes.
+//
+// Revision 3.14  2007/10/26 11:36:35  taurel
+// - Clarify some exception messages
+//
+// Revision 3.13  2007/10/16 08:23:37  taurel
+// - Add management of the TC connection establishment timeout for DB access
+// - Add DB server cache in DS used during DS startup sequence
+// - Comment out the sleep time during DS startup sequence
+//
 // Revision 3.12  2007/04/16 14:57:42  taurel
 // - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
 // - Ported to omniORB4.1
@@ -339,7 +354,7 @@ MultiAttribute::MultiAttribute(string &dev_name,DeviceClass *dev_class_ptr)
 		
 			try
 			{
-				tg->get_database()->get_device_attribute_property(dev_name,db_list);
+				tg->get_database()->get_device_attribute_property(dev_name,db_list,tg->get_db_cache());
 			}
 			catch (Tango::DevFailed &)
 			{		
@@ -349,7 +364,7 @@ MultiAttribute::MultiAttribute(string &dev_name,DeviceClass *dev_class_ptr)
 				Except::throw_exception((const char *)"API_DatabaseAccess",
 				                	o.str(),
 				                	(const char *)"MultiAttribute::MultiAttribute");
-			}		
+			}
 		}
 						
 //
@@ -744,16 +759,16 @@ void MultiAttribute::add_attribute(string &dev_name,
 		
 		try
 		{
-			tg->get_database()->get_device_attribute_property(dev_name,db_list);
+			tg->get_database()->get_device_attribute_property(dev_name,db_list,tg->get_db_cache());
 		}
-		catch (Tango::DevFailed &)
+		catch (Tango::DevFailed &e)
 		{
 			TangoSys_OMemStream o;
 			o << "Can't get device attribute properties for device " << dev_name << ends;
 					
-			Except::throw_exception((const char *)"API_DatabaseAccess",
+			Except::re_throw_exception(e,(const char *)"API_DatabaseAccess",
 				       		 o.str(),
-				        	(const char *)"MultiAttribute::MultiAttribute");
+				        	(const char *)"MultiAttribute::add_attribute");
 		}
 	}
 
@@ -931,7 +946,7 @@ void MultiAttribute::remove_attribute(string &attr_name)
 //
 
 	writable_attr_list.clear();
-	for (long i = 0;i < attr_list.size();i++)
+	for (unsigned long i = 0;i < attr_list.size();i++)
 	{
 		Tango::AttrWriteType w_type = attr_list[i]->get_writable();
 		if ((w_type == Tango::WRITE) ||
@@ -946,7 +961,7 @@ void MultiAttribute::remove_attribute(string &attr_name)
 //
 
 	alarm_attr_list.clear();
-	for (long i = 0;i < attr_list.size();i++)
+	for (unsigned long i = 0;i < attr_list.size();i++)
 	{
 		if (attr_list[i]->is_alarmed().any() == true)
 		{
@@ -961,7 +976,7 @@ void MultiAttribute::remove_attribute(string &attr_name)
 //
 			
 	string default_dev_name("a/b/c");
-	for (long i = 0;i < attr_list.size();i++)
+	for (unsigned long i = 0;i < attr_list.size();i++)
 	{
 		check_associated(i,default_dev_name);
 	}
@@ -1027,12 +1042,14 @@ WAttribute &MultiAttribute::get_w_attr_by_name(const char *attr_name)
 	pos = find_if(attr_list.begin(),attr_list.end(),
 		      bind2nd(WantedAttr<Attribute *,const char *,bool>(),attr_name));
 		      
-	if (pos == attr_list.end())
+	if ( (    pos == attr_list.end() ) ||
+		  ( ((*pos)->get_writable() != Tango::WRITE) &&
+		    ((*pos)->get_writable() != Tango::READ_WRITE) ) )
 	{
 		cout3 << "MultiAttribute::get_w_attr_by_name throwing exception" << endl;
 		TangoSys_OMemStream o;
 		
-		o << attr_name << " attribute not found" << ends;
+		o << attr_name << " writable attribute not found" << ends;
 		Except::throw_exception((const char *)"API_AttrNotFound",
 				      o.str(),
 				      (const char *)"MultiAttribute::get_w_attr_by_name");
