@@ -10,9 +10,12 @@
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 1.2 $
+// $Revision: 1.3 $
 //
 // $Log: pollext.h,v $
+// Revision 1.3  2007/04/20 14:41:33  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
 // Revision 1.2  2005/01/13 08:31:51  taurel
 // - Merge trunk with Release_5_0 from brach Release_5_branch
 //
@@ -126,7 +129,7 @@ public:
  * This class is used to store one element of an attribute history stack.
  *
  * $Author: taurel $
- * $Revision: 1.2 $
+ * $Revision: 1.3 $
  */
  
 template <typename T>
@@ -405,7 +408,7 @@ public:
 	TimedAttrData(DevErrorList &errs,timeval when): AttrData<T>(errs),t_val(when) {}
 //@}
 	
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 
 	TimedAttrData(const T *p,struct _timeb t): AttrData<T>(p) {t_val.tv_sec = t.time;t_val.tv_usec = t.millitm*1000;}	
 	TimedAttrData(const T *p,Tango::AttrQuality q,struct _timeb t): AttrData<T>(p,q) {t_val.tv_sec = t.time;t_val.tv_usec = t.millitm*1000;}
@@ -445,7 +448,7 @@ public:
  * will be used to store one element of the attribute polling buffer
  *
  * $Author: taurel $
- * $Revision: 1.2 $
+ * $Revision: 1.3 $
  */
  
 template <typename T>
@@ -517,7 +520,7 @@ public:
  * This class is used to store one element of a command history stack.
  *
  * $Author: taurel $
- * $Revision: 1.2 $
+ * $Revision: 1.3 $
  */
  
 template <typename T>
@@ -573,7 +576,7 @@ public:
  */
 	TimedCmdData(T *p_data,bool rel,struct timeval when): ptr(p_data),t_val(when),release(rel) {}
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	TimedCmdData(T *p,struct _timeb t): ptr(p),release(false) {t_val.tv_sec = t.time;t_val.tv_usec = t.millitm*1000;} 
 	TimedCmdData(T *p,bool rel,struct _timeb t): ptr(p),release(rel) {t_val.tv_sec = t.time;t_val.tv_usec = t.millitm*1000;}
 #endif
@@ -623,7 +626,7 @@ public:
  * will be used to store one element of the command polling buffer
  *
  * $Author: taurel $
- * $Revision: 1.2 $
+ * $Revision: 1.3 $
  */
  
 template <typename T>
