@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: pollring.cpp,v 3.8 2007/04/20 14:41:33 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: pollring.cpp,v 3.9 2008/03/26 16:50:08 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -14,9 +14,12 @@ static const char *RcsId = "$Id: pollring.cpp,v 3.8 2007/04/20 14:41:33 taurel E
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 3.8 $
+// $Revision: 3.9 $
 //
 // $Log: pollring.cpp,v $
+// Revision 3.9  2008/03/26 16:50:08  taurel
+// - Fix bug when getting history for State pseudo-attribute
+//
 // Revision 3.8  2007/04/20 14:41:33  taurel
 // - Ported to Windows 64 bits x64 architecture
 //
@@ -1132,12 +1135,26 @@ void PollRing::get_attr_history(long n,Tango::DevAttrHistoryList_3 *ptr,long typ
 					break;
 										
 				case Tango::DEV_STATE :
-					(*ring[index].attr_value_3)[0].value >>= tmp_state;
-					new_tmp_state = new DevVarStateArray(tmp_state->length(),tmp_state->length(),
-							const_cast<DevState *>(tmp_state->get_buffer()),false);
-					(*ptr)[seq_index].value.value <<= new_tmp_state;
+				{
+					CORBA::TypeCode_var ty;
+					ty = (*ring[index].attr_value_3)[0].value.type();
+			
+					if (ty->kind() == CORBA::tk_enum)
+					{
+						Tango::DevState tmp_ds;
+						(*ring[index].attr_value_3)[0].value >>= tmp_ds;
+						(*ptr)[seq_index].value.value <<= tmp_ds;
+					}
+					else
+					{
+						(*ring[index].attr_value_3)[0].value >>= tmp_state;
+						new_tmp_state = new DevVarStateArray(tmp_state->length(),tmp_state->length(),
+									const_cast<DevState *>(tmp_state->get_buffer()),false);
+						(*ptr)[seq_index].value.value <<= new_tmp_state;
+					}
 					break;
 				}
+				}
 			}
 		}
 		else			
