@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: pollthread.cpp,v 3.12 2006/05/18 08:51:56 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: pollthread.cpp,v 3.17 2007/05/15 07:46:59 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -13,9 +13,29 @@ static const char *RcsId = "$Id: pollthread.cpp,v 3.12 2006/05/18 08:51:56 taure
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 3.12 $
+// $Revision: 3.17 $
 //
 // $Log: pollthread.cpp,v $
+// Revision 3.17  2007/05/15 07:46:59  taurel
+// - The polling thread is not configured by a separate thread any more.
+// The Add_obj_polling command now support a delta_t to start the first polling
+//
+// Revision 3.16  2007/04/20 14:41:33  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
+// Revision 3.15  2007/04/16 14:57:42  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
+// Revision 3.14  2007/03/29 07:09:25  taurel
+// - Change some data types for 64 bits compatibility
+//
+// Revision 3.13  2007/03/02 09:44:03  jensmeyer
+// Corrected the use of detect_and_push event methods. Uses now the _3 methods
+// for idl_vers>3 everywhere.
+//
 // Revision 3.12  2006/05/18 08:51:56  taurel
 // - Miscellaneous changes due to Python device server ported to Windows
 // - Fix some bugs discovered by Windows VC8 using the test suite
@@ -209,13 +229,13 @@ static const char *RcsId = "$Id: pollthread.cpp,v 3.12 2006/05/18 08:51:56 taure
 #include <eventsupplier.h>
 #include <math.h>
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	#include <sys/timeb.h>
 #else
 	#include <sys/time.h>
 #endif 
 
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 	#include <iomanip>
 #else
 	#include <iomanip.h>
@@ -244,7 +264,7 @@ PollThread::PollThread(PollThCmd &cmd,TangoMonitor &m): shared_cmd(cmd),p_mon(m)
 {
 	attr_names.length(1);
 	
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	LARGE_INTEGER f;
 	BOOL is_ctr;
 	is_ctr = QueryPerformanceFrequency(&f);
@@ -294,7 +314,7 @@ void *PollThread::run_undetached(void *ptr)
 				per_thread_data_created = true;
 			}
 				
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 			_ftime(&now_win);
 			now.tv_sec = (unsigned long)now_win.time;
 			now.tv_usec = (long)now_win.millitm * 1000;
@@ -318,7 +338,7 @@ void *PollThread::run_undetached(void *ptr)
 				break;
 			}
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 			_ftime(&after_win);
 			after.tv_sec = (unsigned long)after_win.time;
 			after.tv_usec = (long)after_win.millitm * 1000;
@@ -345,7 +365,7 @@ void *PollThread::run_undetached(void *ptr)
 		catch (omni_thread_fatal &)
 		{
 			cerr << "OUPS !! A omni thread fatal exception !!!!!!!!" << endl;
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 			time_t t = time(NULL);
 			cerr << ctime(&t);
 #endif
@@ -467,6 +487,11 @@ void PollThread::execute_cmd()
 		if (wo.update != 0)
 		{
 			wo.wake_up_date = now;
+			if (local_cmd.new_upd != 0)
+			{
+				cout5 << "Received a delta from now of " << local_cmd.new_upd << endl;
+				T_ADD(wo.wake_up_date,local_cmd.new_upd * 1000);
+			}
 //			add_random_delay(wo.wake_up_date);
 			insert_in_list(wo);
 			unsigned long nb_works = works.size();
@@ -492,7 +517,7 @@ void PollThread::execute_cmd()
 		name_to_del = local_cmd.name;
 		type_to_del = local_cmd.type;
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 		unsigned int i,nb_elt;
 		nb_elt = works.size();
 		ite = works.begin();
@@ -550,7 +575,7 @@ void PollThread::execute_cmd()
 		cout5 << "Received a Rem device command" << endl;
 		
 		dev_to_del = local_cmd.dev;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 		nb_elt = works.size();
 		ite = works.begin();
 		for (i = 0;i < nb_elt;i++)
@@ -616,7 +641,7 @@ void PollThread::execute_cmd()
 			if (local_cmd.new_upd != 0)
 			{
 				WorkItem tmp_work = *ite;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 				unsigned int i,nb_elt;
 				nb_elt = works.size();
 				ite = works.begin();
@@ -651,7 +676,7 @@ void PollThread::execute_cmd()
 // triggered list
 //
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 				unsigned int i,nb_elt;
 				nb_elt = works.size();
 				ite = works.begin();
@@ -1160,11 +1185,11 @@ void PollThread::tune_list(bool from_needed, long min_delta)
 // description : 	This method computes the new poll date.
 //
 // argument: In :	- time : The actual date
-//			- upd : The polling update period (mS)
+//					- upd : The polling update period (mS)
 //
 //--------------------------------------------------------------------------
 
-void PollThread::compute_new_date(struct timeval &time,long upd)
+void PollThread::compute_new_date(struct timeval &time,int upd)
 {
 	double ori_d = (double)time.tv_sec + ((double)time.tv_usec / 1000000);
 	double new_d = ori_d + ((double)(upd) / 1000);
@@ -1271,7 +1296,14 @@ void PollThread::err_out_of_sync(WorkItem &to_do)
 		Tango::DevFailed except(errs);
 		long idl_vers = to_do.dev->get_dev_idl_version();
 			
-		event_supplier->detect_and_push_events(to_do.dev,
+		if (idl_vers >= 3)
+			event_supplier->detect_and_push_events_3(to_do.dev,
+						       idl_vers,
+						       dummy_att3,
+						       &except,
+						       to_do.name);
+		else	
+			event_supplier->detect_and_push_events(to_do.dev,
 						       idl_vers,
 						       dummy_att,
 						       &except,
@@ -1302,7 +1334,7 @@ void PollThread::poll_cmd(WorkItem &to_do)
 	CORBA::Any *argout = NULL;
 	Tango::DevFailed *save_except = NULL;
 	struct timeval before_cmd,after_cmd,needed_time;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	struct _timeb before_win,after_win;
 	LARGE_INTEGER before,after;
 #endif
@@ -1311,7 +1343,7 @@ void PollThread::poll_cmd(WorkItem &to_do)
 	bool cmd_failed = false;
 	try
 	{
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 		if (ctr_frequency != 0)
 			QueryPerformanceCounter(&before);
 		_ftime(&before_win);
@@ -1328,7 +1360,7 @@ void PollThread::poll_cmd(WorkItem &to_do)
 
 		argout = to_do.dev->command_inout(to_do.name.c_str(),in_any);
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 		if (ctr_frequency != 0)
 		{
 			QueryPerformanceCounter(&after);
@@ -1357,7 +1389,7 @@ void PollThread::poll_cmd(WorkItem &to_do)
 	catch (Tango::DevFailed &e)
 	{
 		cmd_failed = true;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 		if (ctr_frequency != 0)
 		{
 			QueryPerformanceCounter(&after);
@@ -1429,7 +1461,7 @@ void PollThread::poll_attr(WorkItem &to_do)
         << ", Attr name = " << to_do.name << endl;
 	
 	struct timeval before_cmd,after_cmd,needed_time;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	struct _timeb before_win,after_win;
 	LARGE_INTEGER before,after;
 #endif
@@ -1442,7 +1474,7 @@ void PollThread::poll_attr(WorkItem &to_do)
 	long idl_vers = to_do.dev->get_dev_idl_version();
 	try
 	{
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 		if (ctr_frequency != 0)
 			QueryPerformanceCounter(&before);
 		_ftime(&before_win);
@@ -1463,7 +1495,7 @@ void PollThread::poll_attr(WorkItem &to_do)
 		else		
 			argout = to_do.dev->read_attributes(attr_names);
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 		if (ctr_frequency != 0)
 		{
 			QueryPerformanceCounter(&after);
@@ -1492,7 +1524,7 @@ void PollThread::poll_attr(WorkItem &to_do)
 	catch (Tango::DevFailed &e)
 	{
 		attr_failed = true;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 		if (ctr_frequency != 0)
 		{
 			QueryPerformanceCounter(&after);
@@ -1549,8 +1581,15 @@ void PollThread::poll_attr(WorkItem &to_do)
 	if (event_supplier != NULL)
 	{
 		if (attr_failed == true)
-		{			
-			event_supplier->detect_and_push_events(to_do.dev,
+		{		
+			if (idl_vers >= 3)
+				event_supplier->detect_and_push_events_3(to_do.dev,
+						               idl_vers,
+						       	       dummy_att3,
+						               save_except,
+						               to_do.name);
+			else	
+				event_supplier->detect_and_push_events(to_do.dev,
 						               idl_vers,
 						       	       dummy_att,
 						               save_except,
