@@ -10,9 +10,18 @@
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 3.2 $
+// $Revision: 3.4 $
 //
 // $Log: seqvec.h,v $
+// Revision 3.4  2007/04/16 14:57:42  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
+// Revision 3.3  2007/03/06 08:19:03  taurel
+// - Added 64 bits data types for 64 bits computer...
+//
 // Revision 3.2  2006/05/18 08:51:56  taurel
 // - Miscellaneous changes due to Python device server ported to Windows
 // - Fix some bugs discovered by Windows VC8 using the test suite
@@ -225,12 +234,12 @@ inline void operator<<(vector<short> &lval,const DevVarShortArray &rval)
 //=============================================================================
 
 /**
- * Init a DevVarLongArray from a C++ vector of long(s).
+ * Init a DevVarLongArray from a C++ vector of DevLong(s).
  *
  * @param lval The DevVarLongArray to be initialised
  * @param rval The C++ vector
  */
-inline void operator<<(DevVarLongArray &lval,const vector<long> &rval)
+inline void operator<<(DevVarLongArray &lval,const vector<DevLong> &rval)
 {
 	size_t nb_elt = rval.size();
 	lval.length((CORBA::ULong)nb_elt);
@@ -239,12 +248,51 @@ inline void operator<<(DevVarLongArray &lval,const vector<long> &rval)
 }
 
 /**
- * Init a C++ vector of long from a DevVarLongArray.
+ * Init a C++ vector of DevLong from a DevVarLongArray.
  *
  * @param lval The C++ vector to be initialised
  * @param rval The DevVarLongArray
  */
-inline void operator<<(vector<long> &lval,const DevVarLongArray &rval)
+inline void operator<<(vector<DevLong> &lval,const DevVarLongArray &rval)
+{
+	long nb_elt = rval.length();
+	if (lval.size() != 0)
+		lval.clear();
+	for (long i = 0;i < nb_elt;i++)
+		lval.push_back(rval[i]);
+}
+
+
+//=============================================================================
+//
+//			Operator for the Long Long array types
+//
+// description :	These two operators allow simple insertion between
+//			Tango sequence (DevVarLong64Array) and C++ vectors
+//
+//=============================================================================
+
+/**
+ * Init a DevVarLongArray from a C++ vector of DevLong64(s).
+ *
+ * @param lval The DevVarLong64Array to be initialised
+ * @param rval The C++ vector
+ */
+inline void operator<<(DevVarLong64Array &lval,const vector<DevLong64> &rval)
+{
+	size_t nb_elt = rval.size();
+	lval.length((CORBA::ULong)nb_elt);
+	for (unsigned long i = 0;i < nb_elt;i++)
+		lval[i] = rval[i];
+}
+
+/**
+ * Init a C++ vector of long from a DevVarLong64Array.
+ *
+ * @param lval The C++ vector to be initialised
+ * @param rval The DevVarLong64Array
+ */
+inline void operator<<(vector<DevLong64> &lval,const DevVarLong64Array &rval)
 {
 	long nb_elt = rval.length();
 	if (lval.size() != 0)
@@ -420,13 +468,13 @@ inline void operator<<(vector<unsigned short> &lval,const DevVarUShortArray &rva
 //=============================================================================
 
 /**
- * Init a DevVarULongArray from a C++ vector of unsigned long(s).
+ * Init a DevVarULongArray from a C++ vector of DevULong(s).
  *
  * @param lval The DevVarULongArray to be initialised
  * @param rval The C++ vector
  */
  
-inline void operator<<(DevVarULongArray &lval,const vector<unsigned long> &rval)
+inline void operator<<(DevVarULongArray &lval,const vector<DevULong> &rval)
 {
 	size_t nb_elt = rval.size();
 	lval.length((CORBA::ULong)nb_elt);
@@ -440,7 +488,47 @@ inline void operator<<(DevVarULongArray &lval,const vector<unsigned long> &rval)
  * @param lval The C++ vector to be initialised
  * @param rval The DevVarULongArray
  */
-inline void operator<<(vector<unsigned long> &lval,const DevVarULongArray &rval)
+inline void operator<<(vector<DevULong> &lval,const DevVarULongArray &rval)
+{
+	long nb_elt = rval.length();
+	if (lval.size() != 0)
+		lval.clear();
+	for (long i = 0;i < nb_elt;i++)
+		lval.push_back(rval[i]);
+}
+
+
+//=============================================================================
+//
+//			Operator for the Unsigned long long array types
+//
+// description :	These two operators allow simple insertion between
+//			Tango sequence (DevVarULong64Array) and C++ vectors
+//
+//=============================================================================
+
+/**
+ * Init a DevVarULong64Array from a C++ vector of DevULong64(s).
+ *
+ * @param lval The DevVarULong64Array to be initialised
+ * @param rval The C++ vector
+ */
+ 
+inline void operator<<(DevVarULong64Array &lval,const vector<DevULong64> &rval)
+{
+	size_t nb_elt = rval.size();
+	lval.length((CORBA::ULong)nb_elt);
+	for (unsigned long i = 0;i < nb_elt;i++)
+		lval[i] = rval[i];
+}
+
+/**
+ * Init a C++ vector of long from a DevVarULong64Array.
+ *
+ * @param lval The C++ vector to be initialised
+ * @param rval The DevVarULong64Array
+ */
+inline void operator<<(vector<DevULong64> &lval,const DevVarULong64Array &rval)
 {
 	long nb_elt = rval.length();
 	if (lval.size() != 0)
@@ -491,6 +579,43 @@ inline void operator<<(vector<string> &lval,const DevVarStringArray &rval)
 	}
 }
 		
+//=============================================================================
+//
+//			Operator for the DevState array types
+//
+// description :	These two operators allow simple insertion between
+//			Tango sequence (DevVarStateArray) and C++ vectors
+//
+//=============================================================================
+
+/**
+ * Init a DevVarStateArray from a C++ vector of state(s).
+ *
+ * @param lval The DevVarStateArray to be initialised
+ * @param rval The C++ vector
+ */
+inline void operator<<(DevVarStateArray &lval,const vector<DevState> &rval)
+{
+	size_t nb_elt = rval.size();
+	lval.length((CORBA::ULong)nb_elt);
+	for (unsigned long i = 0;i < nb_elt;i++)
+		lval[i] = rval[i];
+}
+
+/**
+ * Init a C++ vector of long from a DevVarStateArray.
+ *
+ * @param lval The C++ vector to be initialised
+ * @param rval The DevVarStateArray
+ */
+inline void operator<<(vector<DevState> &lval,const DevVarStateArray &rval)
+{
+	long nb_elt = rval.length();
+	if (lval.size() != 0)
+		lval.clear();
+	for (long i = 0;i < nb_elt;i++)
+		lval.push_back(rval[i]);
+}
 
 //@}
 
@@ -532,6 +657,15 @@ ostream &operator<<(ostream &lval,const DevVarShortArray &rval);
  * @param rval The DevVarLongArray sequence to be printed
  */
 ostream &operator<<(ostream &lval,const DevVarLongArray &rval);
+/**
+ * Print a DevVarLong64Array.
+ *
+ * One line is printed for each sequence element.
+ *
+ * @param lval The C++ stream used for printing
+ * @param rval The DevVarLong64Array sequence to be printed
+ */
+ostream &operator<<(ostream &lval,const DevVarLong64Array &rval);
 /**
  * Print a DevVarFloatArray.
  *
@@ -577,6 +711,15 @@ ostream &operator<<(ostream &lval,const DevVarUShortArray &rval);
  * @param rval The DevVarULongArray sequence to be printed
  */
 ostream &operator<<(ostream &lval,const DevVarULongArray &rval);
+/**
+ * Print a DevVarULong64Array.
+ *
+ * One line is printed for each sequence element.
+ *
+ * @param lval The C++ stream used for printing
+ * @param rval The DevVarULong64Array sequence to be printed
+ */
+ostream &operator<<(ostream &lval,const DevVarULong64Array &rval);
 /**
  * Print a DevVarStringArray.
  *
@@ -586,6 +729,15 @@ ostream &operator<<(ostream &lval,const DevVarULongArray &rval);
  * @param rval The DevVarStringArray sequence to be printed
  */
 ostream &operator<<(ostream &lval,const DevVarStringArray &rval);
+/**
+ * Print a DevVarStateArray.
+ *
+ * One line is printed for each sequence element.
+ *
+ * @param lval The C++ stream used for printing
+ * @param rval The DevVarStateArray sequence to be printed
+ */
+ostream &operator<<(ostream &lval,const DevVarStateArray &rval);
 //@}
 
 //#endif // TANGO_HAS_LOG4TANGO
