@@ -8,9 +8,21 @@
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.9 $
+// $Revision: 3.12 $
 //
 // $Log: tango.h,v $
+// Revision 3.12  2007/04/20 14:41:34  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
+// Revision 3.11  2007/04/16 14:57:42  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
+// Revision 3.10  2007/03/06 08:19:03  taurel
+// - Added 64 bits data types for 64 bits computer...
+//
 // Revision 3.9  2006/05/18 07:40:11  taurel
 // - Updated to gcc 4
 // - Added a define for Windows VC8 (WIN32_VC8)
@@ -156,7 +168,7 @@
 // Check Win32 VC release
 //
 
-#ifdef WIN32
+#if ((defined WIN32) || (defined WIN64))
 	#if (_MSC_VER >= 1400)       // VC8+ 
 		#define WIN32_VC8
 	#endif   // VC8+
@@ -180,18 +192,20 @@
 //
 // Include IDL generated files which includes CORBA include files
 //
+// Please note that on Windows, this file include Windows include file.
+// The Windows Windef.h file defines a Preprocessor WIN32 on top of the
+// _WIN32 one.
+// This measn that on Windows 64 bits, we will have BOTH WIN32 and WIN64
+// defined even if on the compiler command line we define only WIN64
+//
 
-#ifdef TANGO_LONG64
-#include <idl_64/tango.h>
-#else
 #include <idl/tango.h>
-#endif
 
 //
 // Some Windows specific include (necessary when used with MFC)
 //
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	#if (_WIN32_WINNT >= 0x0400)
 		#include <winsock2.h>
 		#include <mswsock.h>
@@ -204,7 +218,7 @@
 // Include stream header files
 //
 
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 	#include <iostream>
 	#include <sstream>
 	#include <fstream>
