@@ -9,9 +9,18 @@
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.11 $
+// $Revision: 3.14 $
 //
 // $Log: tango_config.h,v $
+// Revision 3.14  2008/01/08 14:38:19  taurel
+// - strcasecmp() is not named like this on Windows !!
+//
+// Revision 3.13  2007/12/12 10:17:18  taurel
+// - Db calls during DS startup has a separate timeout and some retries
+//
+// Revision 3.12  2007/11/23 07:16:48  taurel
+// - Add Claudio's modif
+//
 // Revision 3.11  2007/04/20 14:41:34  taurel
 // - Ported to Windows 64 bits x64 architecture
 //
@@ -290,6 +299,18 @@
 #define Tango_sleep(A) Sleep(A * 1000);
 #endif
 
+//
+// Define a common strcasecmp function
+//
+
+#ifndef _TG_WINDOWS_
+#define TG_strcasecmp ::strcasecmp
+#define TG_strncasecmp ::strncasecmp
+#else
+#define	TG_strcasecmp ::stricmp
+#define TG_strncasecmp ::strnicmp
+#endif
+
 //
 // ACTIVATE (or DEACTIVATE) THE TANGO LOGGING MECHANISM
 //
@@ -309,6 +330,26 @@
 #ifndef _TG_WINDOWS_
 	#include <omniORB4/acconfig.h>
 
+	#ifdef PACKAGE_BUGREPORT
+		#undef PACKAGE_BUGREPORT
+	#endif
+	
+	#ifdef PACKAGE_NAME
+		#undef PACKAGE_NAME
+	#endif
+	
+	#ifdef PACKAGE_STRING
+		#undef PACKAGE_STRING
+	#endif
+
+	#ifdef PACKAGE_TARNAME
+		#undef PACKAGE_TARNAME
+	#endif
+
+	#ifdef PACKAGE_VERSION
+		#undef PACKAGE_VERSION
+	#endif
+
 	#if SIZEOF_LONG == 8
 		#define TANGO_LONG64
 	#else
