@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: thsig.cpp,v 3.9 2007/04/20 14:41:34 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: thsig.cpp,v 3.14 2008/03/14 11:56:03 taurel Exp $\n$Name:  $";
 
 //+=============================================================================
 //
@@ -14,7 +14,7 @@ static const char *RcsId = "$Id: thsig.cpp,v 3.9 2007/04/20 14:41:34 taurel Exp
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.9 $
+// $Revision: 3.14 $
 //
 // copyleft :           European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
@@ -28,6 +28,7 @@ static const char *RcsId = "$Id: thsig.cpp,v 3.9 2007/04/20 14:41:34 taurel Exp
 
 #include <tango.h>
 #include <dserversignal.h>
+#include <eventsupplier.h>
 
 extern omni_thread::key_t key_py_data;
 namespace Tango
@@ -176,15 +177,22 @@ void *DServerSignal::ThSig::run_undetached(void *ptr)
 				
 //
 // Stopping a device server means :
+//		- Mark the server as shutting down
 //  	- Send kill command to the polling thread
-//    	- Join with this polling thread
-//	- Unregister server in database
-//	- Delete devices (except the admin one)
-//	- Force writing file database in case of 
-//	- Shutdown the ORB
-//	- Cleanup Logging
+//    - Join with this polling thread
+//	   - Unregister server in database
+//	   - Delete devices (except the admin one)
+//    - Stop the KeepAliveThread and the EventConsumer Thread when 
+//      they have been started to receive events
+//	   - Force writing file database in case of 
+//	   - Shutdown the ORB
+//	   - Cleanup Logging
 //
 
+					tg->set_svr_shutting_down(true);
+					
+					// send the exit command to the polling thread
+					
 					TangoMonitor &mon = tg->get_poll_monitor();
 					PollThCmd &shared_cmd = tg->get_poll_shared_cmd();
 
@@ -197,10 +205,21 @@ void *DServerSignal::ThSig::run_undetached(void *ptr)
 						mon.signal();
 
 					}
-					tg->get_polling_thread_object()->join(0);
-					tg->clr_poll_th_ptr();
-      	   				tg->unregister_server();
+					// join with the polling thread
 					
+					void *dummy_ptr;
+					tg->get_polling_thread_object()->join(&dummy_ptr);
+					
+					// Unregister the server in the database 
+					tg->clr_poll_th_ptr();
+					try
+					{
+      	   				tg->unregister_server();
+					}
+					catch(...) {}
+					
+					// Delete the devices (except the admin one)
+					// Protect python data 
 					omni_thread::value_t *tmp_py_data = omni_thread::self()->get_value(key_py_data);
 					PyLock *lock_ptr = (static_cast<PyData *>(tmp_py_data))->PerTh_py_lock;
 					lock_ptr->Get();
@@ -208,12 +227,31 @@ void *DServerSignal::ThSig::run_undetached(void *ptr)
 					tg->get_dserver_device()->delete_devices();
 				
 					lock_ptr->Release();
+					
+					// 	Stop the KeepAliveThread and the EventConsumer Thread when 
+					//  	they have been started to receive events.					
+					
+					EventConsumer *ec = ApiUtil::instance()->get_event_consumer();
+					if (ec != NULL)
+						ec->disconnect_from_notifd();					
+					
+					// diconnect the server from the nofifd, when it was connected
+					
+					EventSupplier *ev = tg->get_event_supplier();
+					if (ev != NULL)
+						ev->disconnect_from_notifd();
+						
+					// close access to file database when used
+						
 					if (tg->_FileDb == true)
 					{
 						Database *db_ptr = tg->get_database();
 						delete db_ptr;
 						cout4 << "Database object deleted" << endl;
 					}
+					
+					// shutdown the ORB
+					
 					cout4 << "Going to shutdown ORB" << endl;
 					CORBA::ORB_ptr orb = tg->get_orb();
 					orb->shutdown(true);
@@ -221,6 +259,8 @@ void *DServerSignal::ThSig::run_undetached(void *ptr)
 					cout4 << "ORB shutdown" << endl;
 
 #ifdef TANGO_HAS_LOG4TANGO
+					// clean-up the logging system
+					
 	  				Logging::cleanup();
 					cout4 << "Logging cleaned-up" << endl;
 #endif
