@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: utils.cpp,v 3.29 2007/05/17 07:59:07 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: utils.cpp,v 3.40 2008/03/26 16:51:12 taurel Exp $\n$Name:  $";
 
 //+=============================================================================
 //
@@ -11,9 +11,50 @@ static const char *RcsId = "$Id: utils.cpp,v 3.29 2007/05/17 07:59:07 taurel Exp
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.29 $
+// $Revision: 3.40 $
 //
 // $Log: utils.cpp,v $
+// Revision 3.40  2008/03/26 16:51:12  taurel
+// - Change the printed message when a DS starts (on cout instead of cerr)
+//
+// Revision 3.39  2008/03/25 14:59:01  taurel
+// - Fix bug in the previous change
+//
+// Revision 3.38  2008/03/25 13:32:25  taurel
+// - Change when the RootPOA CORBA object is created. This allows DS to
+// start on windows in daemon mode even if the PC is not connected to the
+// network at boot time
+//
+// Revision 3.37  2008/03/14 11:56:03  taurel
+// - Fix some gcc 4.2 warnings
+//
+// Revision 3.36  2008/02/29 12:53:08  taurel
+// - Increase timeout for DB get data cache command
+// - Add a method to unvalidate the cache
+//
+// Revision 3.35  2008/01/25 15:44:51  taurel
+// - Some changes in the Db cache
+// - A lighter system to shutdown DS in case of dynamic attribute
+//
+// Revision 3.34  2008/01/15 12:29:55  taurel
+// - Change db timeout when getting data for the db cache
+//
+// Revision 3.33  2007/12/12 10:17:18  taurel
+// - Db calls during DS startup has a separate timeout and some retries
+//
+// Revision 3.32  2007/11/08 12:03:45  taurel
+// - Start implementing user interceptors
+// - Fix bug in poll thread pproperty management when removing polling object
+// - Set a database timeout to 6 sec
+//
+// Revision 3.31  2007/10/26 11:27:43  taurel
+// - Add some print-out for test purposes
+//
+// Revision 3.30  2007/10/16 08:24:14  taurel
+// - Add management of the TC connection establishment timeout for DB access
+// - Add DB server cache in DS used during DS startup sequence
+// - Comment out the sleep time during DS startup sequence
+//
 // Revision 3.29  2007/05/17 07:59:07  taurel
 // - The polling is not configured via a separate thread any more. The polling thread add_obj_polling method has been modified to support a parameter telling to the polling thread when it has to polled the object.
 // Add device name in monitor print message
@@ -425,7 +466,7 @@ Util::Util(int argc,char *argv[]):cl_list_ptr(NULL)
 // Do the job
 //
 
-        effective_job(argc,argv);
+	effective_job(argc,argv);
 	_constructed = true;
 }
 
@@ -464,6 +505,12 @@ void Util::effective_job(int argc,char *argv[])
 // Initialise CORBA ORB
 //
 
+#ifdef _TG_WINDOWS_
+		WORD rel = 0x0202;
+		WSADATA dat;
+		int init_res = WSAStartup(rel,&dat);
+#endif
+
 		char h_name[80];
 		int res = gethostname(h_name,80);
 		if (res == 0)
@@ -501,50 +548,6 @@ void Util::effective_job(int argc,char *argv[])
 			orb = CORBA::ORB_init(argc,argv,"omniORB4",options);
 		}
 
-//
-// Install an omniORB interceptors to store client name in blackbox
-// and allocate a key for per thread specific storage
-//
-
-		omni::omniInterceptors *intercep = omniORB::getInterceptors();
-		intercep->serverReceiveRequest.add(get_client_addr);
-		intercep->createThread.add(create_PyPerThData);
-		
-		key = omni_thread::allocate_key();
-		key_py_data = omni_thread::allocate_key();
-				
-//
-// Get some CORBA object references
-//
-
-		CORBA::Object_var poaObj = orb->resolve_initial_references("RootPOA");
-		PortableServer::POA_var root_poa = PortableServer::POA::_narrow(poaObj);
-
-//
-// If the database is not used, we must used the omniINSPOA poa
-// in both cases of database on file or nodb
-// remember that when you have database on file you have
-//  _UseDb == true and  _FileDb == true
-//
-
-		PortableServer::POA_var nodb_poa;
-		if ((_UseDb == false) || (_FileDb == true))
-		{
-			CORBA::Object_var poaInsObj = orb->resolve_initial_references("omniINSPOA");
-			nodb_poa = PortableServer::POA::_narrow(poaInsObj);	
-		}
-		
-//
-// Store POA
-// this is the same test but inverted
-//
-
-		if ((_UseDb == true) && (_FileDb == false))
-			_poa = PortableServer::POA::_duplicate(root_poa);
-		else if ((_UseDb == false) || (_FileDb == true))
-			_poa = PortableServer::POA::_duplicate(nodb_poa);
-
-
 #ifndef TANGO_HAS_LOG4TANGO
 
 //
@@ -564,6 +567,12 @@ void Util::effective_job(int argc,char *argv[])
 # endif 
 #endif // TANGO_HAS_LOG4TANGO
 
+//
+// Init host name
+//
+		
+		init_host_name();
+		
 //
 // Connect to the database
 //
@@ -583,15 +592,23 @@ void Util::effective_job(int argc,char *argv[])
 				display_help_message();
 			}
 		}
+
+//
+// Create the server CORBA objects
+//
+
+		create_CORBA_objects();
 		
 #ifdef TANGO_HAS_LOG4TANGO	
 //
 // Initialize logging stuffs
 //
-		Logging::init(ds_name, (int)_tracelevel,  ((!_FileDb) && _UseDb), *db);
+		Logging::init(ds_name, (int)_tracelevel,  ((!_FileDb) && _UseDb), *db, this);
 #endif
 
 		cout4 << "Connected to database" << endl;
+		if (get_db_cache() == NULL)
+			cout4 << "DbServerCache unavailable, will call db..." << endl;
 
 //
 // Check if the server is not already running somewhere else
@@ -601,11 +618,11 @@ void Util::effective_job(int argc,char *argv[])
 			server_already_running();
 
 //
-// Get process PID, host name and Tango version
+// Get process PID and Tango version
 //
 
 		misc_init();
-
+						
 //
 // Automatically create the EventSupplier object
 //
@@ -636,6 +653,63 @@ void Util::effective_job(int argc,char *argv[])
 }
 
 
+//+----------------------------------------------------------------------------
+//
+// method : 		Util::create_CORBA_objects()
+// 
+// description : 	Create some CORBA objects needed later-on
+//
+//-----------------------------------------------------------------------------
+
+void Util::create_CORBA_objects()
+{
+
+//
+// Install an omniORB interceptors to store client name in blackbox
+// and allocate a key for per thread specific storage
+//
+
+	omni::omniInterceptors *intercep = omniORB::getInterceptors();
+	intercep->serverReceiveRequest.add(get_client_addr);
+	intercep->createThread.add(create_PyPerThData);
+
+	key = omni_thread::allocate_key();
+	key_py_data = omni_thread::allocate_key();
+				
+//
+// Get some CORBA object references
+//
+
+	CORBA::Object_var poaObj = orb->resolve_initial_references("RootPOA");
+	PortableServer::POA_var root_poa = PortableServer::POA::_narrow(poaObj);
+
+//
+// If the database is not used, we must used the omniINSPOA poa
+// in both cases of database on file or nodb
+// remember that when you have database on file you have
+//  _UseDb == true and  _FileDb == true
+//
+
+	PortableServer::POA_var nodb_poa;
+	if ((_UseDb == false) || (_FileDb == true))
+	{
+		CORBA::Object_var poaInsObj = orb->resolve_initial_references("omniINSPOA");
+		nodb_poa = PortableServer::POA::_narrow(poaInsObj);	
+	}
+		
+//
+// Store POA
+// this is the same test but inverted
+//
+
+	if ((_UseDb == true) && (_FileDb == false))
+		_poa = PortableServer::POA::_duplicate(root_poa);
+	else if ((_UseDb == false) || (_FileDb == true))
+		_poa = PortableServer::POA::_duplicate(nodb_poa);
+}
+
+
+
 #ifdef _TG_WINDOWS_
 //+----------------------------------------------------------------------------
 //
@@ -989,7 +1063,7 @@ void Util::display_help_message()
 // Display device server usage string
 //
 
-        TangoSys_OMemStream o;
+	TangoSys_OMemStream o;
 		
 	o << "usage :  " << ds_exec_name << " instance_name [-v[trace level]]";
 	o << " [-nodb [-dlist <device name list>]]";
@@ -1003,23 +1077,16 @@ void Util::display_help_message()
 	str.append("/*");
 
 	vector<string> db_inst;	
-	bool retry = true;
 		
-	while (retry == true)
+	try
 	{
-		try
-		{
-			DbDatum received = db->get_device_member(str);
-			received >> db_inst;
-			retry = false;
-		}
-		catch (Tango::CommunicationFailed &) {}
-		catch (Tango::DevFailed &)
-		{
-			o << ends;
-			print_err_message(o.str(),Tango::INFO);
-			retry = false;
-		}
+		DbDatum received = db->get_device_member(str);
+		received >> db_inst;
+	}
+	catch (Tango::DevFailed &)
+	{
+		o << ends;
+		print_err_message(o.str(),Tango::INFO);
 	}
 	
 //
@@ -1037,7 +1104,7 @@ void Util::display_help_message()
 // Display message
 //
 
-        print_err_message(o.str(),Tango::INFO);
+	print_err_message(o.str(),Tango::INFO);
 
 }
 
@@ -1104,6 +1171,7 @@ void Util::connect_db()
 				else
 					db = new Database(database_file_name);
 #endif
+				db->set_tango_utils(this);
 				connected = true;
 			}
 			catch (Tango::DevFailed &e)
@@ -1128,8 +1196,8 @@ void Util::connect_db()
 	}
 	else
 	{
-       		try
-        	{
+       	try
+        {
 #ifdef _TG_WINDOWS_
 			if (_service == true)
 				db = new Database(orb.in(),
@@ -1148,6 +1216,7 @@ void Util::connect_db()
 			else
 				db = new Database(database_file_name);
 #endif
+			db->set_tango_utils(this);
 		}
 		catch (Tango::DevFailed &e)
 		{
@@ -1169,20 +1238,26 @@ void Util::connect_db()
 		}
 		catch (CORBA::Exception &)
 		{
-                	TangoSys_OMemStream o;
+			TangoSys_OMemStream o;
 
 			o << "Can't build connection to TANGO database server, exiting";
-               		print_err_message(o.str());
+			print_err_message(o.str());
 		}
 	}
 	
 	if (CORBA::is_nil(db->get_dbase()) && _FileDb != true)
 	{
-                TangoSys_OMemStream o;
+		TangoSys_OMemStream o;
 
 		o << "Can't build connection to TANGO database server, exiting" << ends;
-                print_err_message(o.str());
+		print_err_message(o.str());
 	}
+
+//
+// Set a timeout on the database device
+//
+	
+	db->set_timeout_millis(DB_TIMEOUT);
 	
 //
 // Also copy this database object ptr into the ApiUtil object. Therefore,
@@ -1190,8 +1265,38 @@ void Util::connect_db()
 // object created within the server
 //
 
-	ApiUtil::instance()->get_db_vect().push_back(db);
+	ApiUtil *au = ApiUtil::instance();
+	au->get_db_vect().push_back(db);
+	au->in_server(true);
 	
+//
+// Try to create the db cache which will be used during the process
+// startup sequence
+// For servers with many devices and properties, this could take time
+// specially during a massive DS startup (after power cut for instance)
+// Filling the DS cache is a relatively heavy command for the DB server
+// Trying to minimize retry in this case could be a good idea.
+// Therefore, change DB device timeout to execute this command
+//
+
+	if (_FileDb == false)
+	{
+		string &inst_name = get_ds_inst_name();
+		if (inst_name != "-?")
+		{
+			db->set_timeout_millis(DB_TIMEOUT * 3);
+			try
+			{
+				ext->db_cache = new DbServerCache(db,get_ds_name(),get_host_name());
+			}
+			catch (Tango::DevFailed &e) {}
+			catch (...) 
+			{
+				cerr << "Unknown exception while trying to fill database cache" << endl;
+			}
+			db->set_timeout_millis(DB_TIMEOUT);
+		}
+	}
 }
 
 
@@ -1209,7 +1314,6 @@ void Util::reset_filedatabase()
 //			are needed later in the device server startup 
 //			sequence. These variables are :
 //				The process ID
-//				The host name
 //				The Tango version
 //
 //-----------------------------------------------------------------------------
@@ -1321,12 +1425,70 @@ void Util::misc_init()
 //
 
 #ifdef _TG_WINDOWS_
-	main_win_text = "TANGO collaboration (ALBA / ELETTRA / ESRF / SOLEIL)\n";
+	main_win_text = "TANGO collaboration (ALBA / DESY / ELETTRA / ESRF / SOLEIL )\n";
 	main_win_text = main_win_text + "CORBA based device server\n";
 	main_win_text = main_win_text + "Developped by Tango team";
 #endif
 }
 
+//+----------------------------------------------------------------------------
+//
+// method : 		Util::init_host_name()
+// 
+// description : 	This method initialises the process hst name which
+//			is needed later in the device server startup 
+//			sequence.
+//
+//-----------------------------------------------------------------------------
+
+void Util::init_host_name()
+{
+
+//
+// Get the FQDN host name (Fully qualified domain name)
+// If it is not returned by the system call "getname",
+// try with the gethostbyname system call
+//
+		
+	char buffer[80];
+	if (gethostname(buffer,80) == 0)
+	{
+		hostname = buffer;
+		string::size_type pos = hostname.find('.');
+
+		if (pos == string::npos)
+		{
+			struct hostent *he;
+			he = gethostbyname(buffer);
+			
+			if (he != NULL)
+			{
+				string na(he->h_name);
+				pos = na.find('.');
+				if (pos == string::npos)
+				{
+					char **p;
+					for (p = he->h_aliases;*p != 0;++p)
+					{
+						string al(*p);
+						pos = al.find('.');
+						if (pos != string::npos)
+						{
+							hostname = al;
+							break;
+						}					
+					}
+				}
+				else
+					hostname = na;
+			}
+		}
+	}
+	else
+	{
+		print_err_message("Cant retrieve server host name");
+	}
+}
 
 //+----------------------------------------------------------------------------
 //
@@ -1342,7 +1504,7 @@ void Util::create_event_supplier()
 	{		
 		try
 		{
-			ext->event_supplier = EventSupplier::create(orb,ds_name,db,hostname);
+			ext->event_supplier = EventSupplier::create(orb,ds_name,db,hostname,this);
 			ext->event_supplier->connect();
 		}
 		catch (...)
@@ -1439,16 +1601,16 @@ void Util::server_already_running()
 #ifdef _TG_WINDOWS_
 	DWORD backoff;
 	backoff = (DWORD)(1000. * (float)rand() / (float)RAND_MAX);
-	Sleep(backoff);
+//	Sleep(backoff);
 #else
 	struct timespec backoff;
 	backoff.tv_sec = 0;
 	backoff.tv_nsec = (long)(1000000000. * (float)rand() / (float)RAND_MAX);
-	nanosleep(&backoff,NULL);
+//	nanosleep(&backoff,NULL);
 #endif
 
 //
-// Build device name and try to import it from database
+// Build device name and try to import it from database or from cache if available
 //
 
 	string dev_name(DSDeviceDomain);
@@ -1459,19 +1621,27 @@ void Util::server_already_running()
 	
 	try
 	{
-		CORBA::Any send;
-		send <<= dev_name.c_str();
-										
-		CORBA::Any_var received = db->get_dbase()->command_inout("DbImportDevice",send);		
 		const Tango::DevVarLongStringArray *db_dev;
-		if ((received.inout() >>= db_dev) == false)
+		CORBA::Any_var received;
+		if (ext->db_cache != NULL)
 		{
-			TangoSys_OMemStream o;
-			o << "Database error while trying to import " << dev_name << ends;
+			db_dev = ext->db_cache->import_adm_dev();
+		}
+		else
+		{
+			CORBA::Any send;
+			send <<= dev_name.c_str();
+										
+			received = db->get_dbase()->command_inout("DbImportDevice",send);		
+			if ((received.inout() >>= db_dev) == false)
+			{
+				TangoSys_OMemStream o;
+				o << "Database error while trying to import " << dev_name << ends;
 					
-			Except::throw_exception((const char *)"API_DatabaseAccess",
+				Except::throw_exception((const char *)"API_DatabaseAccess",
 				                o.str(),
 				                (const char *)"Util::server_already_running");
+			}
 		}
 
 //
@@ -1485,26 +1655,11 @@ void Util::server_already_running()
 		}
 		
 		CORBA::Object_var obj = orb->string_to_object((db_dev->svalue)[1]);
-
-//
-// Set its timeout policy to 3 seconds
-//
-
-//	        CORBA::PolicyList policies;
-//	        policies.length(1);
-	
-//	        CORBA::ULong t = 2000;
-//	        CORBA::Any a;
-//	        a <<= t;
-
-//	        policies[0] = orb->create_policy(OB::TIMEOUT_POLICY_ID,a);
-//        	CORBA::Object_var pol_obj = obj->_set_policy_overrides(policies,CORBA::ADD_OVERRIDE);
 		
 //
 // Try to narrow the reference to a Tango::Device object
 //
 		
-//		dev = Tango::Device::_narrow(pol_obj);
 		dev = Tango::Device::_narrow(obj);
 	}
 	catch (Tango::DevFailed &)
@@ -1566,19 +1721,19 @@ void Util::server_already_running()
 // print_err_message method under windows
 //
 
-                throw;
-        }
-        catch (CORBA::NO_RESPONSE &)
-        {
-                try
-                {
-                        print_err_message("This server is already running but is blocked!");
-                }
-                catch (Tango::DevFailed &)
-                {
-                        throw;
-                }
-        }
+		throw;
+	}
+	catch (CORBA::NO_RESPONSE &)
+	{
+		try
+		{
+			print_err_message("This server is already running but is blocked!");
+		}
+		catch (Tango::DevFailed &)
+		{
+			throw;
+		}
+	}
 	catch (CORBA::SystemException &) {}
 	catch (CORBA::Exception &) {}
 	
@@ -1688,7 +1843,16 @@ void Util::server_init(bool with_window)
 #ifdef _TG_WINDOWS_
 	}
 #endif /* _TG_WINDOWS_ */
+
+//
+// Delete the db cache if it has been used
+//
 	
+	if (ext->db_cache != NULL)
+	{
+		delete ext->db_cache;
+		ext->db_cache = NULL;
+	}
 }
 
 //+----------------------------------------------------------------------------
@@ -1746,7 +1910,7 @@ void Util::server_run()
 		}			
 		else
 		{
-			cerr << "Ready to accept request" << endl;
+			cout << "Ready to accept request" << endl;
 
 			if (th_id == 0)
 			{
@@ -2501,7 +2665,12 @@ void Util::polling_configure()
 				if (throw_ex == true)
 				{
 					TangoSys_OMemStream o;
-					o << "Error when configuring polling for device " << v_poll_cmd[i]->svalue[0].in() << ends;
+					o << "Error when configuring polling for device " << v_poll_cmd[i]->svalue[0].in();
+					if (::strcmp(v_poll_cmd[i]->svalue[1].in(),"command") == 0)
+						o << ", cmd = ";
+					else
+						o << ", attr = ";
+					o << v_poll_cmd[i]->svalue[2].in() << ends;
 					Except::re_throw_exception(e,(const char *)"API_BadConfigurationProperty",
 								o.str(),
 								(const char *)"Util::polling_configure");
@@ -2613,10 +2782,10 @@ void Util::trigger_attr_polling(Tango::DeviceImpl *dev,string &name)
 // Wait for thread to execute command
 //
 
-		TangoMonitor &dev_mon = dev->get_dev_monitor();
+/*		TangoMonitor &dev_mon = dev->get_dev_monitor();
 		omni_thread *th = omni_thread::self();
 		
-/*		while (shared_cmd.trigger == true)
+		while (shared_cmd.trigger == true)
 		{
 
 //
@@ -2748,10 +2917,10 @@ void Util::trigger_cmd_polling(Tango::DeviceImpl *dev,string &name)
 // Wait for thread to execute command
 //
 
-		TangoMonitor &dev_mon = dev->get_dev_monitor();
+/*		TangoMonitor &dev_mon = dev->get_dev_monitor();
 		omni_thread *th = omni_thread::self();
 		
-/*		while (shared_cmd.trigger == true)
+		while (shared_cmd.trigger == true)
 		{
 		
 //
@@ -2815,6 +2984,111 @@ long Util::get_tango_lib_release()
 	return _convert_tango_lib_release(TgLibVers);
 }
 
+//+----------------------------------------------------------------------------
+//
+// method : 		Util::clean_attr_polled_prop()
+// 
+// description : 	Clean in database the prop used to memorized which attribute
+//					are polled	
+//
+//-----------------------------------------------------------------------------
+
+void Util::clean_attr_polled_prop()
+{
+	if (Tango::Util::_UseDb == true)
+	{
+		DbData send_data;
+		DbDatum db_info("polled_attr");
+	
+		for (unsigned int loop = 0;loop < ext->polled_dyn_attr_names.size();loop++)
+		{		
+			vector<string>::iterator ite_attr = find(ext->polled_att_list.begin(),ext->polled_att_list.end(), ext->polled_dyn_attr_names[loop]);	
+			if (ite_attr != ext->polled_att_list.end())
+			{
+				ite_attr = ext->polled_att_list.erase(ite_attr);
+				ext->polled_att_list.erase(ite_attr);
+			}
+			else
+			{
+				TangoSys_OMemStream o;
+
+				o << "Polling properties for attribute " << ext->polled_dyn_attr_names[loop] << " on device " << ext->dyn_att_dev_name;
+				o << " not found device in polled attribute list!" << ends;		
+
+				Except::throw_exception((const char *)"API_MethodArgument",o.str(),
+										(const char *)"Util::clean_attr_polling_prop");
+			}
+		}		
+
+		db_info << ext->polled_att_list;
+		send_data.push_back(db_info);
+		
+		if (db_info.size() == 0)
+			db->delete_device_property(ext->dyn_att_dev_name,send_data);
+		else
+			db->put_device_property(ext->dyn_att_dev_name,send_data);
+	}	
+}
+
+
+//+----------------------------------------------------------------------------
+//
+// method : 		Util::clean_dyn_attr_prop()
+// 
+// description : 	Clean in database the dynamic attribute property(ies)
+//
+//-----------------------------------------------------------------------------
+
+void Util::clean_dyn_attr_prop()
+{
+	if (Tango::Util::_UseDb == true)
+	{
+		DbData send_data;
+	
+		for (unsigned long loop = 0;loop < ext->all_dyn_attr.size();loop++)
+		{
+			DbDatum db_dat(ext->all_dyn_attr[loop]);
+			send_data.push_back(db_dat);
+		}
+		
+		db->delete_all_device_attribute_property(ext->dyn_att_dev_name,send_data);
+	}	
+}
+
+//+----------------------------------------------------------------------------
+//
+// method : 		Util::set_db_svr_version()
+// 
+// description : 	Cheack some BD command to guess db server version
+//
+//-----------------------------------------------------------------------------
+
+void Util::set_db_svr_version()
+{
+
+//
+// Is the DbDeleteAllDeviceAttributeProperty command available ?
+//
+
+	try
+	{
+		string dummy_dev_name("a/b/c");
+		DbData db_dat;
+		DbDatum datum("Dummy_att");
+		db_dat.push_back(datum);
+		
+		db->delete_all_device_attribute_property(dummy_dev_name,db_dat);
+		ext->db_svr_version = 400;
+	}
+	catch(Tango::DevFailed &e)
+	{
+		if (::strcmp(e.errors[0].reason.in(),"API_CommandNotFound") == 0)
+			ext->db_svr_version = 399;
+		else
+			ext->db_svr_version = 300;
+	}
+}
+
 
 #ifdef _TG_WINDOWS_
 //+----------------------------------------------------------------------------
@@ -3022,11 +3296,27 @@ void create_PyPerThData(omni::omniInterceptors::createThread_T::info_T &info)
 
 	omni_thread::self()->set_value(key_py_data,py_dat_ptr);
 
+	Util *tg = NULL;
+	Interceptors *Inter = NULL;
+	
+	try
+	{
+		tg = Util::instance(false);
+		Inter = tg->get_interceptors();
+	}
+	catch(Tango::DevFailed &) {}
+	
+	if (Inter != NULL)
+		Inter->create_thread();
+
 	info.run();
 	
 	omni_thread::self()->remove_value(key_py_data);
 	delete py_dat_ptr;
 
+	if (Inter != NULL)
+		Inter->delete_thread();
+	
 	return;
 }
 
