@@ -8,9 +8,39 @@
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.26 $
+// $Revision: 3.33 $
 //
 // $Log: utils.h,v $
+// Revision 3.33  2008/03/25 13:32:26  taurel
+// - Change when the RootPOA CORBA object is created. This allows DS to
+// start on windows in daemon mode even if the PC is not connected to the
+// network at boot time
+//
+// Revision 3.32  2008/02/29 12:53:08  taurel
+// - Increase timeout for DB get data cache command
+// - Add a method to unvalidate the cache
+//
+// Revision 3.31  2008/01/25 15:44:51  taurel
+// - Some changes in the Db cache
+// - A lighter system to shutdown DS in case of dynamic attribute
+//
+// Revision 3.30  2007/12/12 10:17:19  taurel
+// - Db calls during DS startup has a separate timeout and some retries
+//
+// Revision 3.29  2007/11/08 12:03:45  taurel
+// - Start implementing user interceptors
+// - Fix bug in poll thread pproperty management when removing polling object
+// - Set a database timeout to 6 sec
+//
+// Revision 3.28  2007/10/16 08:24:14  taurel
+// - Add management of the TC connection establishment timeout for DB access
+// - Add DB server cache in DS used during DS startup sequence
+// - Comment out the sleep time during DS startup sequence
+//
+// Revision 3.27  2007/07/03 08:28:22  jensmeyer
+// Corrected handling of CORBA ANY in fill_attr_polling_buffer().
+// THe beahaviour of the any changed with omniORB4.1!
+//
 // Revision 3.26  2007/05/17 07:59:07  taurel
 // - The polling is not configured via a separate thread any more. The polling thread add_obj_polling method has been modified to support a parameter telling to the polling thread when it has to polled the object.
 // Add device name in monitor print message
@@ -52,192 +82,6 @@
 // - It is now possiblt to fill the polling buffer even for attribute which are
 // not read only
 //
-// Revision 3.15  2006/03/03 10:51:56  taurel
-// - Added a new python debugger flag
-//
-// Revision 3.14  2006/02/17 16:48:18  jensmeyer
-// Correction for porting to VC7 under windows.
-//
-// Revision 3.13  2006/01/20 08:41:32  taurel
-// - Only change in indentention...
-//
-// Revision 3.12  2006/01/20 08:33:28  taurel
-// - Added necessary changes to support Device server written in Python
-//
-// Revision 3.11  2005/12/16 17:41:12  jensmeyer
-// Added a check fro ATTR_WARNING quality.
-//
-// Revision 3.10  2005/06/29 08:31:19  taurel
-// - Last commit before release 5.2 ?
-//
-// Revision 3.9  2005/01/21 19:58:30  taurel
-// - Some changes in include files for gcc 3.4.2
-//
-// Revision 3.8  2005/01/13 09:27:53  taurel
-// Fix some bugs :
-// - R/W attribute : W value not returned when read if set by set_write_value
-// - Core dumped when retrieving attribute polling history for Device_2Impl device which
-//   has stored an exception
-// - Remove device_name in lib default attribute label property
-// - Lib default value for label not store in db any more
-// - Size of the DaData used by the Database::get_device_attribute_property() and
-//   Database::get_class_attribute_property()
-// - R/W attribute: W part not returned when read for Device_2Impl device
-// Some changes :
-// - Improvement of the -file option error management (now throw exception in case of
-//   error)
-// - Reset "string" attribute property to the default value (lib or user) when new
-//   value is an empty string
-//
-// Revision 3.6.2.11  2004/12/06 14:39:30  taurel
-// - Polling starts in a separate thread
-// - For Windows : Polling thread cmd/attr measurement used PerformanceCounter
-// - Fix bug in pollext.h in methods to externally fill polling buffer
-//
-// Revision 3.6.2.10  2004/11/22 14:14:46  taurel
-// - Added methods for the AttrDesc class for the new attribute config
-//   parameters.
-// - Some minor changes
-//
-// Revision 3.6.2.9  2004/11/10 13:00:57  taurel
-// - Some minor changes for the WIN32 port
-//
-// Revision 3.6.2.8  2004/11/04 09:46:40  taurel
-// - Add a tuning method in the polling thread
-// - Some minor fixes to pass test suite
-//
-// Revision 3.6.2.7  2004/10/27 05:59:47  taurel
-// - Some minor changes to compile on all our supported platforms
-//
-// Revision 3.6.2.6  2004/10/05 13:55:03  maxchiandone
-// First upload for filedatabase.
-//
-// Revision 3.6.2.5  2004/09/27 09:10:06  taurel
-// - Changes to allow reading state and/or status as attributes
-//
-// Revision 3.6.2.4  2004/09/15 06:47:17  taurel
-// - Added four new types for attributes (boolean, float, unsigned short and unsigned char)
-// - It is also possible to read state and status as attributes
-// - Fix bug in Database::get_class_property() method (missing ends insertion)
-// - Fix bug in admin device DevRestart command (device name case problem)
-//
-// Revision 3.6.2.3  2004/08/26 07:34:45  taurel
-// - Implement a way to directly fills command or attribute polling buffer
-//
-// Revision 3.6.2.2  2004/08/19 07:44:59  taurel
-// - Replace server low level database access call by Database class method call
-// - Split device monitor in 3 : 1 to protect harware access, 1 to protect cache access and one mutex for device black box
-//
-// Revision 3.6.2.1  2004/07/15 15:04:06  taurel
-// - Added the way to externally filled the polling buffer for attribute
-//   (Command will come soon)
-//
-// Revision 3.6  2004/07/07 08:40:13  taurel
-//
-// - Fisrt commit after merge between Trunk and release 4 branch
-// - Add EventData copy ctor, asiignement operator and dtor
-// - Add Database and DeviceProxy::get_alias() method
-// - Add AttributeProxy ctor from "device_alias/attribute_name"
-// - Exception thrown when subscribing two times for exactly yhe same event
-//
-// Revision 3.5  2004/06/15 12:38:22  taurel
-// - Last commit before merge with release 4 branch
-//
-// Revision 3.4  2003/08/21 07:24:37  taurel
-// - End of the implementation of the new way to transfer data for read and
-//   write attributes (better use of exception)
-// - Added Attribute::set_date() and Attribute::set_value_date_quality() methods
-// - Added DeviceAttribute ctors from "const char *"
-// - Enable writing of spectrum and image attributes
-// - Many new DeviceAttribute ctors/inserters to enable easy image and spectrums
-//   attribute writing
-// - Attribute date automatically set in case of attribute quality factor set to INVALID
-// - Change in the polling thread discarding element algo. to support case of polling
-//   several cmd/atts at the same polling period with cmd/attr having a long response time
-// - Take cmd/attr execution time into account in the "Data not updated since" polling
-//   status string
-// - Split "str().c_str()" code in two lines of code. It was the reason of some problem
-//   on Windows device server
-// - Add the possibility to set a cmd/attr polling as "externally triggered". Add method
-//   to send trigger to the polling thread
-//
-// Revision 3.3  2003/07/03 07:40:51  taurel
-// - Change in Tango IDL file : Implement a new way to tranfer data for read_attribute and write_attribute CORBA operation
-// - Handle this new IDL release in DeviceProxy class
-// - New exception methods in DeviceAttribute class
-// - New way to get data out of DeviceAttribute object
-// - Fix bugs in DeviceProxy copy constructor and assignement operator
-// - Change some method names in DeviceDataHistory and DeviceAttributeHistory classes
-// - Change the implementation of the DeviceProxy::write_attribute() method to avoid DeviceAttribute copying
-// - Clean-up how a server is killed via a CTRL-C or a dserver device kill command
-// - Add a server_cleanup() method in the Util class
-// - Win32 : Update debug menu in the server graphical window to support logging feature
-// - Win32 : Display library CVS tag in the "Help->About" sub-window
-//
-// Revision 3.2.2.7  2004/04/07 11:22:10  taurel
-// - Add some import/export declaration for Windows DLL
-// - Add test on minor code for the CORBA::IMP_LIMIT exception before
-//   printing it
-//
-// Revision 3.2.2.6  2004/02/27 16:47:01  taurel
-// - Updated DeviceImpl and Util classes sizes for compatibility
-//
-// Revision 3.2.2.5  2004/01/20 08:32:38  taurel
-// -First commit after merge with the event branch and work on the AttributeProxy class
-// - Fix bug in the stream "clear()" method usage when used with gcc 3.3
-//
-// Revision 3.2.2.4  2003/11/28 17:08:44  taurel
-// Changes to handle tango lib build as a Windows DLL and used in a device
-// server
-//
-// Revision 3.2.2.2  2003/10/03 13:34:26  taurel
-// - Fix bug for device server started without database, with device name specified on command line using mix of upper and lower cases
-// - It's now possible to send a command to the polling thread from itself
-// - Add a release lock if the insert into the polling buffer failed
-//
-// Revision 3.2.2.1  2003/09/30 11:49:25  taurel
-// Add some changes foreseen for release 4.1 and already implemented on
-// the trunck into this release 4.0 branch
-//
-// Revision 3.2  2003/05/16 08:46:16  taurel
-// Many changes for release 3.0.1. The most important ones are :
-// - Timeout are backs
-// - Multiple db servers (change in TANGO_HOST syntax)
-// - Added methods to print DeviceData, DeviceDataHistory, DeviceAttribute and DeviceAttributeHistory instances
-// - Attributes name stored in blackbox
-// - Remove check if a class is created without any device
-// - It's now possible to create a DeviceProxy from its alias name
-// - Command, attribute names are case insensitive
-// - Change parameters of some DeviceProxy logging methods
-// - Change parameters of DeviceProxy asynchronous replies calls
-// - New serialization model in device server (no serialization model)
-// - Win32 (2000) device server service does not exit at loggoff anymore
-// - Miscellaneous bug fixes
-//
-// Revision 3.1  2003/04/02 12:26:04  taurel
-// Miscellaneous changes to :
-//  - Modify TANGO_HOST env. variable syntax to be able to have several db servers
-//  - Asynchronous calls are now done on device_2 or device depending on the device IDL version
-//  - Timeout are bcks (omniORB 4.0.1)
-//  - Fix bug when killing a device server via the kill command of the admin device
-//    (Solaris only)
-//  - Cleanup device server code to have only one ORB and one db connection within a server
-// Revision 3.0.2.1  2003/04/08 13:12:54  andy_gotz
-// first version of TANGO events
-//
-// Revision 3.0  2003/03/25 16:48:02  taurel
-// Many changes for Tango release 3.0 including
-// - Added full logging features
-// - Added asynchronous calls
-// - Host name of clients now stored in black-box
-// - Three serialization model in DS
-// - Fix miscellaneous bugs
-// - Ported to gcc 3.2
-// - Added ApiUtil::cleanup() and destructor methods
-// - Some internal cleanups
-// - Change the way how TangoMonitor class is implemented. It's a recursive
-//   mutex
-//
 //
 // copyleft :           European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
@@ -292,6 +136,7 @@ class Util;
 class EventSupplier;
 class PyLock;
 class CreatePyLock;
+class DbServerCache;
 
 #ifdef _TG_WINDOWS_
 class CoutBuf;
@@ -318,6 +163,15 @@ public:
 	virtual PyLock *create() {return new PyLock();}
 };
 
+class Interceptors
+{
+public:
+	Interceptors() {}
+	virtual ~Interceptors() {}
+	
+	virtual void	create_thread() {}
+	virtual	void	delete_thread() {}
+};
 
 //=============================================================================
 //
@@ -339,25 +193,38 @@ class UtilExt
 {
 public:
 	UtilExt():poll_mon("utils_poll"),ser_model(BY_DEVICE),only_one("process"),
-	 	  py_interp(NULL),py_ds(false),py_dbg(false)
+	 	  py_interp(NULL),py_ds(false),py_dbg(false),db_cache(NULL),inter(NULL),
+		  svr_starting(true),svr_stopping(false),db_svr_version(0)
 	{shared_data.cmd_pending=false;shared_data.trigger=false;
 	cr_py_lock = new CreatePyLock();}
 	
-	vector<string>		cmd_line_name_list;
+	vector<string>	cmd_line_name_list;
 	
 	PollThread		*poll_th;		// The polling thread object
-	int			poll_th_id;		// The polling thread identifier
-	PollThCmd		shared_data;		// The shared buffer
-	TangoMonitor		poll_mon;		// The monitor
+	int				poll_th_id;		// The polling thread identifier
+	PollThCmd		shared_data;	// The shared buffer
+	TangoMonitor	poll_mon;		// The monitor
 	bool			poll_on;		// Polling on flag
 	SerialModel		ser_model;		// The serialization model
-	TangoMonitor		only_one;		// Serialization monitor
-	EventSupplier		*event_supplier;	// The event supplier object
+	TangoMonitor	only_one;		// Serialization monitor
+	EventSupplier	*event_supplier;	// The event supplier object
 
 	void			*py_interp;		// The Python interpreter
 	bool			py_ds;			// The Python DS flag
-	CreatePyLock		*cr_py_lock;		// The python lock creator pointer
+	CreatePyLock	*cr_py_lock;	// The python lock creator pointer
 	bool			py_dbg;			// Badly written Python dbg flag
+	
+	DbServerCache	*db_cache;		// The db cache
+	Interceptors	*inter;			// The user interceptors
+	
+	bool			svr_starting;	// Server is starting flag
+	bool			svr_stopping;	// Server is shutting down flag
+	
+	vector<string>	polled_dyn_attr_names;	// Dynamic att. names (used for polling clean-up)
+	vector<string>	polled_att_list;		// Full polled att list
+	vector<string>	all_dyn_attr;			// All dynamic attr name list
+	string			dyn_att_dev_name;		// Device name (use for dyn att clean-up)
+	int				db_svr_version;			// Db server version;
 };
 
 /**
@@ -367,7 +234,7 @@ public:
  * one instance of this class and its constructor is not public.
  *
  * $Author: taurel $
- * $Revision: 3.26 $
+ * $Revision: 3.33 $
  */
  
 class Util
@@ -604,7 +471,7 @@ public:
 /**
  * Set the serialization model
  *
- * @param The new serialazation model. The serialization model must be one
+ * @param The new serialization model. The serialization model must be one
  * of BY_DEVICE, BY_CLASS, BY_PROCESS or NO_SYNC
  */	
 	void set_serial_model(SerialModel ser) {ext->ser_model = ser;}
@@ -622,6 +489,21 @@ public:
  * @return The EventSupplier object
  */
 	EventSupplier *get_event_supplier() {return ext->event_supplier;}
+	
+/**
+ * Set the interceptors
+ * 
+ * @param A pointer to a class inheriting from the Tango Interceptors class
+ * This class has virtual methods allowing user to define its own interceptors
+ */	
+	void set_interceptors(Interceptors *in) {ext->inter = in;}
+	
+/**
+ * Get the interceptors
+ * 
+ * @retrun The pointer to the interceptors object
+ */	
+	Interceptors *get_interceptors() {return ext->inter;}
 //@}
 
 	vector<string> &get_cmd_line_name_list() {return ext->cmd_line_name_list;}
@@ -647,6 +529,26 @@ public:
 
 	void set_py_lock_creator(CreatePyLock *py) {ext->cr_py_lock = py;}
 	CreatePyLock *get_py_lock_creator() {return ext->cr_py_lock;}
+	
+	DbServerCache *get_db_cache() {return ext->db_cache;}
+	void unvalidate_db_cache() {if (ext->db_cache!=NULL){delete ext->db_cache;ext->db_cache = NULL;}}
+	
+	bool is_svr_starting() {return ext->svr_starting;}
+	void set_svr_starting(bool val) {ext->svr_starting = val;}
+	
+	bool is_svr_shutting_down() {return ext->svr_stopping;}
+	void set_svr_shutting_down(bool val) {ext->svr_stopping = val;}
+	
+	vector<string> &get_polled_dyn_attr_names() {return ext->polled_dyn_attr_names;}
+	vector<string> &get_full_polled_att_list() {return ext->polled_att_list;}
+	string &get_dyn_att_dev_name() {return ext->dyn_att_dev_name;}
+	vector<string> &get_all_dyn_attr_names() {return ext->all_dyn_attr;}
+	
+	int get_db_svr_version() {return ext->db_svr_version;}
+	void set_db_svr_version();
+	
+	void clean_attr_polled_prop();
+	void clean_dyn_attr_prop();
 			
 /**@name Database related methods */
 //@{
@@ -885,20 +787,20 @@ protected:
 #endif
 		
 private:
-	TANGO_IMP static Util		*_instance;
-	static bool		_constructed;
+	TANGO_IMP static Util	*_instance;
+	static bool				_constructed;
 #ifdef _TG_WINDOWS_
-        static bool             _win;
-        int                     argc;
-        char                    **argv;
-	int			nCmd;
-	CoutBuf			*pcb;
-	W32Win			*ds_window;
-	string			main_win_text;
-	bool			go;
-	TangoMonitor		mon;
+	static bool				_win;
+	int						argc;
+	char					**argv;
+	int						nCmd;
+	CoutBuf					*pcb;
+	W32Win					*ds_window;
+	string					main_win_text;
+	bool					go;
+	TangoMonitor			mon;
 
-        void                    build_argc_argv();
+	void build_argc_argv();
 	
 	class ORBWin32Loop: public omni_thread	
 	{
@@ -916,21 +818,21 @@ private:
 	ORBWin32Loop *loop_th;
 #endif
 
-	CORBA::ORB_var 		orb;
+	CORBA::ORB_var 			orb;
 	PortableServer::POA_var _poa;
 	
-	string			ds_instance_name;	// The instance name
-	string			ds_exec_name;		// The server exec. name
-	string			ds_name;		// The server name
+	string					ds_instance_name;	// The instance name
+	string					ds_exec_name;		// The server exec. name
+	string					ds_name;			// The server name
 	
-	string			hostname;		// The host name
-	string			pid_str;		// The process PID (as string)
-	TangoSys_Pid		pid;			// The process PID
+	string					hostname;			// The host name
+	string					pid_str;			// The process PID (as string)
+	TangoSys_Pid			pid;				// The process PID
 
-	string			version_str;		// Tango version
-	string			server_version;		// Device server version
+	string					version_str;		// Tango version
+	string					server_version;		// Device server version
 
-	string      database_file_name;
+	string      			database_file_name;
 
 #ifndef TANGO_HAS_LOG4TANGO	
 	string			trace_output;
@@ -938,25 +840,27 @@ private:
 	ofstream		*file_stream;
 #endif //TANGO_HAS_LOG4TANGO
 
-	Database		*db;			// The db proxy
+	Database				*db;			// The db proxy
 	
-        void effective_job(int,char *[]);
+	void effective_job(int,char *[]);
+	void create_CORBA_objects();
 	void misc_init();
+	void init_host_name();
 	void server_already_running();	
 	void print_usage(char *);
-        static void print_err_message(const char *,Tango::MessBoxType type = Tango::STOP);
-        void print_err_message(const string &mess,Tango::MessBoxType type = Tango::STOP)
-        {
-                Util::print_err_message(mess.c_str(),type);
-        }
+	static void print_err_message(const char *,Tango::MessBoxType type = Tango::STOP);
+	void print_err_message(const string &mess,Tango::MessBoxType type = Tango::STOP)
+	{
+		Util::print_err_message(mess.c_str(),type);
+	}
 	void check_args(int, char *[]);
 	void display_help_message();
 	DeviceImpl *find_device_name_core(string &);
 
-	bool  display_help;				// display help message flag
-	const vector<DeviceClass *>	*cl_list_ptr;	// Ptr to server device class list	
-	UtilExt				*ext;		// Class extension
-	vector<DeviceClass *>		cl_list;	// Full class list ptr
+	bool  							display_help;	// display help message flag
+	const vector<DeviceClass *>		*cl_list_ptr;	// Ptr to server device class list	
+	UtilExt							*ext;			// Class extension
+	vector<DeviceClass *>			cl_list;		// Full class list ptr
 
 public:
 
@@ -1170,36 +1074,100 @@ public:
 					switch (att.get_data_type())
 					{
 					case Tango::DEV_SHORT :
-						a <<= att.get_short_value();
+						{
+						Tango::DevVarShortArray *ptr = att.get_short_value();
+						a <<= *ptr;
+						delete ptr;
 						break;
+						}
 
 					case Tango::DEV_LONG :
-						a <<= att.get_long_value();
+						{
+						Tango::DevVarLongArray *ptr = att.get_long_value();
+						a <<= *ptr;
+						delete ptr;
 						break;
+						}
+
+					case Tango::DEV_LONG64 :
+						{
+						Tango::DevVarLong64Array *ptr = att.get_long64_value();
+						a <<= *ptr;
+						delete ptr;
+						break;
+						}
 
 					case Tango::DEV_DOUBLE :
-						a <<= att.get_double_value();
+						{
+						Tango::DevVarDoubleArray *ptr = att.get_double_value();
+						a <<= *ptr;
+						delete ptr;
 						break;
-
+						}
+				
 					case Tango::DEV_STRING :
-						a <<= att.get_string_value();
+						{
+						Tango::DevVarStringArray *ptr = att.get_string_value();
+						a <<= *ptr;
+						delete ptr;
 						break;
-						
+						}
+							
 					case Tango::DEV_FLOAT :
-						a <<= att.get_float_value();
+						{
+						Tango::DevVarFloatArray *ptr = att.get_float_value();
+						a <<= *ptr;
+						delete ptr;
 						break;
-
+						}
+				
 					case Tango::DEV_BOOLEAN :
-						a <<= att.get_boolean_value();
+						{
+						Tango::DevVarBooleanArray *ptr = att.get_boolean_value();
+						a <<= *ptr;
+						delete ptr;
 						break;
-
+						}
+				
 					case Tango::DEV_USHORT :
-						a <<= att.get_ushort_value();
+						{
+						Tango::DevVarUShortArray *ptr = att.get_ushort_value();
+						a <<= *ptr;
+						delete ptr;
 						break;
-
+						}
+				
 					case Tango::DEV_UCHAR :
-						a <<= att.get_uchar_value();
+						{
+						Tango::DevVarUCharArray *ptr = att.get_uchar_value();
+						a <<= *ptr;
+						delete ptr;
 						break;
+						}
+							
+					case Tango::DEV_ULONG :
+						{
+						Tango::DevVarULongArray *ptr = att.get_ulong_value();
+						a <<= *ptr;
+						delete ptr;
+						break;
+						}
+							
+					case Tango::DEV_ULONG64 :
+						{
+						Tango::DevVarULong64Array *ptr = att.get_ulong64_value();
+						a <<= *ptr;
+						delete ptr;
+						break;
+						}
+							
+					case Tango::DEV_STATE :
+						{
+						Tango::DevVarStateArray *ptr = att.get_state_value();
+						a <<= *ptr;
+						delete ptr;
+						break;
+						}
 					}
 					(*back)[0].r_dim.dim_x = (data.get_data())[i].x;
 					(*back)[0].r_dim.dim_y = (data.get_data())[i].y;				
@@ -1212,7 +1180,7 @@ public:
 
 			try
 			{
-				dev->get_poll_monitor().get_monitor();		
+				dev->get_poll_monitor().get_monitor();
 				vector<PollObj *>::iterator ite = dev->get_polled_obj_by_type_name(Tango::POLL_ATTR,obj_name);
 
 				if (attr_failed == false)
@@ -1509,21 +1477,21 @@ public:
 			delete PerTh_py_lock;
 	}
 
-	DevVarCharArray			PerTh_dvca;
-	DevVarShortArray		PerTh_dvsha;
-	DevVarLongArray			PerTh_dvla;
-	DevVarFloatArray		PerTh_dvfa;
-	DevVarDoubleArray		PerTh_dvda;
-	DevVarUShortArray		PerTh_dvusa;
-	DevVarULongArray		PerTh_dvula;
-	DevVarStringArray 		PerTh_dvsa;
+	DevVarCharArray				PerTh_dvca;
+	DevVarShortArray			PerTh_dvsha;
+	DevVarLongArray				PerTh_dvla;
+	DevVarFloatArray			PerTh_dvfa;
+	DevVarDoubleArray			PerTh_dvda;
+	DevVarUShortArray			PerTh_dvusa;
+	DevVarULongArray			PerTh_dvula;
+	DevVarStringArray 			PerTh_dvsa;
 	DevVarLongStringArray		PerTh_dvlsa;
 	DevVarDoubleStringArray		PerTh_dvdsa;
-	DevVarLong64Array		PerTh_dvl64a;
-	DevVarULong64Array		PerTh_dvul64a;
+	DevVarLong64Array			PerTh_dvl64a;
+	DevVarULong64Array			PerTh_dvul64a;
 	
-	string					PerTh_string;
-	DevFailed				PerTh_df;
+	string						PerTh_string;
+	DevFailed					PerTh_df;
 	vector<string>				PerTh_vec_str;
 	vector<double>				PerTh_vec_db;
 	DevErrorList				PerTh_del;
