@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: w_attribute.cpp,v 1.17 2007/04/20 14:41:35 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: w_attribute.cpp,v 1.21 2008/03/14 11:56:04 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -14,9 +14,21 @@ static const char *RcsId = "$Id: w_attribute.cpp,v 1.17 2007/04/20 14:41:35 taur
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 1.17 $
+// $Revision: 1.21 $
 //
 // $Log: w_attribute.cpp,v $
+// Revision 1.21  2008/03/14 11:56:04  taurel
+// - Fix some gcc 4.2 warnings
+//
+// Revision 1.20  2008/02/06 15:48:44  jensmeyer
+// Changed the default value for the x dimension of the set value.
+//
+// Revision 1.19  2008/01/30 12:52:08  jensmeyer
+// Optimising set_write_value methods with input vector.
+//
+// Revision 1.18  2008/01/30 10:26:18  jensmeyer
+// Added set_write_value methods for spectrum and image attributes.
+//
 // Revision 1.17  2007/04/20 14:41:35  taurel
 // - Ported to Windows 64 bits x64 architecture
 //
@@ -151,6 +163,7 @@ WAttribute::WAttribute(vector<AttrProperty> &prop_list,
 //
 
 	w_ext = new WAttributeExt();
+
 	
 //
 // Init some data
@@ -197,7 +210,8 @@ WAttribute::WAttribute(vector<AttrProperty> &prop_list,
 	
 				
 	short_ptr = &short_val;
-	w_dim_x = w_dim_y = 0;
+	w_dim_x = 1;
+	w_dim_y = 0;
 	write_date.tv_sec = write_date.tv_usec = 0;
 
 //
@@ -1096,11 +1110,14 @@ long WAttribute::get_write_value_length()
 // method : 		WAttribute::set_write_value() methods
 // 
 // description : 	Set the attribute internal value.
-//			There are different methods according to 
-//			attribute data type
+//			There are different methods according to the 
+//			attribute data type and the attribute type (scalar,
+//       spectrum or image)
 //
 //--------------------------------------------------------------------------
 
+// DevShort:
+
 void WAttribute::set_write_value(Tango::DevShort val)
 {
 	CORBA::Any tmp_any;
@@ -1113,6 +1130,39 @@ void WAttribute::set_write_value(Tango::DevShort val)
 	copy_data(tmp_any);	
 }
 
+void WAttribute::set_write_value(Tango::DevShort *val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	long 			nb_data;
+	
+	if (y == 0)
+		nb_data = x;
+	else
+		nb_data = x * y;
+
+	Tango::DevVarShortArray *tmp_seq;
+	tmp_seq = new Tango::DevVarShortArray(nb_data, nb_data, val, false);
+	tmp_any <<= *tmp_seq;
+	delete tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);	
+}
+
+void WAttribute::set_write_value(vector<Tango::DevShort> &val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	Tango::DevVarShortArray tmp_seq;
+		
+	tmp_seq << val;
+	tmp_any <<= tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);	
+}
+
+// DevLong:
+
 void WAttribute::set_write_value(Tango::DevLong val)
 {
 	CORBA::Any tmp_any;
@@ -1125,6 +1175,39 @@ void WAttribute::set_write_value(Tango::DevLong val)
 	copy_data(tmp_any);
 }
 
+void WAttribute::set_write_value(Tango::DevLong *val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	long 			nb_data;
+	
+	if (y == 0)
+		nb_data = x;
+	else
+		nb_data = x * y;
+
+	Tango::DevVarLongArray *tmp_seq;
+	tmp_seq = new Tango::DevVarLongArray(nb_data, nb_data, val, false);
+	tmp_any <<= *tmp_seq;
+	delete tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);	
+}
+
+void WAttribute::set_write_value(vector<Tango::DevLong> &val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	Tango::DevVarLongArray tmp_seq;
+		
+	tmp_seq << val;
+	tmp_any <<= tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);		
+}
+
+// DevLong64:
+
 void WAttribute::set_write_value(Tango::DevLong64 val)
 {
 	CORBA::Any tmp_any;
@@ -1137,6 +1220,39 @@ void WAttribute::set_write_value(Tango::DevLong64 val)
 	copy_data(tmp_any);
 }
 
+void WAttribute::set_write_value(Tango::DevLong64 *val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	long 			nb_data;
+	
+	if (y == 0)
+		nb_data = x;
+	else
+		nb_data = x * y;
+
+	Tango::DevVarLong64Array *tmp_seq;
+	tmp_seq = new Tango::DevVarLong64Array(nb_data, nb_data, val, false);
+	tmp_any <<= *tmp_seq;
+	delete tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);	
+}
+
+void WAttribute::set_write_value(vector<Tango::DevLong64> &val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	Tango::DevVarLong64Array tmp_seq;
+		
+	tmp_seq << val;
+	tmp_any <<= tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);		
+}
+
+// DevDouble:
+
 void WAttribute::set_write_value(Tango::DevDouble val)
 {
 	CORBA::Any tmp_any;
@@ -1149,6 +1265,39 @@ void WAttribute::set_write_value(Tango::DevDouble val)
 	copy_data(tmp_any);
 }
 
+void WAttribute::set_write_value(Tango::DevDouble *val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	long 			nb_data;
+	
+	if (y == 0)
+		nb_data = x;
+	else
+		nb_data = x * y;
+
+	Tango::DevVarDoubleArray *tmp_seq;
+	tmp_seq = new Tango::DevVarDoubleArray(nb_data, nb_data, val, false);
+	tmp_any <<= *tmp_seq;
+	delete tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);	
+}
+
+void WAttribute::set_write_value(vector<Tango::DevDouble> &val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	Tango::DevVarDoubleArray tmp_seq;
+		
+	tmp_seq << val;
+	tmp_any <<= tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);	
+}
+
+// DevString:
+
 void WAttribute::set_write_value(Tango::DevString val)
 {
 	CORBA::Any tmp_any;
@@ -1173,6 +1322,39 @@ void WAttribute::set_write_value(string &val)
 	copy_data(tmp_any);
 }
 
+void WAttribute::set_write_value(Tango::DevString *val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	long 			nb_data;
+	
+	if (y == 0)
+		nb_data = x;
+	else
+		nb_data = x * y;
+
+	Tango::DevVarStringArray *tmp_seq;
+	tmp_seq = new Tango::DevVarStringArray(nb_data, nb_data, val, false);
+	tmp_any <<= *tmp_seq;
+	delete tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);	
+}
+
+void WAttribute::set_write_value(vector<string> &val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	Tango::DevVarStringArray tmp_seq;
+		
+	tmp_seq << val;
+	tmp_any <<= tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);	
+}
+
+// DevFloat:
+
 void WAttribute::set_write_value(Tango::DevFloat val)
 {
 	CORBA::Any tmp_any;
@@ -1185,6 +1367,39 @@ void WAttribute::set_write_value(Tango::DevFloat val)
 	copy_data(tmp_any);	
 }
 
+void WAttribute::set_write_value(Tango::DevFloat *val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	long 			nb_data;
+	
+	if (y == 0)
+		nb_data = x;
+	else
+		nb_data = x * y;
+
+	Tango::DevVarFloatArray *tmp_seq;
+	tmp_seq = new Tango::DevVarFloatArray(nb_data, nb_data, val, false);
+	tmp_any <<= *tmp_seq;
+	delete tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);	
+}
+
+void WAttribute::set_write_value(vector<Tango::DevFloat> &val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	Tango::DevVarFloatArray tmp_seq;
+		
+	tmp_seq << val;
+	tmp_any <<= tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);		
+}
+
+// DevBoolean:
+
 void WAttribute::set_write_value(Tango::DevBoolean val)
 {
 	CORBA::Any tmp_any;
@@ -1197,6 +1412,39 @@ void WAttribute::set_write_value(Tango::DevBoolean val)
 	copy_data(tmp_any);
 }
 
+void WAttribute::set_write_value(Tango::DevBoolean *val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	long 			nb_data;
+	
+	if (y == 0)
+		nb_data = x;
+	else
+		nb_data = x * y;
+
+	Tango::DevVarBooleanArray *tmp_seq;
+	tmp_seq = new Tango::DevVarBooleanArray(nb_data, nb_data, val, false);
+	tmp_any <<= *tmp_seq;
+	delete tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);	
+}
+
+void WAttribute::set_write_value(vector<Tango::DevBoolean> &val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	Tango::DevVarBooleanArray tmp_seq;
+		
+	tmp_seq << val;
+	tmp_any <<= tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);		
+}
+
+// DevUShort:
+
 void WAttribute::set_write_value(Tango::DevUShort val)
 {
 	CORBA::Any tmp_any;
@@ -1209,6 +1457,39 @@ void WAttribute::set_write_value(Tango::DevUShort val)
 	copy_data(tmp_any);
 }
 
+void WAttribute::set_write_value(Tango::DevUShort *val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	long 			nb_data;
+	
+	if (y == 0)
+		nb_data = x;
+	else
+		nb_data = x * y;
+
+	Tango::DevVarUShortArray *tmp_seq;
+	tmp_seq = new Tango::DevVarUShortArray(nb_data, nb_data, val, false);
+	tmp_any <<= *tmp_seq;
+	delete tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);	
+}
+
+void WAttribute::set_write_value(vector<Tango::DevUShort> &val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	Tango::DevVarUShortArray tmp_seq;
+		
+	tmp_seq << val;
+	tmp_any <<= tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);	
+}
+
+// DevUChar:
+
 void WAttribute::set_write_value(Tango::DevUChar val)
 {
 	CORBA::Any tmp_any;
@@ -1221,6 +1502,39 @@ void WAttribute::set_write_value(Tango::DevUChar val)
 	copy_data(tmp_any);
 }
 
+void WAttribute::set_write_value(Tango::DevUChar *val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	long 			nb_data;
+	
+	if (y == 0)
+		nb_data = x;
+	else
+		nb_data = x * y;
+
+	Tango::DevVarUCharArray *tmp_seq;
+	tmp_seq = new Tango::DevVarUCharArray(nb_data, nb_data, val, false);
+	tmp_any <<= *tmp_seq;
+	delete tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);	
+}
+
+void WAttribute::set_write_value(vector<Tango::DevUChar> &val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	Tango::DevVarUCharArray tmp_seq;
+		
+	tmp_seq << val;
+	tmp_any <<= tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);		
+}
+
+// DevULong:
+
 void WAttribute::set_write_value(Tango::DevULong val)
 {
 	CORBA::Any tmp_any;
@@ -1233,6 +1547,39 @@ void WAttribute::set_write_value(Tango::DevULong val)
 	copy_data(tmp_any);
 }
 
+void WAttribute::set_write_value(Tango::DevULong *val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	long 			nb_data;
+	
+	if (y == 0)
+		nb_data = x;
+	else
+		nb_data = x * y;
+
+	Tango::DevVarULongArray *tmp_seq;
+	tmp_seq = new Tango::DevVarULongArray(nb_data, nb_data, val, false);
+	tmp_any <<= *tmp_seq;
+	delete tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);	
+}
+
+void WAttribute::set_write_value(vector<Tango::DevULong> &val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	Tango::DevVarULongArray tmp_seq;
+		
+	tmp_seq << val;
+	tmp_any <<= tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);	
+}
+
+// DevULong64:
+
 void WAttribute::set_write_value(Tango::DevULong64 val)
 {
 	CORBA::Any tmp_any;
@@ -1245,6 +1592,39 @@ void WAttribute::set_write_value(Tango::DevULong64 val)
 	copy_data(tmp_any);
 }
 
+void WAttribute::set_write_value(Tango::DevULong64 *val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	long 			nb_data;
+	
+	if (y == 0)
+		nb_data = x;
+	else
+		nb_data = x * y;
+
+	Tango::DevVarULong64Array *tmp_seq;
+	tmp_seq = new Tango::DevVarULong64Array(nb_data, nb_data, val, false);
+	tmp_any <<= *tmp_seq;
+	delete tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);	
+}
+
+void WAttribute::set_write_value(vector<Tango::DevULong64> &val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	Tango::DevVarULong64Array tmp_seq;
+		
+	tmp_seq << val;
+	tmp_any <<= tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);		
+}
+
+// DevState:
+
 void WAttribute::set_write_value(Tango::DevState val)
 {
 	CORBA::Any tmp_any;
@@ -1257,6 +1637,38 @@ void WAttribute::set_write_value(Tango::DevState val)
 	copy_data(tmp_any);
 }
 
+void WAttribute::set_write_value(Tango::DevState *val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	long 			nb_data;
+	
+	if (y == 0)
+		nb_data = x;
+	else
+		nb_data = x * y;
+
+	Tango::DevVarStateArray *tmp_seq;
+	tmp_seq = new Tango::DevVarStateArray(nb_data, nb_data, val, false);
+	tmp_any <<= *tmp_seq;
+	delete tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);	
+}
+
+void WAttribute::set_write_value(vector<Tango::DevState> &val, long x, long y)
+{
+	CORBA::Any 	tmp_any;
+	Tango::DevVarStateArray tmp_seq;
+		
+	tmp_seq << val;
+	tmp_any <<= tmp_seq;
+	
+	set_write_value(tmp_any, x, y);
+	copy_data(tmp_any);	
+}
+
+
 //+-------------------------------------------------------------------------
 //
 // method : 		WAttribute::rollback
@@ -1339,7 +1751,7 @@ void WAttribute::copy_data(const CORBA::Any &any)
 	{
 	case Tango::DEV_SHORT :
 		const Tango::DevVarShortArray *sh_ptr;
-		any >>= sh_ptr;
+		any >>= sh_ptr;	
 		short_array_val = *sh_ptr;	
 		break;
 		
@@ -1629,7 +2041,7 @@ bool WAttribute::check_rds_alarm()
 			for (i = 0;i < nb_data;i++)
 			{
 				DevLong delta = w_ext->ulong_array_val[i] - (*value.ulg_seq)[i];
-				if (abs(delta) >= delta_val.ulg)
+				if ((unsigned int)abs(delta) >= delta_val.ulg)
 				{
 					quality = Tango::ATTR_ALARM;
 					alarm.set(rds);
@@ -1647,7 +2059,7 @@ bool WAttribute::check_rds_alarm()
 			{
 				DevLong64 delta = w_ext->ulong64_array_val[i] - (*value.ulg64_seq)[i];
 				
-				DevLong64 abs_delta;
+				DevULong64 abs_delta;
 				if (delta < 0)
 					abs_delta = -delta;
 				else
