@@ -8,9 +8,12 @@
 //
 // author(s) :		A.Gotz + E.Taurel
 //
-// $Revision: 1.8 $
+// $Revision: 1.9 $
 //
 // $Log: w_attribute.h,v $
+// Revision 1.9  2008/01/30 10:26:18  jensmeyer
+// Added set_write_value methods for spectrum and image attributes.
+//
 // Revision 1.8  2007/04/16 14:57:46  taurel
 // - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
 // - Ported to omniORB4.1
@@ -109,8 +112,8 @@ public:
  * This class represents a writable attribute. It inherits from the Attribute
  * class and only add what is specific to writable attribute.
  *
- * $Author: taurel $
- * $Revision: 1.8 $
+ * $Author: jensmeyer $
+ * $Revision: 1.9 $
  */
  
 class WAttribute:public Attribute
@@ -673,109 +676,336 @@ public:
  */
  //@{
 /**
- * Set the writable attribute value when attribute data type is 
+ * Set the writable scalar attribute value when the attribute data type is 
  * Tango::DevShort.
  *
  * @param val A reference to a Tango::DevShort data 
  */	
 	void set_write_value(Tango::DevShort val);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevShort.
+ *
+ * @param val A reference to the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ 
+ */	   
+	void set_write_value(Tango::DevShort *val, long x = 1, long y = 0);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevShort.
+ *
+ * @param val A vector containing the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ */		
+	void set_write_value(vector<Tango::DevShort> &val, long x = 1, long y = 0);
 
 /**
- * Set the writable attribute value when attribute data type is 
+ * Set the writable scalar attribute value when the attribute data type is 
  * Tango::DevLong.
  *
  * @param val A reference to a Tango::DevLong data 
  */
 	void set_write_value(Tango::DevLong val);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevLong.
+ *
+ * @param val A reference to the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ 
+ */	   
+	void set_write_value(Tango::DevLong *val, long x = 1, long y = 0);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevLong.
+ *
+ * @param val A vector containing the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ */
+	void set_write_value(vector<Tango::DevLong> &val, long x = 1, long y = 0);
 	
 /**
- * Set the writable attribute value when attribute data type is 
+ * Set the writable scalar attribute value when the attribute data type is 
  * Tango::DevLong64.
  *
  * @param val A reference to a Tango::DevLong64 data 
  */
 	void set_write_value(Tango::DevLong64 val);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevLong64.
+ *
+ * @param val A reference to the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ 
+ */	   
+	void set_write_value(Tango::DevLong64 *val, long x = 1, long y = 0);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevLong64.
+ *
+ * @param val A vector containing the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ */
+	void set_write_value(vector<Tango::DevLong64> &val, long x = 1, long y = 0);
 
 /**
- * Set the writable attribute value when attribute data type is 
+ * Set the writable scalar attribute value when the attribute data type is 
  * Tango::DevDouble.
  *
  * @param val A reference to a Tango::DevDouble
  */
 	void set_write_value(Tango::DevDouble val);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevDouble.
+ *
+ * @param val A reference to the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ 
+ */	   
+	void set_write_value(Tango::DevDouble *val, long x = 1, long y = 0);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevDouble.
+ *
+ * @param val A vector containing the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ */
+	void set_write_value(vector<Tango::DevDouble> &val, long x = 1, long y = 0);
 
 /**
- * Set the writable attribute value when attribute data type is 
+ * Set the writable scalar attribute value when the attribute data type is 
  * Tango::DevString.
  *
  * @param val A reference to a Tango::DevString 
  */
 	void set_write_value(Tango::DevString val);
-	
 /**
- * Set the writable attribute value when attribute data type is 
+ * Set the writable scalar attribute value when the attribute data type is 
  * Tango::DevString.
  *
  * @param val A reference to a std::string
  */
 	void set_write_value(string &val);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevString.
+ *
+ * @param val A reference to the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0 
+ */	   	
+	void set_write_value(Tango::DevString *val, long x = 1, long y = 0);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevString.
+ *
+ * @param val A vector of string containing the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ */
+	void set_write_value(vector<string> &val, long x = 1, long y = 0);
 	
 /**
- * Set the writable attribute value when attribute data type is 
+ * Set the writable scalar attribute value when the attribute data type is 
  * Tango::DevFloat.
  *
  * @param val A reference to a Tango::DevFloat
  */
 	void set_write_value(Tango::DevFloat val);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevFloat.
+ *
+ * @param val A reference to the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ 
+ */	   
+	void set_write_value(Tango::DevFloat *val, long x = 1, long y = 0);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevFloat.
+ *
+ * @param val A vector containing the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ */
+	void set_write_value(vector<Tango::DevFloat> &val, long x = 1, long y = 0);
 	
 /**
- * Set the writable attribute value when attribute data type is 
+ * Set the writable scalar attribute value when the attribute data type is 
  * Tango::DevBoolean.
  *
  * @param val A reference to a Tango::DevBoolean
  */
 	void set_write_value(Tango::DevBoolean val);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevBoolean.
+ *
+ * @param val A reference to the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ 
+ */	   
+	void set_write_value(Tango::DevBoolean *val, long x = 1, long y = 0);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevBoolean.
+ *
+ * @param val A vector containing the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ */
+	void set_write_value(vector<Tango::DevBoolean> &val, long x = 1, long y = 0);
 	
 /**
- * Set the writable attribute value when attribute data type is 
+ * Set the writable scalar attribute value when the attribute data type is 
  * Tango::DevUShort.
  *
  * @param val A reference to a Tango::DevUShort
  */
 	void set_write_value(Tango::DevUShort val);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevUShort.
+ *
+ * @param val A reference to the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ 
+ */	   
+	void set_write_value(Tango::DevUShort *val, long x = 1, long y = 0);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevUShort.
+ *
+ * @param val A vector containing the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ */
+	void set_write_value(vector<Tango::DevUShort> &val, long x = 1, long y = 0);
 	
 /**
- * Set the writable attribute value when attribute data type is 
+ * Set the writable scalar attribute value when the attribute data type is 
  * Tango::DevUChar.
  *
  * @param val A reference to a Tango::DevUChar
  */
 	void set_write_value(Tango::DevUChar val);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevUChar.
+ *
+ * @param val A reference to the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ 
+ */	   
+	void set_write_value(Tango::DevUChar *val, long x = 1, long y = 0);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevUChar.
+ *
+ * @param val A vector containing the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ */
+	void set_write_value(vector<Tango::DevUChar> &val, long x = 1, long y = 0);
 	
 /**
- * Set the writable attribute value when attribute data type is 
+ * Set the writable scalar attribute value when the attribute data type is 
  * Tango::DevULong.
  *
  * @param val A reference to a Tango::DevULong data 
  */
 	void set_write_value(Tango::DevULong val);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevULong.
+ *
+ * @param val A reference to the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ 
+ */	   
+	void set_write_value(Tango::DevULong *val, long x = 1, long y = 0);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevULong.
+ *
+ * @param val A vector containing the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ */
+	void set_write_value(vector<Tango::DevULong> &val, long x = 1, long y = 0);
 	
 /**
- * Set the writable attribute value when attribute data type is 
+ * Set the writable scalar attribute value when the attribute data type is 
  * Tango::DevULong64.
  *
  * @param val A reference to a Tango::DevULong64 data 
  */
 	void set_write_value(Tango::DevULong64 val);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevULong64.
+ *
+ * @param val A reference to the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ 
+ */	   
+	void set_write_value(Tango::DevULong64 *val, long x = 1, long y = 0);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevULong64.
+ *
+ * @param val A vector containing the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ */
+	void set_write_value(vector<Tango::DevULong64> &val, long x = 1, long y = 0);
 	
 /**
- * Set the writable attribute value when attribute data type is 
+ * Set the writable scalar attribute value when the attribute data type is 
  * Tango::DevState.
  *
  * @param val A reference to a Tango::DevState data 
  */
-	void set_write_value(Tango::DevState val);
+	void set_write_value(Tango::DevState val);	
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevState.
+ *
+ * @param val A reference to the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ */	   
+	void set_write_value(Tango::DevState *val, long x = 1, long y = 0);
+/**
+ * Set the writable spectrum or image attribute value when the attribute 
+ * data type is Tango::DevState.
+ *
+ * @param val A vector containing the attribute set value
+ * @param x The attribute set value x length. Default value is 1
+ * @param y The attribute set value y length. Default value is 0
+ */
+	void set_write_value(vector<Tango::DevState> &val, long x = 1, long y = 0);
 //@}	
+	
+	
 	virtual void set_rvalue();
 	
 	void rollback();	
